@@ -5,58 +5,27 @@ a structured document for pretty printing.
 
 module PrettySyntax
   ( PPInfo(..)
-  , prettyPrintSimple
-  , prettyPrintTokenId, prettyPrintId, simplePrintId -- , prettyPrintTraceId
+--  , prettyPrintSimple
+  , prettyPrintTokenId, prettyPrintId, simplePrintId
   , ppModule, ppTopDecls, ppClassCodes
   , ppType, ppContexts, ppSimple, ppDecl
   ) where 
 
-#if !defined(__HASKELL98__)
-#define isAlphaNum isAlphanum
-#endif
-
-import Extra(Pos(..),noPos)
+import Extra(noPos)
 import PrettyLib
-import Syntax hiding (noDecls)
+import Syntax hiding (noDecls,TokenId)
 import TokenId(TokenId(..),extractV,t_Arrow,t_List)
 import Id(Id)
 import IntState(IntState,strIS,lookupIS)
 import Nice(niceInt,niceNT,mkAL)
 import Info(Info(InfoData),tidI)
 import NT(NewType(NewType))
-import PackedString (unpackPS)
-import Char(isAlphaNum)
+import SysDeps (unpackPS,isAlphaNum)
 import Maybe(isJust,fromJust)
 import Flags(Flags,sShowWidth,sShowQualified,sShowIndent)
--- import TraceId(TraceId,hasInfo,arity,isLambdaBound,tokenId)
--- import AuxTypes (AuxiliaryInfo)	-- hbc's broken import mechanism needs this
 
 
 {-
-prettyPrintTraceId :: Flags -> (PPInfo TraceId -> a -> Doc) -> a -> String
-prettyPrintTraceId flags pp =
-  pretty (sShowWidth flags) . 
-  pp PPInfo{withPositions = False
-           ,indent = sShowIndent flags
-           ,id2str = id2strTraceId
-           ,tyVar2str = id2strTraceId -- show
-           ,isFunctionArrow = (\tr-> tokenId tr == t_Arrow)
-           ,isList = (\tr-> tokenId tr == t_List)
-           ,maybeTuple = maybeTupleTraceId}
-  where
-  id2strTraceId t = ( if sShowQualified flags
-                      then show (tokenId t)
-                      else (reverse . unpackPS . extractV) (tokenId t)
-                    ) ++
-                    ( if hasInfo t 
-                        then "{-"++ show (arity t) ++"/"++
-{-			     ( if isLambdaBound t then "lam-}" else "let-}" )
-                        else "")
-  maybeTupleTraceId t = case tokenId t of
-                          TupleId n -> Just n
-                          _         -> Nothing
--}
-
 prettyPrintSimple :: Int -> (PPInfo TokenId -> a -> Doc) -> a -> String
 prettyPrintSimple width pp =
   pretty width . 
@@ -64,6 +33,7 @@ prettyPrintSimple width pp =
            ,indent = 4
            ,id2str = show
            ,tyVar2str = show
+           ,intState = error "prettyPrintTokenId: no intState"
            ,isFunctionArrow = (== t_Arrow)
            ,isList = (== t_List)
            ,maybeTuple = maybeTupleTokenId}
@@ -71,7 +41,7 @@ prettyPrintSimple width pp =
   maybeTupleTokenId t = case t of
                           TupleId n -> Just n
                           _         -> Nothing
-
+-}
 
 prettyPrintTokenId :: Flags -> (PPInfo TokenId -> a -> Doc) -> a -> String
 prettyPrintTokenId flags pp =
@@ -80,6 +50,7 @@ prettyPrintTokenId flags pp =
            ,indent = sShowIndent flags
            ,id2str = id2strTokenId
            ,tyVar2str = show
+           ,intState = error "prettyPrintTokenId: no intState"
            ,isFunctionArrow = (== t_Arrow)
            ,isList = (== t_List)
            ,maybeTuple = maybeTupleTokenId}
@@ -193,9 +164,6 @@ fSpace = fdelimiter " "
 fComma :: Doc
 fComma = fdelimiter "" <> text ","
 
-fSemi :: Doc
-fSemi = fdelimiter ";"
-
 dSemiSpace :: Doc
 dSemiSpace = delimiter "; "
 
@@ -743,16 +711,6 @@ ppField info (FieldPun pos var) =
   ppPos info pos <> ppIdAsVar info var
 
 
-ppQual :: PPInfo a -> Qual a -> Doc
-ppQual info (QualExp e) = ppExp info e
-ppQual info (QualPatExp pat e) =
-  groupNestS info $
-    ppPat info pat <> dSpace <> text "<- " <>  ppExp info e
-ppQual info (QualLet ds) =
-  groupNestS info $ text "let" <> dSpace <> ppDecls info ds 
-
-
-
 ppAlt :: PPInfo a -> String -> Alt a -> Doc
 ppAlt info delimiter (Alt pat rhs w) =
   groupNestS info $
