@@ -4,9 +4,11 @@ module Hat.Array
     ,gassocs,aassocs,hassocs,gaccumArray,aaccumArray,haccumArray,(!//),(+//)
     ,(*//),gaccum,aaccum,haccum,gixmap,aixmap,hixmap) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 import Hat.Ix 
 import Hat.List  ((!\\))
@@ -16,8 +18,7 @@ data Ix a => Array a b = MkArray (T.R (T.Tuple2 a a)) (T.R (T.Fun a b))
 instance Ix a => T.WrapVal ((Array a b))
   where
   
-  T.wrapVal pwrapVal (kwrapVal@(MkArray (T.R _ z1wrapVal) (T.R _ z2wrapVal)))
-    p =
+  wrapVal pwrapVal (kwrapVal@(MkArray (T.R _ z1wrapVal) (T.R _ z2wrapVal))) p =
     T.R kwrapVal (T.mkValueApp2 p pwrapVal aMkArray z1wrapVal z2wrapVal)
   
 
@@ -43,46 +44,46 @@ harray fb fivs p =
             (\ f_x f_y p ->
               T.uccase T.mkNoSrcPos p
                 (let
-                  v0v0v1 (T.R (T.Tuple2 fi _) _) p =
+                  v0v0v0v0v1 (T.R (T.Tuple2 fi _) _) p =
                     T.uap1 T.mkNoSrcPos p
                       (T.pa1 T.Cons T.cn1 T.mkNoSrcPos p T.aCons
                         (T.uap2 T.mkNoSrcPos p (ginRange T.mkNoSrcPos p) fb fi))
                       f_y
-                  v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in (v0v0v1))
-                f_x)) fivs) (T.fromExpList T.mkNoSrcPos p [])))
+                  v0v0v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in
+                  (v0v0v0v0v1)) f_x)) fivs) (T.fromExpList T.mkNoSrcPos p [])))
     (T.con2 T.mkNoSrcPos p MkArray aMkArray fb
       (T.ufun1 T.mkLambda T.mkNoSrcPos p
         (\ fj p ->
           T.uccase T.mkNoSrcPos p
             (let
-              v18v29v1 (T.R (T.Cons fv (T.R T.List _)) _) p =
+              v18v29v23v75v1 (T.R (T.Cons fv (T.R T.List _)) _) p =
                 T.projection T.mkNoSrcPos p fv
-              v18v29v1 (T.R T.List _) p =
+              v18v29v23v75v1 (T.R T.List _) p =
                 T.uwrapForward p
                   (herror
                     (T.fromLitString T.mkNoSrcPos p
                       "Array.!: undefined array element") p)
-              v18v29v1 _ p =
+              v18v29v23v75v1 _ p =
                 T.uwrapForward p
                   (herror
                     (T.fromLitString T.mkNoSrcPos p
                       "Array.!: multiply defined array element") p) in
-              (v18v29v1))
+              (v18v29v23v75v1))
             (T.uap1 T.mkNoSrcPos p
               (T.uap2 T.mkNoSrcPos p (Hat.Prelude.g_foldr T.mkNoSrcPos p)
                 (T.ufun2 T.mkLambda T.mkNoSrcPos p
                   (\ f_x f_y p ->
                     T.uccase T.mkNoSrcPos p
                       (let
-                        v0v0v1 (T.R (T.Tuple2 fi fv) _) p =
+                        v0v0v0v0v1 (T.R (T.Tuple2 fi fv) _) p =
                           T.uap1 T.mkNoSrcPos p
                             (T.uap2 T.mkNoSrcPos p
                               (Hat.Prelude.g_filter T.mkNoSrcPos p)
                               (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !== p) fi fj)
-                              (T.pa1 T.Cons T.cn1 T.mkNoSrcPos p T.aCons fv))
-                            f_y
-                        v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in
-                        (v0v0v1)) f_x)) fivs)
+                              (T.pa1 T.Cons T.cn1 T.mkNoSrcPos p T.aCons
+                                (T.projection T.mkNoSrcPos p fv))) f_y
+                        v0v0v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in
+                        (v0v0v0v0v1)) f_x)) fivs)
               (T.fromExpList T.mkNoSrcPos p [])))))
     (T.uwrapForward p
       (herror
@@ -154,12 +155,12 @@ helems fa p =
         (\ f_x f_y p ->
           T.uccase T.mkNoSrcPos p
             (let
-              v0v0v1 fi p =
+              v0v0v0v0v1 fi p =
                 T.uap1 T.mkNoSrcPos p
                   (T.pa1 T.Cons T.cn1 T.mkNoSrcPos p T.aCons
                     (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !! p) fa fi)) f_y
-              v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in (v0v0v1)) f_x))
-      (T.uap1 T.mkNoSrcPos p (gindices T.mkNoSrcPos p) fa))
+              v0v0v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in (v0v0v0v0v1))
+            f_x)) (T.uap1 T.mkNoSrcPos p (gindices T.mkNoSrcPos p) fa))
     (T.fromExpList T.mkNoSrcPos p [])
 
 gassocs ::
@@ -177,13 +178,13 @@ hassocs fa p =
         (\ f_x f_y p ->
           T.uccase T.mkNoSrcPos p
             (let
-              v0v0v1 fi p =
+              v0v0v0v0v1 fi p =
                 T.uap1 T.mkNoSrcPos p
                   (T.pa1 T.Cons T.cn1 T.mkNoSrcPos p T.aCons
                     (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2 fi
                       (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !! p) fa fi))) f_y
-              v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in (v0v0v1)) f_x))
-      (T.uap1 T.mkNoSrcPos p (gindices T.mkNoSrcPos p) fa))
+              v0v0v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in (v0v0v0v0v1))
+            f_x)) (T.uap1 T.mkNoSrcPos p (gindices T.mkNoSrcPos p) fa))
     (T.fromExpList T.mkNoSrcPos p [])
 
 (!//) ::
@@ -208,7 +209,7 @@ hassocs fa p =
   gold_ivs pold_ivs p = T.uconstUse pold_ivs p sold_ivs
   
   sold_ivs =
-    T.uconstDef p a47v20old_ivs
+    T.uconstDef p a47v20v47v26old_ivs
       (\ p ->
         T.uap1 T.mkNoSrcPos p
           (T.uap2 T.mkNoSrcPos p (Hat.Prelude.g_foldr T.mkNoSrcPos p)
@@ -216,7 +217,7 @@ hassocs fa p =
               (\ f_x f_y p ->
                 T.uccase T.mkNoSrcPos p
                   (let
-                    v0v0v1 fi p =
+                    v0v0v0v0v1 fi p =
                       T.uap1 T.mkNoSrcPos p
                         (T.uap2 T.mkNoSrcPos p
                           (Hat.Prelude.g_filter T.mkNoSrcPos p)
@@ -226,14 +227,15 @@ hassocs fa p =
                             (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2 fi
                               (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !! p) fa
                                 fi)))) f_y
-                    v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in (v0v0v1))
-                  f_x)) (T.uap1 T.mkNoSrcPos p (gindices T.mkNoSrcPos p) fa))
+                    v0v0v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in
+                    (v0v0v0v0v1)) f_x))
+            (T.uap1 T.mkNoSrcPos p (gindices T.mkNoSrcPos p) fa))
           (T.fromExpList T.mkNoSrcPos p []))
   
   gnew_is pnew_is p = T.uconstUse pnew_is p snew_is
   
   snew_is =
-    T.uconstDef p a49v20new_is
+    T.uconstDef p a49v20v49v25new_is
       (\ p ->
         T.uap1 T.mkNoSrcPos p
           (T.uap2 T.mkNoSrcPos p (Hat.Prelude.g_foldr T.mkNoSrcPos p)
@@ -241,11 +243,13 @@ hassocs fa p =
               (\ f_x f_y p ->
                 T.uccase T.mkNoSrcPos p
                   (let
-                    v0v0v1 (T.R (T.Tuple2 fi _) _) p =
+                    v0v0v0v0v1 (T.R (T.Tuple2 fi _) _) p =
                       T.uap1 T.mkNoSrcPos p
-                        (T.pa1 T.Cons T.cn1 T.mkNoSrcPos p T.aCons fi) f_y
-                    v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in (v0v0v1))
-                  f_x)) fnew_ivs) (T.fromExpList T.mkNoSrcPos p []))
+                        (T.pa1 T.Cons T.cn1 T.mkNoSrcPos p T.aCons
+                          (T.projection T.mkNoSrcPos p fi)) f_y
+                    v0v0v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in
+                    (v0v0v0v0v1)) f_x)) fnew_ivs)
+          (T.fromExpList T.mkNoSrcPos p []))
   
 
 gaccum ::
@@ -267,14 +271,15 @@ gaccum paccum p = T.ufun1 aaccum paccum p haccum
 haccum ff p =
   T.uap1 T.mkNoSrcPos p (gfoldl T.mkNoSrcPos p)
     (T.ufun2 T.mkLambda T.mkNoSrcPos p
-      (\ v53v33v1 v53v33v2 p ->
-        case (v53v33v1,v53v33v2) of
+      (\ v53v33v53v64v1 v53v33v53v64v2 p ->
+        case (v53v33v53v64v1,v53v33v53v64v2) of
           (fa,T.R (T.Tuple2 fi fv) _) ->
             T.uwrapForward p
               ((fa
                   *//
                   (T.fromExpList T.mkNoSrcPos p
-                    [T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2 fi
+                    [T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
+                        (T.projection T.mkNoSrcPos p fi)
                         (T.uap2 T.mkNoSrcPos p ff
                           (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !! p) fa fi)
                           fv)])) p)
@@ -308,12 +313,13 @@ haccumArray ff fz fb p =
               (\ f_x f_y p ->
                 T.uccase T.mkNoSrcPos p
                   (let
-                    v0v0v1 fi p =
+                    v0v0v0v0v1 fi p =
                       T.uap1 T.mkNoSrcPos p
                         (T.pa1 T.Cons T.cn1 T.mkNoSrcPos p T.aCons
                           (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2 fi fz)) f_y
-                    v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in (v0v0v1))
-                  f_x)) (T.uap1 T.mkNoSrcPos p (grange T.mkNoSrcPos p) fb))
+                    v0v0v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in
+                    (v0v0v0v0v1)) f_x))
+            (T.uap1 T.mkNoSrcPos p (grange T.mkNoSrcPos p) fb))
           (T.fromExpList T.mkNoSrcPos p [])) p))
 
 gixmap ::
@@ -340,21 +346,22 @@ hixmap fb ff fa p =
             (\ f_x f_y p ->
               T.uccase T.mkNoSrcPos p
                 (let
-                  v0v0v1 fi p =
+                  v0v0v0v0v1 fi p =
                     T.uap1 T.mkNoSrcPos p
                       (T.pa1 T.Cons T.cn1 T.mkNoSrcPos p T.aCons
                         (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2 fi
                           (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !! p) fa
                             (T.uap1 T.mkNoSrcPos p ff fi)))) f_y
-                  v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in (v0v0v1))
-                f_x)) (T.uap1 T.mkNoSrcPos p (grange T.mkNoSrcPos p) fb))
+                  v0v0v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in
+                  (v0v0v0v0v1)) f_x))
+          (T.uap1 T.mkNoSrcPos p (grange T.mkNoSrcPos p) fb))
         (T.fromExpList T.mkNoSrcPos p [])) p)
 
 instance Ix a => Functor ((Array a))
   where
   
   gfmap pfmap p =
-    T.ufun2 a64v5fmap pfmap p hfmap
+    T.ufun2 a64v5v64v46fmap pfmap p hfmap
     where
     
     hfmap ffn (T.R (MkArray fb ff) _) p =
@@ -368,7 +375,7 @@ instance (Ix a,Eq b) => Eq ((Array a b))
   where
   
   (!==) (%==) p =
-    T.ufun2 (++^=^==) (%==) p (*==)
+    T.ufun2 (++^=^=+^=@==) (%==) p (*==)
     where
     
     (*==) fa fa' p =
@@ -381,7 +388,7 @@ instance (Ix a,Ord b) => Ord ((Array a b))
   where
   
   (!<=) (%<=) p =
-    T.ufun2 (+^!=^<=) (%<=) p (*<=)
+    T.ufun2 (+^!=^=^!=@<=) (%<=) p (*<=)
     where
     
     (*<=) fa fa' p =
@@ -394,7 +401,7 @@ instance (Ix a,Show a,Show b) => Show ((Array a b))
   where
   
   gshowsPrec pshowsPrec p =
-    T.ufun2 a73v5showsPrec pshowsPrec p hshowsPrec
+    T.ufun2 a73v5v76v35showsPrec pshowsPrec p hshowsPrec
     where
     
     hshowsPrec fp fa p =
@@ -422,7 +429,7 @@ instance (Ix a,Read a,Read b) => Read ((Array a b))
   where
   
   greadsPrec preadsPrec p =
-    T.ufun1 a79v5readsPrec preadsPrec p hreadsPrec
+    T.ufun1 a79v5v80v13readsPrec preadsPrec p hreadsPrec
     where
     
     hreadsPrec fp p =
@@ -440,7 +447,7 @@ instance (Ix a,Read a,Read b) => Read ((Array a b))
                     (\ f_x f_y p ->
                       T.uccase T.mkNoSrcPos p
                         (let
-                          v0v0v1
+                          v0v0v0v0v1
                             (T.R
                               (T.Tuple2
                                 (T.R
@@ -462,7 +469,7 @@ instance (Ix a,Read a,Read b) => Read ((Array a b))
                                   (\ f_x f_y p ->
                                     T.uccase T.mkNoSrcPos p
                                       (let
-                                        v0v0v1 (T.R (T.Tuple2 fb ft) _) p =
+                                        v0v0v0v0v1 (T.R (T.Tuple2 fb ft) _) p =
                                           T.uap1 T.mkNoSrcPos p
                                             (T.uap2 T.mkNoSrcPos p
                                               (Hat.Prelude.g_foldr T.mkNoSrcPos
@@ -471,7 +478,7 @@ instance (Ix a,Read a,Read b) => Read ((Array a b))
                                                 (\ f_x f_y p ->
                                                   T.uccase T.mkNoSrcPos p
                                                     (let
-                                                      v0v0v1
+                                                      v0v0v0v0v1
                                                         (T.R (T.Tuple2 fas fu)
                                                           _) p =
                                                         T.uap1 T.mkNoSrcPos p
@@ -484,59 +491,62 @@ instance (Ix a,Read a,Read b) => Read ((Array a b))
                                                               (T.uwrapForward p
                                                                 (harray fb fas
                                                                   p)) fu)) f_y
-                                                      v0v0v1 _ p =
+                                                      v0v0v0v0v1 _ p =
                                                         T.projection
                                                           T.mkNoSrcPos p f_y in
-                                                      (v0v0v1)) f_x))
+                                                      (v0v0v0v0v1)) f_x))
                                               (T.uap1 T.mkNoSrcPos p
                                                 (greads T.mkNoSrcPos p) ft)) f_y
-                                        v0v0v1 _ p =
+                                        v0v0v0v0v1 _ p =
                                           T.projection T.mkNoSrcPos p f_y in
-                                        (v0v0v1)) f_x))
+                                        (v0v0v0v0v1)) f_x))
                                 (T.uap1 T.mkNoSrcPos p (greads T.mkNoSrcPos p)
                                   fs)) f_y
-                          v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in
-                          (v0v0v1)) f_x)) (T.uwrapForward p (hlex fr p)))
+                          v0v0v0v0v1 _ p = T.projection T.mkNoSrcPos p f_y in
+                          (v0v0v0v0v1)) f_x)) (T.uwrapForward p (hlex fr p)))
                 (T.fromExpList T.mkNoSrcPos p []))) p)
     
   
 
 tArray = T.mkModule "Array" "Array.hs" Prelude.False
 
-aMkArray = T.mkConstructor tArray 120028 3 2 "MkArray"
+aMkArray = T.mkConstructor tArray 120028 120034 3 2 "MkArray"
 
-aarray = T.mkVariable tArray 150001 3 2 "array" Prelude.False
+aarray = T.mkVariable tArray 150001 240064 3 2 "array" Prelude.False
 
-alistArray = T.mkVariable tArray 270001 3 2 "listArray" Prelude.False
+alistArray = T.mkVariable tArray 270001 270071 3 2 "listArray" Prelude.False
 
-(+!) = T.mkVariable tArray 300002 38 1 "!" Prelude.False
+(+!) = T.mkVariable tArray 300002 300026 38 1 "!" Prelude.False
 
-abounds = T.mkVariable tArray 330001 3 1 "bounds" Prelude.False
+abounds = T.mkVariable tArray 330001 330026 3 1 "bounds" Prelude.False
 
-aindices = T.mkVariable tArray 360001 3 0 "indices" Prelude.False
+aindices = T.mkVariable tArray 360001 360039 3 0 "indices" Prelude.False
 
-aelems = T.mkVariable tArray 390001 3 1 "elems" Prelude.False
+aelems = T.mkVariable tArray 390001 390005 3 1 "elems" Prelude.False
 
-aassocs = T.mkVariable tArray 420001 3 1 "assocs" Prelude.False
+aassocs = T.mkVariable tArray 420001 420006 3 1 "assocs" Prelude.False
 
-(+//) = T.mkVariable tArray 450003 38 2 "//" Prelude.False
+(+//) = T.mkVariable tArray 450003 450004 38 2 "//" Prelude.False
 
-aaccum = T.mkVariable tArray 530001 3 1 "accum" Prelude.False
+aaccum = T.mkVariable tArray 530001 530064 3 1 "accum" Prelude.False
 
-aaccumArray = T.mkVariable tArray 570001 3 3 "accumArray" Prelude.False
+aaccumArray = T.mkVariable tArray 570001 570039 3 3 "accumArray" Prelude.False
 
-aixmap = T.mkVariable tArray 610001 3 3 "ixmap" Prelude.False
+aixmap = T.mkVariable tArray 610001 610029 3 3 "ixmap" Prelude.False
 
-a64v5fmap = T.mkVariable tArray 640005 3 2 "fmap" Prelude.False
+a64v5v64v46fmap = T.mkVariable tArray 640005 640046 3 2 "fmap" Prelude.False
 
-(++^=^==) = T.mkVariable tArray 670007 16 2 "==" Prelude.False
+(++^=^=+^=@==) = T.mkVariable tArray 670007 670008 16 2 "==" Prelude.False
 
-(+^!=^<=) = T.mkVariable tArray 700007 16 2 "<=" Prelude.False
+(+^!=^=^!=@<=) = T.mkVariable tArray 700007 700008 16 2 "<=" Prelude.False
 
-a73v5showsPrec = T.mkVariable tArray 730005 3 2 "showsPrec" Prelude.False
+a73v5v76v35showsPrec =
+  T.mkVariable tArray 730005 760035 3 2 "showsPrec" Prelude.False
 
-a79v5readsPrec = T.mkVariable tArray 790005 3 1 "readsPrec" Prelude.False
+a79v5v80v13readsPrec =
+  T.mkVariable tArray 790005 800013 3 1 "readsPrec" Prelude.False
 
-a47v20old_ivs = T.mkVariable tArray 470020 3 0 "old_ivs" Prelude.True
+a47v20v47v26old_ivs =
+  T.mkVariable tArray 470020 470026 3 0 "old_ivs" Prelude.True
 
-a49v20new_is = T.mkVariable tArray 490020 3 0 "new_is" Prelude.True
+a49v20v49v25new_is = T.mkVariable tArray 490020 490025 3 0 "new_is" Prelude.True
