@@ -4,9 +4,11 @@ module Hat.Complex
     ,acis,hcis,gpolar,apolar,hpolar,gmagnitude,amagnitude,hmagnitude,gphase
     ,aphase,hphase) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 
 data RealFloat a => Complex a = (:+) !(T.R a) !(T.R a)
@@ -14,7 +16,7 @@ data RealFloat a => Complex a = (:+) !(T.R a) !(T.R a)
 instance RealFloat a => T.WrapVal ((Complex a))
   where
   
-  T.wrapVal pwrapVal (kwrapVal@((T.R _ z1wrapVal) :+ (T.R _ z2wrapVal))) p =
+  wrapVal pwrapVal (kwrapVal@((T.R _ z1wrapVal) :+ (T.R _ z2wrapVal))) p =
     T.R kwrapVal (T.mkValueApp2 p pwrapVal (+:+) z1wrapVal z2wrapVal)
   
 
@@ -22,7 +24,7 @@ instance (RealFloat a,Eq a) => Eq ((Complex a))
   where
   
   (!==) (%==) p =
-    T.ufun2 (++=+!==) (%==) p (*==)
+    T.ufun2 (++=+!=+=+#==) (%==) p (*==)
     where
     
     (*==) (T.R (fy1 :+ fy2) _) (T.R (fy3 :+ fy4) _) p =
@@ -38,7 +40,7 @@ instance (RealFloat a,Read a) => Read ((Complex a))
   where
   
   greadsPrec preadsPrec p =
-    T.ufun1 a6v63readsPrec preadsPrec p hreadsPrec
+    T.ufun1 a6v63v6v66readsPrec preadsPrec p hreadsPrec
     where
     
     hreadsPrec fy1 p =
@@ -71,7 +73,7 @@ instance (RealFloat a,Show a) => Show ((Complex a))
   where
   
   gshowsPrec pshowsPrec p =
-    T.ufun2 a6v68showsPrec pshowsPrec p hshowsPrec
+    T.ufun2 a6v68v6v71showsPrec pshowsPrec p hshowsPrec
     where
     
     hshowsPrec fy1 (T.R (fy2 :+ fy3) _) p =
@@ -195,7 +197,7 @@ hmagnitude (T.R (fx :+ fy) _) p =
   gk pk p = T.uconstUse pk p sk
   
   sk =
-    T.uconstDef p a28v11k
+    T.uconstDef p a28v11v28v43k
       (\ p ->
         T.uap2 T.mkNoSrcPos p (gmax T.mkNoSrcPos p)
           (T.uap1 T.mkNoSrcPos p (gexponent T.mkNoSrcPos p) fx)
@@ -204,7 +206,7 @@ hmagnitude (T.R (fx :+ fy) _) p =
   gmk pmk p = T.uconstUse pmk p smk
   
   smk =
-    T.uconstDef p a29v11mk
+    T.uconstDef p a29v11v29v18mk
       (\ p ->
         T.uap1 T.mkNoSrcPos p (Hat.Prelude.gnegate T.mkNoSrcPos p)
           (gk T.mkNoSrcPos p))
@@ -217,16 +219,17 @@ hphase :: RealFloat a => (T.R (Complex a)) -> T.RefExp -> T.R a
 
 gphase pphase p = T.ufun1 aphase pphase p hphase
 
-hphase (z1phase@(T.R (fv32v8n :+ v32v13n) _)) p =
+hphase (z1phase@(T.R (fv32v8v32v8n :+ v32v13v32v13n) _)) p =
   T.ucguard
-    (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv32v8n
+    (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv32v8v32v8n
       (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-        (T.conInteger T.mkNoSrcPos p 0))) (h v32v13n p) (y1phase z1phase p)
+        (T.conInteger T.mkNoSrcPos p 0))) (h v32v13v32v13n p)
+    (y1phase z1phase p)
   where
   
-  h fv32v13n p =
+  h fv32v13v32v13n p =
     T.ucguard
-      (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv32v13n
+      (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv32v13v32v13n
         (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
           (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1phase z1phase p)
     where
@@ -248,7 +251,7 @@ instance RealFloat a => Num ((Complex a))
   where
   
   (!+) (%+) p =
-    T.ufun2 (+%^=#$+) (%+) p (*+)
+    T.ufun2 (+%^=#$=%^=#$+) (%+) p (*+)
     where
     
     (*+) (T.R (fx :+ fy) _) (T.R (fx' :+ fy') _) p =
@@ -259,7 +262,7 @@ instance RealFloat a => Num ((Complex a))
     
   
   (!-) (%-) p =
-    T.ufun2 (+%@=#$-) (%-) p (*-)
+    T.ufun2 (+%@=#$=%@=#$-) (%-) p (*-)
     where
     
     (*-) (T.R (fx :+ fy) _) (T.R (fx' :+ fy') _) p =
@@ -270,7 +273,7 @@ instance RealFloat a => Num ((Complex a))
     
   
   (!*) (%*) p =
-    T.ufun2 (+%>=#$*) (%*) p (**)
+    T.ufun2 (+%>=#$=%>=#$*) (%*) p (**)
     where
     
     (**) (T.R (fx :+ fy) _) (T.R (fx' :+ fy') _) p =
@@ -285,7 +288,7 @@ instance RealFloat a => Num ((Complex a))
     
   
   gnegate pnegate p =
-    T.ufun1 a40v5negate pnegate p hnegate
+    T.ufun1 a40v5v40v41negate pnegate p hnegate
     where
     
     hnegate (T.R (fx :+ fy) _) p =
@@ -296,7 +299,7 @@ instance RealFloat a => Num ((Complex a))
     
   
   gabs pabs p =
-    T.ufun1 a41v5abs pabs p habs
+    T.ufun1 a41v5v41v29abs pabs p habs
     where
     
     habs fz p =
@@ -306,22 +309,22 @@ instance RealFloat a => Num ((Complex a))
     
   
   gsignum psignum p =
-    T.ufun1 a42v5signum psignum p hsignum
+    T.ufun1 a42v5v43v56signum psignum p hsignum
     where
     
-    hsignum fv42v12n p =
+    hsignum fv42v12v42v12n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv42v12n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv42v12v42v12n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1signum fv42v12n p)
+            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1signum fv42v12v42v12n p)
       where
       
       h p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
           (T.conInteger T.mkNoSrcPos p 0)
-      h p = y1signum fv42v12n p
+      h p = y1signum fv42v12v42v12n p
       
-    hsignum fv42v12n p = y1signum fv42v12n p
+    hsignum fv42v12v42v12n p = y1signum fv42v12v42v12n p
     
     y1signum (fz@(T.R (fx :+ fy) _)) p =
       T.con2 T.mkNoSrcPos p (:+) (+:+)
@@ -331,13 +334,14 @@ instance RealFloat a => Num ((Complex a))
       
       gr pr p = T.uconstUse pr p sr
       
-      sr = T.uconstDef p a43v42r (\ p -> T.uwrapForward p (hmagnitude fz p))
+      sr =
+        T.uconstDef p a43v42v43v56r (\ p -> T.uwrapForward p (hmagnitude fz p))
       
     y1signum _ p = T.fatal p
     
   
   gfromInteger pfromInteger p =
-    T.ufun1 a44v5fromInteger pfromInteger p hfromInteger
+    T.ufun1 a44v5v44v39fromInteger pfromInteger p hfromInteger
     where
     
     hfromInteger fn p =
@@ -352,7 +356,7 @@ instance RealFloat a => Fractional ((Complex a))
   where
   
   (!/) (%/) p =
-    T.ufun2 (+&^=#!/) (%/) p (*/)
+    T.ufun2 (+&^=#!=&^=#!/) (%/) p (*/)
     where
     
     (*/) (T.R (fx :+ fy) _) (T.R (fx' :+ fy') _) p =
@@ -372,7 +376,7 @@ instance RealFloat a => Fractional ((Complex a))
       gx'' px'' p = T.uconstUse px'' p sx''
       
       sx'' =
-        T.uconstDef p a49v5x''
+        T.uconstDef p a49v5v49v25x''
           (\ p ->
             T.uap2 T.mkNoSrcPos p (gscaleFloat T.mkNoSrcPos p)
               (gk T.mkNoSrcPos p) fx')
@@ -380,7 +384,7 @@ instance RealFloat a => Fractional ((Complex a))
       gy'' py'' p = T.uconstUse py'' p sy''
       
       sy'' =
-        T.uconstDef p a50v5y''
+        T.uconstDef p a50v5v50v25y''
           (\ p ->
             T.uap2 T.mkNoSrcPos p (gscaleFloat T.mkNoSrcPos p)
               (gk T.mkNoSrcPos p) fy')
@@ -388,7 +392,7 @@ instance RealFloat a => Fractional ((Complex a))
       gk pk p = T.uconstUse pk p sk
       
       sk =
-        T.uconstDef p a51v5k
+        T.uconstDef p a51v5v51v42k
           (\ p ->
             T.uap1 T.mkNoSrcPos p (Hat.Prelude.gnegate T.mkNoSrcPos p)
               (T.uap2 T.mkNoSrcPos p (gmax T.mkNoSrcPos p)
@@ -398,7 +402,7 @@ instance RealFloat a => Fractional ((Complex a))
       gd pd p = T.uconstUse pd p sd
       
       sd =
-        T.uconstDef p a52v5d
+        T.uconstDef p a52v5v52v25d
           (\ p ->
             T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p)
               (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !* p) fx'
@@ -410,7 +414,7 @@ instance RealFloat a => Fractional ((Complex a))
     
   
   gfromRational pfromRational p =
-    T.ufun1 a54v3fromRational pfromRational p hfromRational
+    T.ufun1 a54v3v54v39fromRational pfromRational p hfromRational
     where
     
     hfromRational fa p =
@@ -427,14 +431,14 @@ instance RealFloat a => Floating ((Complex a))
   gpi ppi p = T.uconstUse ppi p spi
   
   spi =
-    T.uconstDef T.mkRoot a57v5pi
+    T.uconstDef T.mkRoot a57v5v57v29pi
       (\ p ->
         T.con2 T.mkNoSrcPos p (:+) (+:+) (gpi T.mkNoSrcPos p)
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
             (T.conInteger T.mkNoSrcPos p 0)))
   
   gexp pexp p =
-    T.ufun1 a58v5exp pexp p hexp
+    T.ufun1 a58v5v59v40exp pexp p hexp
     where
     
     hexp (T.R (fx :+ fy) _) p =
@@ -448,14 +452,14 @@ instance RealFloat a => Floating ((Complex a))
       gexpx pexpx p = T.uconstUse pexpx p sexpx
       
       sexpx =
-        T.uconstDef p a59v29expx
+        T.uconstDef p a59v29v59v40expx
           (\ p -> T.uap1 T.mkNoSrcPos p (gexp T.mkNoSrcPos p) fx)
       
     hexp _ p = T.fatal p
     
   
   glog plog p =
-    T.ufun1 a60v5log plog p hlog
+    T.ufun1 a60v5v60v50log plog p hlog
     where
     
     hlog fz p =
@@ -465,22 +469,22 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gsqrt psqrt p =
-    T.ufun1 a62v5sqrt psqrt p hsqrt
+    T.ufun1 a62v5v66v67sqrt psqrt p hsqrt
     where
     
-    hsqrt fv62v10n p =
+    hsqrt fv62v10v62v10n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv62v10n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv62v10v62v10n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1sqrt fv62v10n p)
+            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1sqrt fv62v10v62v10n p)
       where
       
       h p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
           (T.conInteger T.mkNoSrcPos p 0)
-      h p = y1sqrt fv62v10n p
+      h p = y1sqrt fv62v10v62v10n p
       
-    hsqrt fv62v10n p = y1sqrt fv62v10n p
+    hsqrt fv62v10v62v10n p = y1sqrt fv62v10v62v10n p
     
     y1sqrt (fz@(T.R (fx :+ fy) _)) p =
       T.con2 T.mkNoSrcPos p (:+) (+:+) (gu T.mkNoSrcPos p)
@@ -497,7 +501,7 @@ instance RealFloat a => Floating ((Complex a))
       
       gv pu p = T.uconstUse pu p sv
       
-      j64v29u =
+      j64v29v64v33u =
         case
           T.ucif p
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !< p) fx
@@ -511,14 +515,18 @@ instance RealFloat a => Floating ((Complex a))
           T.R (T.Tuple2 fu fv) ku -> (ku,fu,fv)
           _ -> T.fatal p
       
-      su = T.uconstDef p a64v30u (\ _ -> case j64v29u of (ku,fu,fv) -> fu)
+      su =
+        T.uconstDef p a64v30v64v30u
+          (\ _ -> case j64v29v64v33u of (ku,fu,fv) -> fu)
       
-      sv = T.uconstDef p a64v32v (\ _ -> case j64v29u of (ku,fu,fv) -> fv)
+      sv =
+        T.uconstDef p a64v32v64v32v
+          (\ _ -> case j64v29v64v33u of (ku,fu,fv) -> fv)
       
       gv' pv' p = T.uconstUse pv' p sv'
       
       sv' =
-        T.uconstDef p a65v29v'
+        T.uconstDef p a65v29v65v49v'
           (\ p ->
             T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !/ p)
               (T.uap1 T.mkNoSrcPos p (gabs T.mkNoSrcPos p) fy)
@@ -530,7 +538,7 @@ instance RealFloat a => Floating ((Complex a))
       gu' pu' p = T.uconstUse pu' p su'
       
       su' =
-        T.uconstDef p a66v29u'
+        T.uconstDef p a66v29v66v67u'
           (\ p ->
             T.uap1 T.mkNoSrcPos p (gsqrt T.mkNoSrcPos p)
               (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !/ p)
@@ -545,7 +553,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gsin psin p =
-    T.ufun1 a68v5sin psin p hsin
+    T.ufun1 a68v5v68v54sin psin p hsin
     where
     
     hsin (T.R (fx :+ fy) _) p =
@@ -560,7 +568,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gcos pcos p =
-    T.ufun1 a69v5cos pcos p hcos
+    T.ufun1 a69v5v69v57cos pcos p hcos
     where
     
     hcos (T.R (fx :+ fy) _) p =
@@ -576,7 +584,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gtan ptan p =
-    T.ufun1 a70v5tan ptan p htan
+    T.ufun1 a70v5v74v42tan ptan p htan
     where
     
     htan (T.R (fx :+ fy) _) p =
@@ -597,32 +605,32 @@ instance RealFloat a => Floating ((Complex a))
       gsinx psinx p = T.uconstUse psinx p ssinx
       
       ssinx =
-        T.uconstDef p a71v29sinx
+        T.uconstDef p a71v29v71v41sinx
           (\ p -> T.uap1 T.mkNoSrcPos p (gsin T.mkNoSrcPos p) fx)
       
       gcosx pcosx p = T.uconstUse pcosx p scosx
       
       scosx =
-        T.uconstDef p a72v29cosx
+        T.uconstDef p a72v29v72v41cosx
           (\ p -> T.uap1 T.mkNoSrcPos p (gcos T.mkNoSrcPos p) fx)
       
       gsinhy psinhy p = T.uconstUse psinhy p ssinhy
       
       ssinhy =
-        T.uconstDef p a73v29sinhy
+        T.uconstDef p a73v29v73v42sinhy
           (\ p -> T.uap1 T.mkNoSrcPos p (gsinh T.mkNoSrcPos p) fy)
       
       gcoshy pcoshy p = T.uconstUse pcoshy p scoshy
       
       scoshy =
-        T.uconstDef p a74v29coshy
+        T.uconstDef p a74v29v74v42coshy
           (\ p -> T.uap1 T.mkNoSrcPos p (gcosh T.mkNoSrcPos p) fy)
       
     htan _ p = T.fatal p
     
   
   gsinh psinh p =
-    T.ufun1 a76v5sinh psinh p hsinh
+    T.ufun1 a76v5v76v55sinh psinh p hsinh
     where
     
     hsinh (T.R (fx :+ fy) _) p =
@@ -637,7 +645,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gcosh pcosh p =
-    T.ufun1 a77v5cosh pcosh p hcosh
+    T.ufun1 a77v5v77v54cosh pcosh p hcosh
     where
     
     hcosh (T.R (fx :+ fy) _) p =
@@ -652,7 +660,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gtanh ptanh p =
-    T.ufun1 a78v5tanh ptanh p htanh
+    T.ufun1 a78v5v82v42tanh ptanh p htanh
     where
     
     htanh (T.R (fx :+ fy) _) p =
@@ -672,32 +680,32 @@ instance RealFloat a => Floating ((Complex a))
       gsiny psiny p = T.uconstUse psiny p ssiny
       
       ssiny =
-        T.uconstDef p a79v29siny
+        T.uconstDef p a79v29v79v41siny
           (\ p -> T.uap1 T.mkNoSrcPos p (gsin T.mkNoSrcPos p) fy)
       
       gcosy pcosy p = T.uconstUse pcosy p scosy
       
       scosy =
-        T.uconstDef p a80v29cosy
+        T.uconstDef p a80v29v80v41cosy
           (\ p -> T.uap1 T.mkNoSrcPos p (gcos T.mkNoSrcPos p) fy)
       
       gsinhx psinhx p = T.uconstUse psinhx p ssinhx
       
       ssinhx =
-        T.uconstDef p a81v29sinhx
+        T.uconstDef p a81v29v81v42sinhx
           (\ p -> T.uap1 T.mkNoSrcPos p (gsinh T.mkNoSrcPos p) fx)
       
       gcoshx pcoshx p = T.uconstUse pcoshx p scoshx
       
       scoshx =
-        T.uconstDef p a82v29coshx
+        T.uconstDef p a82v29v82v42coshx
           (\ p -> T.uap1 T.mkNoSrcPos p (gcosh T.mkNoSrcPos p) fx)
       
     htanh _ p = T.fatal p
     
   
   gasin pasin p =
-    T.ufun1 a84v5asin pasin p hasin
+    T.ufun1 a84v5v85v70asin pasin p hasin
     where
     
     hasin (fz@(T.R (fx :+ fy) _)) p =
@@ -710,13 +718,13 @@ instance RealFloat a => Floating ((Complex a))
       
       gy' px' p = T.uconstUse px' p sy'
       
-      j85v33x' =
+      j85v31v85v36x' =
         case
           T.uap1 T.mkNoSrcPos p (glog T.mkNoSrcPos p)
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p)
               (T.con2 T.mkNoSrcPos p (:+) (+:+)
                 (T.uap1 T.mkNoSrcPos p (Hat.Prelude.gnegate T.mkNoSrcPos p) fy)
-                fx)
+                (T.projection T.mkNoSrcPos p fx))
               (T.uap1 T.mkNoSrcPos p (gsqrt T.mkNoSrcPos p)
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !- p)
                   (T.uap1 T.mkNoSrcPos p
@@ -727,16 +735,18 @@ instance RealFloat a => Floating ((Complex a))
           _ -> T.fatal p
       
       sx' =
-        T.uconstDef p a85v31x' (\ _ -> case j85v33x' of (kx',fx',fy') -> fx')
+        T.uconstDef p a85v31v85v32x'
+          (\ _ -> case j85v31v85v36x' of (kx',fx',fy') -> fx')
       
       sy' =
-        T.uconstDef p a85v35y' (\ _ -> case j85v33x' of (kx',fx',fy') -> fy')
+        T.uconstDef p a85v35v85v36y'
+          (\ _ -> case j85v31v85v36x' of (kx',fx',fy') -> fy')
       
     hasin _ p = T.fatal p
     
   
   gacos pacos p =
-    T.ufun1 a86v5acos pacos p hacos
+    T.ufun1 a86v5v88v54acos pacos p hacos
     where
     
     hacos (fz@(T.R (fx :+ fy) _)) p =
@@ -749,7 +759,7 @@ instance RealFloat a => Floating ((Complex a))
       
       gy'' px'' p = T.uconstUse px'' p sy''
       
-      j87v33x'' =
+      j87v30v87v37x'' =
         case
           T.uap1 T.mkNoSrcPos p (glog T.mkNoSrcPos p)
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p) fz
@@ -760,18 +770,18 @@ instance RealFloat a => Floating ((Complex a))
           _ -> T.fatal p
       
       sx'' =
-        T.uconstDef p a87v30x''
-          (\ _ -> case j87v33x'' of (kx'',fx'',fy'') -> fx'')
+        T.uconstDef p a87v30v87v32x''
+          (\ _ -> case j87v30v87v37x'' of (kx'',fx'',fy'') -> fx'')
       
       sy'' =
-        T.uconstDef p a87v35y''
-          (\ _ -> case j87v33x'' of (kx'',fx'',fy'') -> fy'')
+        T.uconstDef p a87v35v87v37y''
+          (\ _ -> case j87v30v87v37x'' of (kx'',fx'',fy'') -> fy'')
       
       gx' px' p = T.uconstUse px' p sx'
       
       gy' px' p = T.uconstUse px' p sy'
       
-      j88v32x' =
+      j88v30v88v35x' =
         case
           T.uap1 T.mkNoSrcPos p (gsqrt T.mkNoSrcPos p)
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !- p)
@@ -783,16 +793,18 @@ instance RealFloat a => Floating ((Complex a))
           _ -> T.fatal p
       
       sx' =
-        T.uconstDef p a88v30x' (\ _ -> case j88v32x' of (kx',fx',fy') -> fx')
+        T.uconstDef p a88v30v88v31x'
+          (\ _ -> case j88v30v88v35x' of (kx',fx',fy') -> fx')
       
       sy' =
-        T.uconstDef p a88v34y' (\ _ -> case j88v32x' of (kx',fx',fy') -> fy')
+        T.uconstDef p a88v34v88v35y'
+          (\ _ -> case j88v30v88v35x' of (kx',fx',fy') -> fy')
       
     hacos _ p = T.fatal p
     
   
   gatan patan p =
-    T.ufun1 a89v5atan patan p hatan
+    T.ufun1 a89v5v90v68atan patan p hatan
     where
     
     hatan (fz@(T.R (fx :+ fy) _)) p =
@@ -805,7 +817,7 @@ instance RealFloat a => Floating ((Complex a))
       
       gy' px' p = T.uconstUse px' p sy'
       
-      j90v32x' =
+      j90v30v90v35x' =
         case
           T.uap1 T.mkNoSrcPos p (glog T.mkNoSrcPos p)
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !/ p)
@@ -813,7 +825,8 @@ instance RealFloat a => Floating ((Complex a))
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !- p)
                   (T.uap1 T.mkNoSrcPos p
                     (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-                    (T.conInteger T.mkNoSrcPos p 1)) fy) fx)
+                    (T.conInteger T.mkNoSrcPos p 1)) fy)
+                (T.projection T.mkNoSrcPos p fx))
               (T.uap1 T.mkNoSrcPos p (gsqrt T.mkNoSrcPos p)
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p)
                   (T.uap1 T.mkNoSrcPos p
@@ -824,16 +837,18 @@ instance RealFloat a => Floating ((Complex a))
           _ -> T.fatal p
       
       sx' =
-        T.uconstDef p a90v30x' (\ _ -> case j90v32x' of (kx',fx',fy') -> fx')
+        T.uconstDef p a90v30v90v31x'
+          (\ _ -> case j90v30v90v35x' of (kx',fx',fy') -> fx')
       
       sy' =
-        T.uconstDef p a90v34y' (\ _ -> case j90v32x' of (kx',fx',fy') -> fy')
+        T.uconstDef p a90v34v90v35y'
+          (\ _ -> case j90v30v90v35x' of (kx',fx',fy') -> fy')
       
     hatan _ p = T.fatal p
     
   
   gasinh pasinh p =
-    T.ufun1 a92v5asinh pasinh p hasinh
+    T.ufun1 a92v5v92v42asinh pasinh p hasinh
     where
     
     hasinh fz p =
@@ -848,7 +863,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gacosh pacosh p =
-    T.ufun1 a93v5acosh pacosh p hacosh
+    T.ufun1 a93v5v93v55acosh pacosh p hacosh
     where
     
     hacosh fz p =
@@ -872,7 +887,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gatanh patanh p =
-    T.ufun1 a94v5atanh patanh p hatanh
+    T.ufun1 a94v5v94v46atanh patanh p hatanh
     where
     
     hatanh fz p =
@@ -893,133 +908,138 @@ instance RealFloat a => Floating ((Complex a))
 
 tComplex = T.mkModule "Complex" "Complex.hs" Prelude.False
 
-(+:+) = T.mkConstructor tComplex 60043 24 2 ":+"
+(+:+) = T.mkConstructor tComplex 60043 60044 24 2 ":+"
 
-(++=+!==) = T.mkVariable tComplex 60060 3 2 "==" Prelude.False
+(++=+!=+=+#==) = T.mkVariable tComplex 60060 60061 3 2 "==" Prelude.False
 
-a6v63readsPrec = T.mkVariable tComplex 60063 3 1 "readsPrec" Prelude.False
+a6v63v6v66readsPrec =
+  T.mkVariable tComplex 60063 60066 3 1 "readsPrec" Prelude.False
 
-a6v68showsPrec = T.mkVariable tComplex 60068 3 2 "showsPrec" Prelude.False
+a6v68v6v71showsPrec =
+  T.mkVariable tComplex 60068 60071 3 2 "showsPrec" Prelude.False
 
-arealPart = T.mkVariable tComplex 100001 3 1 "realPart" Prelude.False
+arealPart = T.mkVariable tComplex 100001 100021 3 1 "realPart" Prelude.False
 
-aimagPart = T.mkVariable tComplex 110001 3 1 "imagPart" Prelude.False
+aimagPart = T.mkVariable tComplex 110001 110021 3 1 "imagPart" Prelude.False
 
-aconjugate = T.mkVariable tComplex 140001 3 1 "conjugate" Prelude.False
+aconjugate = T.mkVariable tComplex 140001 140028 3 1 "conjugate" Prelude.False
 
-amkPolar = T.mkVariable tComplex 170001 3 2 "mkPolar" Prelude.False
+amkPolar = T.mkVariable tComplex 170001 170050 3 2 "mkPolar" Prelude.False
 
-acis = T.mkVariable tComplex 200001 3 1 "cis" Prelude.False
+acis = T.mkVariable tComplex 200001 200036 3 1 "cis" Prelude.False
 
-apolar = T.mkVariable tComplex 230001 3 1 "polar" Prelude.False
+apolar = T.mkVariable tComplex 230001 230034 3 1 "polar" Prelude.False
 
-amagnitude = T.mkVariable tComplex 260001 3 1 "magnitude" Prelude.False
+amagnitude = T.mkVariable tComplex 260001 290018 3 1 "magnitude" Prelude.False
 
-aphase = T.mkVariable tComplex 320001 3 1 "phase" Prelude.False
+aphase = T.mkVariable tComplex 320001 330026 3 1 "phase" Prelude.False
 
-(+%^=#$+) = T.mkVariable tComplex 370012 26 2 "+" Prelude.False
+(+%^=#$=%^=#$+) = T.mkVariable tComplex 370012 370012 26 2 "+" Prelude.False
 
-(+%@=#$-) = T.mkVariable tComplex 380012 26 2 "-" Prelude.False
+(+%@=#$=%@=#$-) = T.mkVariable tComplex 380012 380012 26 2 "-" Prelude.False
 
-(+%>=#$*) = T.mkVariable tComplex 390012 30 2 "*" Prelude.False
+(+%>=#$=%>=#$*) = T.mkVariable tComplex 390012 390012 30 2 "*" Prelude.False
 
-a40v5negate = T.mkVariable tComplex 400005 3 1 "negate" Prelude.False
+a40v5v40v41negate =
+  T.mkVariable tComplex 400005 400041 3 1 "negate" Prelude.False
 
-a41v5abs = T.mkVariable tComplex 410005 3 1 "abs" Prelude.False
+a41v5v41v29abs = T.mkVariable tComplex 410005 410029 3 1 "abs" Prelude.False
 
-a42v5signum = T.mkVariable tComplex 420005 3 1 "signum" Prelude.False
+a42v5v43v56signum =
+  T.mkVariable tComplex 420005 430056 3 1 "signum" Prelude.False
 
-a44v5fromInteger = T.mkVariable tComplex 440005 3 1 "fromInteger" Prelude.False
+a44v5v44v39fromInteger =
+  T.mkVariable tComplex 440005 440039 3 1 "fromInteger" Prelude.False
 
-(+&^=#!/) = T.mkVariable tComplex 470010 30 2 "/" Prelude.False
+(+&^=#!=&^=#!/) = T.mkVariable tComplex 470010 470010 30 2 "/" Prelude.False
 
-a54v3fromRational =
-  T.mkVariable tComplex 540003 3 1 "fromRational" Prelude.False
+a54v3v54v39fromRational =
+  T.mkVariable tComplex 540003 540039 3 1 "fromRational" Prelude.False
 
-a57v5pi = T.mkVariable tComplex 570005 3 0 "pi" Prelude.False
+a57v5v57v29pi = T.mkVariable tComplex 570005 570029 3 0 "pi" Prelude.False
 
-a58v5exp = T.mkVariable tComplex 580005 3 1 "exp" Prelude.False
+a58v5v59v40exp = T.mkVariable tComplex 580005 590040 3 1 "exp" Prelude.False
 
-a60v5log = T.mkVariable tComplex 600005 3 1 "log" Prelude.False
+a60v5v60v50log = T.mkVariable tComplex 600005 600050 3 1 "log" Prelude.False
 
-a62v5sqrt = T.mkVariable tComplex 620005 3 1 "sqrt" Prelude.False
+a62v5v66v67sqrt = T.mkVariable tComplex 620005 660067 3 1 "sqrt" Prelude.False
 
-a68v5sin = T.mkVariable tComplex 680005 3 1 "sin" Prelude.False
+a68v5v68v54sin = T.mkVariable tComplex 680005 680054 3 1 "sin" Prelude.False
 
-a69v5cos = T.mkVariable tComplex 690005 3 1 "cos" Prelude.False
+a69v5v69v57cos = T.mkVariable tComplex 690005 690057 3 1 "cos" Prelude.False
 
-a70v5tan = T.mkVariable tComplex 700005 3 1 "tan" Prelude.False
+a70v5v74v42tan = T.mkVariable tComplex 700005 740042 3 1 "tan" Prelude.False
 
-a76v5sinh = T.mkVariable tComplex 760005 3 1 "sinh" Prelude.False
+a76v5v76v55sinh = T.mkVariable tComplex 760005 760055 3 1 "sinh" Prelude.False
 
-a77v5cosh = T.mkVariable tComplex 770005 3 1 "cosh" Prelude.False
+a77v5v77v54cosh = T.mkVariable tComplex 770005 770054 3 1 "cosh" Prelude.False
 
-a78v5tanh = T.mkVariable tComplex 780005 3 1 "tanh" Prelude.False
+a78v5v82v42tanh = T.mkVariable tComplex 780005 820042 3 1 "tanh" Prelude.False
 
-a84v5asin = T.mkVariable tComplex 840005 3 1 "asin" Prelude.False
+a84v5v85v70asin = T.mkVariable tComplex 840005 850070 3 1 "asin" Prelude.False
 
-a86v5acos = T.mkVariable tComplex 860005 3 1 "acos" Prelude.False
+a86v5v88v54acos = T.mkVariable tComplex 860005 880054 3 1 "acos" Prelude.False
 
-a89v5atan = T.mkVariable tComplex 890005 3 1 "atan" Prelude.False
+a89v5v90v68atan = T.mkVariable tComplex 890005 900068 3 1 "atan" Prelude.False
 
-a92v5asinh = T.mkVariable tComplex 920005 3 1 "asinh" Prelude.False
+a92v5v92v42asinh = T.mkVariable tComplex 920005 920042 3 1 "asinh" Prelude.False
 
-a93v5acosh = T.mkVariable tComplex 930005 3 1 "acosh" Prelude.False
+a93v5v93v55acosh = T.mkVariable tComplex 930005 930055 3 1 "acosh" Prelude.False
 
-a94v5atanh = T.mkVariable tComplex 940005 3 1 "atanh" Prelude.False
+a94v5v94v46atanh = T.mkVariable tComplex 940005 940046 3 1 "atanh" Prelude.False
 
-a28v11k = T.mkVariable tComplex 280011 3 0 "k" Prelude.True
+a28v11v28v43k = T.mkVariable tComplex 280011 280043 3 0 "k" Prelude.True
 
-a29v11mk = T.mkVariable tComplex 290011 3 0 "mk" Prelude.True
+a29v11v29v18mk = T.mkVariable tComplex 290011 290018 3 0 "mk" Prelude.True
 
-a43v42r = T.mkVariable tComplex 430042 3 0 "r" Prelude.True
+a43v42v43v56r = T.mkVariable tComplex 430042 430056 3 0 "r" Prelude.True
 
-a49v5x'' = T.mkVariable tComplex 490005 3 0 "x''" Prelude.True
+a49v5v49v25x'' = T.mkVariable tComplex 490005 490025 3 0 "x''" Prelude.True
 
-a50v5y'' = T.mkVariable tComplex 500005 3 0 "y''" Prelude.True
+a50v5v50v25y'' = T.mkVariable tComplex 500005 500025 3 0 "y''" Prelude.True
 
-a51v5k = T.mkVariable tComplex 510005 3 0 "k" Prelude.True
+a51v5v51v42k = T.mkVariable tComplex 510005 510042 3 0 "k" Prelude.True
 
-a52v5d = T.mkVariable tComplex 520005 3 0 "d" Prelude.True
+a52v5v52v25d = T.mkVariable tComplex 520005 520025 3 0 "d" Prelude.True
 
-a59v29expx = T.mkVariable tComplex 590029 3 0 "expx" Prelude.True
+a59v29v59v40expx = T.mkVariable tComplex 590029 590040 3 0 "expx" Prelude.True
 
-a64v30u = T.mkVariable tComplex 640030 3 0 "u" Prelude.True
+a64v30v64v30u = T.mkVariable tComplex 640030 640030 3 0 "u" Prelude.True
 
-a64v32v = T.mkVariable tComplex 640032 3 0 "v" Prelude.True
+a64v32v64v32v = T.mkVariable tComplex 640032 640032 3 0 "v" Prelude.True
 
-a65v29v' = T.mkVariable tComplex 650029 3 0 "v'" Prelude.True
+a65v29v65v49v' = T.mkVariable tComplex 650029 650049 3 0 "v'" Prelude.True
 
-a66v29u' = T.mkVariable tComplex 660029 3 0 "u'" Prelude.True
+a66v29v66v67u' = T.mkVariable tComplex 660029 660067 3 0 "u'" Prelude.True
 
-a71v29sinx = T.mkVariable tComplex 710029 3 0 "sinx" Prelude.True
+a71v29v71v41sinx = T.mkVariable tComplex 710029 710041 3 0 "sinx" Prelude.True
 
-a72v29cosx = T.mkVariable tComplex 720029 3 0 "cosx" Prelude.True
+a72v29v72v41cosx = T.mkVariable tComplex 720029 720041 3 0 "cosx" Prelude.True
 
-a73v29sinhy = T.mkVariable tComplex 730029 3 0 "sinhy" Prelude.True
+a73v29v73v42sinhy = T.mkVariable tComplex 730029 730042 3 0 "sinhy" Prelude.True
 
-a74v29coshy = T.mkVariable tComplex 740029 3 0 "coshy" Prelude.True
+a74v29v74v42coshy = T.mkVariable tComplex 740029 740042 3 0 "coshy" Prelude.True
 
-a79v29siny = T.mkVariable tComplex 790029 3 0 "siny" Prelude.True
+a79v29v79v41siny = T.mkVariable tComplex 790029 790041 3 0 "siny" Prelude.True
 
-a80v29cosy = T.mkVariable tComplex 800029 3 0 "cosy" Prelude.True
+a80v29v80v41cosy = T.mkVariable tComplex 800029 800041 3 0 "cosy" Prelude.True
 
-a81v29sinhx = T.mkVariable tComplex 810029 3 0 "sinhx" Prelude.True
+a81v29v81v42sinhx = T.mkVariable tComplex 810029 810042 3 0 "sinhx" Prelude.True
 
-a82v29coshx = T.mkVariable tComplex 820029 3 0 "coshx" Prelude.True
+a82v29v82v42coshx = T.mkVariable tComplex 820029 820042 3 0 "coshx" Prelude.True
 
-a85v31x' = T.mkVariable tComplex 850031 3 0 "x'" Prelude.True
+a85v31v85v32x' = T.mkVariable tComplex 850031 850032 3 0 "x'" Prelude.True
 
-a85v35y' = T.mkVariable tComplex 850035 3 0 "y'" Prelude.True
+a85v35v85v36y' = T.mkVariable tComplex 850035 850036 3 0 "y'" Prelude.True
 
-a87v30x'' = T.mkVariable tComplex 870030 3 0 "x''" Prelude.True
+a87v30v87v32x'' = T.mkVariable tComplex 870030 870032 3 0 "x''" Prelude.True
 
-a87v35y'' = T.mkVariable tComplex 870035 3 0 "y''" Prelude.True
+a87v35v87v37y'' = T.mkVariable tComplex 870035 870037 3 0 "y''" Prelude.True
 
-a88v30x' = T.mkVariable tComplex 880030 3 0 "x'" Prelude.True
+a88v30v88v31x' = T.mkVariable tComplex 880030 880031 3 0 "x'" Prelude.True
 
-a88v34y' = T.mkVariable tComplex 880034 3 0 "y'" Prelude.True
+a88v34v88v35y' = T.mkVariable tComplex 880034 880035 3 0 "y'" Prelude.True
 
-a90v30x' = T.mkVariable tComplex 900030 3 0 "x'" Prelude.True
+a90v30v90v31x' = T.mkVariable tComplex 900030 900031 3 0 "x'" Prelude.True
 
-a90v34y' = T.mkVariable tComplex 900034 3 0 "y'" Prelude.True
+a90v34v90v35y' = T.mkVariable tComplex 900034 900035 3 0 "y'" Prelude.True
