@@ -4,9 +4,11 @@ module Hat.Data.Array
     ,helems,gassocs,aassocs,hassocs,gaccumArray,aaccumArray,haccumArray,(!//)
     ,(+//),(*//),gaccum,aaccum,haccum,gixmap,aixmap,hixmap) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 import Hat.Data.Ix 
 import Hat.Array 
