@@ -4,9 +4,11 @@ module Hat.Data.Complex
     ,acis,hcis,gpolar,apolar,hpolar,gmagnitude,amagnitude,hmagnitude,gphase
     ,aphase,hphase) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 
 data RealFloat a => Complex a = (:+) !(T.R a) !(T.R a)
@@ -14,7 +16,7 @@ data RealFloat a => Complex a = (:+) !(T.R a) !(T.R a)
 instance RealFloat a => T.WrapVal ((Complex a))
   where
   
-  T.wrapVal pwrapVal (kwrapVal@((T.R _ z1wrapVal) :+ (T.R _ z2wrapVal))) p =
+  wrapVal pwrapVal (kwrapVal@((T.R _ z1wrapVal) :+ (T.R _ z2wrapVal))) p =
     T.R kwrapVal (T.mkValueApp2 p pwrapVal (+:+) z1wrapVal z2wrapVal)
   
 
@@ -22,7 +24,7 @@ instance (RealFloat a,Eq a) => Eq ((Complex a))
   where
   
   (!==) (%==) p =
-    T.ufun2 (+%$=+!==) (%==) p (*==)
+    T.ufun2 (+%$=+!=%$=+#==) (%==) p (*==)
     where
     
     (*==) (T.R (fy1 :+ fy2) _) (T.R (fy3 :+ fy4) _) p =
@@ -38,7 +40,7 @@ instance (RealFloat a,Read a) => Read ((Complex a))
   where
   
   greadsPrec preadsPrec p =
-    T.ufun1 a32v64readsPrec preadsPrec p hreadsPrec
+    T.ufun1 a32v64v32v67readsPrec preadsPrec p hreadsPrec
     where
     
     hreadsPrec fy1 p =
@@ -71,7 +73,7 @@ instance (RealFloat a,Show a) => Show ((Complex a))
   where
   
   gshowsPrec pshowsPrec p =
-    T.ufun2 a32v70showsPrec pshowsPrec p hshowsPrec
+    T.ufun2 a32v70v32v73showsPrec pshowsPrec p hshowsPrec
     where
     
     hshowsPrec fy1 (T.R (fy2 :+ fy3) _) p =
@@ -195,7 +197,7 @@ hmagnitude (T.R (fx :+ fy) _) p =
   gk pk p = T.uconstUse pk p sk
   
   sk =
-    T.uconstDef p a57v27k
+    T.uconstDef p a57v27v57v59k
       (\ p ->
         T.uap2 T.mkNoSrcPos p (gmax T.mkNoSrcPos p)
           (T.uap1 T.mkNoSrcPos p (gexponent T.mkNoSrcPos p) fx)
@@ -204,7 +206,7 @@ hmagnitude (T.R (fx :+ fy) _) p =
   gmk pmk p = T.uconstUse pmk p smk
   
   smk =
-    T.uconstDef p a58v27mk
+    T.uconstDef p a58v27v58v34mk
       (\ p ->
         T.uap1 T.mkNoSrcPos p (Hat.Prelude.gnegate T.mkNoSrcPos p)
           (gk T.mkNoSrcPos p))
@@ -217,16 +219,17 @@ hphase :: RealFloat a => (T.R (Complex a)) -> T.RefExp -> T.R a
 
 gphase pphase p = T.ufun1 aphase pphase p hphase
 
-hphase (z1phase@(T.R (fv61v8n :+ v61v13n) _)) p =
+hphase (z1phase@(T.R (fv61v8v61v8n :+ v61v13v61v13n) _)) p =
   T.ucguard
-    (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv61v8n
+    (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv61v8v61v8n
       (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-        (T.conInteger T.mkNoSrcPos p 0))) (h v61v13n p) (y1phase z1phase p)
+        (T.conInteger T.mkNoSrcPos p 0))) (h v61v13v61v13n p)
+    (y1phase z1phase p)
   where
   
-  h fv61v13n p =
+  h fv61v13v61v13n p =
     T.ucguard
-      (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv61v13n
+      (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv61v13v61v13n
         (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
           (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1phase z1phase p)
     where
@@ -248,7 +251,7 @@ instance RealFloat a => Num ((Complex a))
   where
   
   (!+) (%+) p =
-    T.ufun2 (++>=#$+) (%+) p (*+)
+    T.ufun2 (++>=#$=+>=#$+) (%+) p (*+)
     where
     
     (*+) (T.R (fx :+ fy) _) (T.R (fx' :+ fy') _) p =
@@ -259,7 +262,7 @@ instance RealFloat a => Num ((Complex a))
     
   
   (!-) (%-) p =
-    T.ufun2 (+^!=#$-) (%-) p (*-)
+    T.ufun2 (+^!=#$=^!=#$-) (%-) p (*-)
     where
     
     (*-) (T.R (fx :+ fy) _) (T.R (fx' :+ fy') _) p =
@@ -270,7 +273,7 @@ instance RealFloat a => Num ((Complex a))
     
   
   (!*) (%*) p =
-    T.ufun2 (+^#=#$*) (%*) p (**)
+    T.ufun2 (+^#=#$=^#=#$*) (%*) p (**)
     where
     
     (**) (T.R (fx :+ fy) _) (T.R (fx' :+ fy') _) p =
@@ -285,7 +288,7 @@ instance RealFloat a => Num ((Complex a))
     
   
   gnegate pnegate p =
-    T.ufun1 a72v5negate pnegate p hnegate
+    T.ufun1 a72v5v72v47negate pnegate p hnegate
     where
     
     hnegate (T.R (fx :+ fy) _) p =
@@ -296,7 +299,7 @@ instance RealFloat a => Num ((Complex a))
     
   
   gabs pabs p =
-    T.ufun1 a73v5abs pabs p habs
+    T.ufun1 a73v5v73v43abs pabs p habs
     where
     
     habs fz p =
@@ -306,22 +309,22 @@ instance RealFloat a => Num ((Complex a))
     
   
   gsignum psignum p =
-    T.ufun1 a74v5signum psignum p hsignum
+    T.ufun1 a74v5v75v60signum psignum p hsignum
     where
     
-    hsignum fv74v12n p =
+    hsignum fv74v12v74v12n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv74v12n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv74v12v74v12n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1signum fv74v12n p)
+            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1signum fv74v12v74v12n p)
       where
       
       h p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
           (T.conInteger T.mkNoSrcPos p 0)
-      h p = y1signum fv74v12n p
+      h p = y1signum fv74v12v74v12n p
       
-    hsignum fv74v12n p = y1signum fv74v12n p
+    hsignum fv74v12v74v12n p = y1signum fv74v12v74v12n p
     
     y1signum (fz@(T.R (fx :+ fy) _)) p =
       T.con2 T.mkNoSrcPos p (:+) (+:+)
@@ -331,13 +334,14 @@ instance RealFloat a => Num ((Complex a))
       
       gr pr p = T.uconstUse pr p sr
       
-      sr = T.uconstDef p a75v46r (\ p -> T.uwrapForward p (hmagnitude fz p))
+      sr =
+        T.uconstDef p a75v46v75v60r (\ p -> T.uwrapForward p (hmagnitude fz p))
       
     y1signum _ p = T.fatal p
     
   
   gfromInteger pfromInteger p =
-    T.ufun1 a76v5fromInteger pfromInteger p hfromInteger
+    T.ufun1 a76v5v76v45fromInteger pfromInteger p hfromInteger
     where
     
     hfromInteger fn p =
@@ -352,7 +356,7 @@ instance RealFloat a => Fractional ((Complex a))
   where
   
   (!/) (%/) p =
-    T.ufun2 (+^>=#$/) (%/) p (*/)
+    T.ufun2 (+^>=#$=^>=#$/) (%/) p (*/)
     where
     
     (*/) (T.R (fx :+ fy) _) (T.R (fx' :+ fy') _) p =
@@ -372,7 +376,7 @@ instance RealFloat a => Fractional ((Complex a))
       gx'' px'' p = T.uconstUse px'' p sx''
       
       sx'' =
-        T.uconstDef p a80v34x''
+        T.uconstDef p a80v34v80v54x''
           (\ p ->
             T.uap2 T.mkNoSrcPos p (gscaleFloat T.mkNoSrcPos p)
               (gk T.mkNoSrcPos p) fx')
@@ -380,7 +384,7 @@ instance RealFloat a => Fractional ((Complex a))
       gy'' py'' p = T.uconstUse py'' p sy''
       
       sy'' =
-        T.uconstDef p a81v34y''
+        T.uconstDef p a81v34v81v54y''
           (\ p ->
             T.uap2 T.mkNoSrcPos p (gscaleFloat T.mkNoSrcPos p)
               (gk T.mkNoSrcPos p) fy')
@@ -388,7 +392,7 @@ instance RealFloat a => Fractional ((Complex a))
       gk pk p = T.uconstUse pk p sk
       
       sk =
-        T.uconstDef p a82v34k
+        T.uconstDef p a82v34v82v71k
           (\ p ->
             T.uap1 T.mkNoSrcPos p (Hat.Prelude.gnegate T.mkNoSrcPos p)
               (T.uap2 T.mkNoSrcPos p (gmax T.mkNoSrcPos p)
@@ -398,7 +402,7 @@ instance RealFloat a => Fractional ((Complex a))
       gd pd p = T.uconstUse pd p sd
       
       sd =
-        T.uconstDef p a83v34d
+        T.uconstDef p a83v34v83v54d
           (\ p ->
             T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p)
               (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !* p) fx'
@@ -410,7 +414,7 @@ instance RealFloat a => Fractional ((Complex a))
     
   
   gfromRational pfromRational p =
-    T.ufun1 a85v5fromRational pfromRational p hfromRational
+    T.ufun1 a85v5v85v46fromRational pfromRational p hfromRational
     where
     
     hfromRational fa p =
@@ -427,14 +431,14 @@ instance RealFloat a => Floating ((Complex a))
   gpi ppi p = T.uconstUse ppi p spi
   
   spi =
-    T.uconstDef T.mkRoot a88v5pi
+    T.uconstDef T.mkRoot a88v5v88v29pi
       (\ p ->
         T.con2 T.mkNoSrcPos p (:+) (+:+) (gpi T.mkNoSrcPos p)
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
             (T.conInteger T.mkNoSrcPos p 0)))
   
   gexp pexp p =
-    T.ufun1 a89v5exp pexp p hexp
+    T.ufun1 a89v5v90v40exp pexp p hexp
     where
     
     hexp (T.R (fx :+ fy) _) p =
@@ -448,14 +452,14 @@ instance RealFloat a => Floating ((Complex a))
       gexpx pexpx p = T.uconstUse pexpx p sexpx
       
       sexpx =
-        T.uconstDef p a90v29expx
+        T.uconstDef p a90v29v90v40expx
           (\ p -> T.uap1 T.mkNoSrcPos p (gexp T.mkNoSrcPos p) fx)
       
     hexp _ p = T.fatal p
     
   
   glog plog p =
-    T.ufun1 a91v5log plog p hlog
+    T.ufun1 a91v5v91v50log plog p hlog
     where
     
     hlog fz p =
@@ -465,22 +469,22 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gsqrt psqrt p =
-    T.ufun1 a93v5sqrt psqrt p hsqrt
+    T.ufun1 a93v5v97v67sqrt psqrt p hsqrt
     where
     
-    hsqrt fv93v10n p =
+    hsqrt fv93v10v93v10n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv93v10n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv93v10v93v10n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1sqrt fv93v10n p)
+            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1sqrt fv93v10v93v10n p)
       where
       
       h p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
           (T.conInteger T.mkNoSrcPos p 0)
-      h p = y1sqrt fv93v10n p
+      h p = y1sqrt fv93v10v93v10n p
       
-    hsqrt fv93v10n p = y1sqrt fv93v10n p
+    hsqrt fv93v10v93v10n p = y1sqrt fv93v10v93v10n p
     
     y1sqrt (fz@(T.R (fx :+ fy) _)) p =
       T.con2 T.mkNoSrcPos p (:+) (+:+) (gu T.mkNoSrcPos p)
@@ -497,7 +501,7 @@ instance RealFloat a => Floating ((Complex a))
       
       gv pu p = T.uconstUse pu p sv
       
-      j95v29u =
+      j95v29v95v33u =
         case
           T.ucif p
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !< p) fx
@@ -511,14 +515,18 @@ instance RealFloat a => Floating ((Complex a))
           T.R (T.Tuple2 fu fv) ku -> (ku,fu,fv)
           _ -> T.fatal p
       
-      su = T.uconstDef p a95v30u (\ _ -> case j95v29u of (ku,fu,fv) -> fu)
+      su =
+        T.uconstDef p a95v30v95v30u
+          (\ _ -> case j95v29v95v33u of (ku,fu,fv) -> fu)
       
-      sv = T.uconstDef p a95v32v (\ _ -> case j95v29u of (ku,fu,fv) -> fv)
+      sv =
+        T.uconstDef p a95v32v95v32v
+          (\ _ -> case j95v29v95v33u of (ku,fu,fv) -> fv)
       
       gv' pv' p = T.uconstUse pv' p sv'
       
       sv' =
-        T.uconstDef p a96v29v'
+        T.uconstDef p a96v29v96v49v'
           (\ p ->
             T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !/ p)
               (T.uap1 T.mkNoSrcPos p (gabs T.mkNoSrcPos p) fy)
@@ -530,7 +538,7 @@ instance RealFloat a => Floating ((Complex a))
       gu' pu' p = T.uconstUse pu' p su'
       
       su' =
-        T.uconstDef p a97v29u'
+        T.uconstDef p a97v29v97v67u'
           (\ p ->
             T.uap1 T.mkNoSrcPos p (gsqrt T.mkNoSrcPos p)
               (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !/ p)
@@ -545,7 +553,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gsin psin p =
-    T.ufun1 a99v5sin psin p hsin
+    T.ufun1 a99v5v99v54sin psin p hsin
     where
     
     hsin (T.R (fx :+ fy) _) p =
@@ -560,7 +568,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gcos pcos p =
-    T.ufun1 a100v5cos pcos p hcos
+    T.ufun1 a100v5v100v57cos pcos p hcos
     where
     
     hcos (T.R (fx :+ fy) _) p =
@@ -576,7 +584,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gtan ptan p =
-    T.ufun1 a101v5tan ptan p htan
+    T.ufun1 a101v5v105v42tan ptan p htan
     where
     
     htan (T.R (fx :+ fy) _) p =
@@ -597,32 +605,32 @@ instance RealFloat a => Floating ((Complex a))
       gsinx psinx p = T.uconstUse psinx p ssinx
       
       ssinx =
-        T.uconstDef p a102v29sinx
+        T.uconstDef p a102v29v102v41sinx
           (\ p -> T.uap1 T.mkNoSrcPos p (gsin T.mkNoSrcPos p) fx)
       
       gcosx pcosx p = T.uconstUse pcosx p scosx
       
       scosx =
-        T.uconstDef p a103v29cosx
+        T.uconstDef p a103v29v103v41cosx
           (\ p -> T.uap1 T.mkNoSrcPos p (gcos T.mkNoSrcPos p) fx)
       
       gsinhy psinhy p = T.uconstUse psinhy p ssinhy
       
       ssinhy =
-        T.uconstDef p a104v29sinhy
+        T.uconstDef p a104v29v104v42sinhy
           (\ p -> T.uap1 T.mkNoSrcPos p (gsinh T.mkNoSrcPos p) fy)
       
       gcoshy pcoshy p = T.uconstUse pcoshy p scoshy
       
       scoshy =
-        T.uconstDef p a105v29coshy
+        T.uconstDef p a105v29v105v42coshy
           (\ p -> T.uap1 T.mkNoSrcPos p (gcosh T.mkNoSrcPos p) fy)
       
     htan _ p = T.fatal p
     
   
   gsinh psinh p =
-    T.ufun1 a107v5sinh psinh p hsinh
+    T.ufun1 a107v5v107v55sinh psinh p hsinh
     where
     
     hsinh (T.R (fx :+ fy) _) p =
@@ -637,7 +645,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gcosh pcosh p =
-    T.ufun1 a108v5cosh pcosh p hcosh
+    T.ufun1 a108v5v108v54cosh pcosh p hcosh
     where
     
     hcosh (T.R (fx :+ fy) _) p =
@@ -652,7 +660,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gtanh ptanh p =
-    T.ufun1 a109v5tanh ptanh p htanh
+    T.ufun1 a109v5v113v42tanh ptanh p htanh
     where
     
     htanh (T.R (fx :+ fy) _) p =
@@ -672,32 +680,32 @@ instance RealFloat a => Floating ((Complex a))
       gsiny psiny p = T.uconstUse psiny p ssiny
       
       ssiny =
-        T.uconstDef p a110v29siny
+        T.uconstDef p a110v29v110v41siny
           (\ p -> T.uap1 T.mkNoSrcPos p (gsin T.mkNoSrcPos p) fy)
       
       gcosy pcosy p = T.uconstUse pcosy p scosy
       
       scosy =
-        T.uconstDef p a111v29cosy
+        T.uconstDef p a111v29v111v41cosy
           (\ p -> T.uap1 T.mkNoSrcPos p (gcos T.mkNoSrcPos p) fy)
       
       gsinhx psinhx p = T.uconstUse psinhx p ssinhx
       
       ssinhx =
-        T.uconstDef p a112v29sinhx
+        T.uconstDef p a112v29v112v42sinhx
           (\ p -> T.uap1 T.mkNoSrcPos p (gsinh T.mkNoSrcPos p) fx)
       
       gcoshx pcoshx p = T.uconstUse pcoshx p scoshx
       
       scoshx =
-        T.uconstDef p a113v29coshx
+        T.uconstDef p a113v29v113v42coshx
           (\ p -> T.uap1 T.mkNoSrcPos p (gcosh T.mkNoSrcPos p) fx)
       
     htanh _ p = T.fatal p
     
   
   gasin pasin p =
-    T.ufun1 a115v5asin pasin p hasin
+    T.ufun1 a115v5v116v70asin pasin p hasin
     where
     
     hasin (fz@(T.R (fx :+ fy) _)) p =
@@ -710,13 +718,13 @@ instance RealFloat a => Floating ((Complex a))
       
       gy' px' p = T.uconstUse px' p sy'
       
-      j116v33x' =
+      j116v31v116v36x' =
         case
           T.uap1 T.mkNoSrcPos p (glog T.mkNoSrcPos p)
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p)
               (T.con2 T.mkNoSrcPos p (:+) (+:+)
                 (T.uap1 T.mkNoSrcPos p (Hat.Prelude.gnegate T.mkNoSrcPos p) fy)
-                fx)
+                (T.projection T.mkNoSrcPos p fx))
               (T.uap1 T.mkNoSrcPos p (gsqrt T.mkNoSrcPos p)
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !- p)
                   (T.uap1 T.mkNoSrcPos p
@@ -727,16 +735,18 @@ instance RealFloat a => Floating ((Complex a))
           _ -> T.fatal p
       
       sx' =
-        T.uconstDef p a116v31x' (\ _ -> case j116v33x' of (kx',fx',fy') -> fx')
+        T.uconstDef p a116v31v116v32x'
+          (\ _ -> case j116v31v116v36x' of (kx',fx',fy') -> fx')
       
       sy' =
-        T.uconstDef p a116v35y' (\ _ -> case j116v33x' of (kx',fx',fy') -> fy')
+        T.uconstDef p a116v35v116v36y'
+          (\ _ -> case j116v31v116v36x' of (kx',fx',fy') -> fy')
       
     hasin _ p = T.fatal p
     
   
   gacos pacos p =
-    T.ufun1 a117v5acos pacos p hacos
+    T.ufun1 a117v5v119v54acos pacos p hacos
     where
     
     hacos fz p =
@@ -749,7 +759,7 @@ instance RealFloat a => Floating ((Complex a))
       
       gy'' px'' p = T.uconstUse px'' p sy''
       
-      j118v33x'' =
+      j118v30v118v37x'' =
         case
           T.uap1 T.mkNoSrcPos p (glog T.mkNoSrcPos p)
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p) fz
@@ -760,18 +770,18 @@ instance RealFloat a => Floating ((Complex a))
           _ -> T.fatal p
       
       sx'' =
-        T.uconstDef p a118v30x''
-          (\ _ -> case j118v33x'' of (kx'',fx'',fy'') -> fx'')
+        T.uconstDef p a118v30v118v32x''
+          (\ _ -> case j118v30v118v37x'' of (kx'',fx'',fy'') -> fx'')
       
       sy'' =
-        T.uconstDef p a118v35y''
-          (\ _ -> case j118v33x'' of (kx'',fx'',fy'') -> fy'')
+        T.uconstDef p a118v35v118v37y''
+          (\ _ -> case j118v30v118v37x'' of (kx'',fx'',fy'') -> fy'')
       
       gx' px' p = T.uconstUse px' p sx'
       
       gy' px' p = T.uconstUse px' p sy'
       
-      j119v32x' =
+      j119v30v119v35x' =
         case
           T.uap1 T.mkNoSrcPos p (gsqrt T.mkNoSrcPos p)
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !- p)
@@ -783,15 +793,17 @@ instance RealFloat a => Floating ((Complex a))
           _ -> T.fatal p
       
       sx' =
-        T.uconstDef p a119v30x' (\ _ -> case j119v32x' of (kx',fx',fy') -> fx')
+        T.uconstDef p a119v30v119v31x'
+          (\ _ -> case j119v30v119v35x' of (kx',fx',fy') -> fx')
       
       sy' =
-        T.uconstDef p a119v34y' (\ _ -> case j119v32x' of (kx',fx',fy') -> fy')
+        T.uconstDef p a119v34v119v35y'
+          (\ _ -> case j119v30v119v35x' of (kx',fx',fy') -> fy')
       
     
   
   gatan patan p =
-    T.ufun1 a120v5atan patan p hatan
+    T.ufun1 a120v5v121v68atan patan p hatan
     where
     
     hatan (fz@(T.R (fx :+ fy) _)) p =
@@ -804,7 +816,7 @@ instance RealFloat a => Floating ((Complex a))
       
       gy' px' p = T.uconstUse px' p sy'
       
-      j121v32x' =
+      j121v30v121v35x' =
         case
           T.uap1 T.mkNoSrcPos p (glog T.mkNoSrcPos p)
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !/ p)
@@ -812,7 +824,8 @@ instance RealFloat a => Floating ((Complex a))
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !- p)
                   (T.uap1 T.mkNoSrcPos p
                     (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-                    (T.conInteger T.mkNoSrcPos p 1)) fy) fx)
+                    (T.conInteger T.mkNoSrcPos p 1)) fy)
+                (T.projection T.mkNoSrcPos p fx))
               (T.uap1 T.mkNoSrcPos p (gsqrt T.mkNoSrcPos p)
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p)
                   (T.uap1 T.mkNoSrcPos p
@@ -823,16 +836,18 @@ instance RealFloat a => Floating ((Complex a))
           _ -> T.fatal p
       
       sx' =
-        T.uconstDef p a121v30x' (\ _ -> case j121v32x' of (kx',fx',fy') -> fx')
+        T.uconstDef p a121v30v121v31x'
+          (\ _ -> case j121v30v121v35x' of (kx',fx',fy') -> fx')
       
       sy' =
-        T.uconstDef p a121v34y' (\ _ -> case j121v32x' of (kx',fx',fy') -> fy')
+        T.uconstDef p a121v34v121v35y'
+          (\ _ -> case j121v30v121v35x' of (kx',fx',fy') -> fy')
       
     hatan _ p = T.fatal p
     
   
   gasinh pasinh p =
-    T.ufun1 a123v5asinh pasinh p hasinh
+    T.ufun1 a123v5v123v42asinh pasinh p hasinh
     where
     
     hasinh fz p =
@@ -847,7 +862,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gacosh pacosh p =
-    T.ufun1 a124v5acosh pacosh p hacosh
+    T.ufun1 a124v5v124v55acosh pacosh p hacosh
     where
     
     hacosh fz p =
@@ -871,7 +886,7 @@ instance RealFloat a => Floating ((Complex a))
     
   
   gatanh patanh p =
-    T.ufun1 a125v5atanh patanh p hatanh
+    T.ufun1 a125v5v125v46atanh patanh p hatanh
     where
     
     hatanh fz p =
@@ -892,136 +907,182 @@ instance RealFloat a => Floating ((Complex a))
 
 tData_Complex = T.mkModule "Data.Complex" "Data/Complex.hs" Prelude.False
 
-(+:+) = T.mkConstructor tData_Complex 320043 24 2 ":+"
+(+:+) = T.mkConstructor tData_Complex 320043 320044 24 2 ":+"
 
-(+%$=+!==) = T.mkVariable tData_Complex 320060 3 2 "==" Prelude.False
+(+%$=+!=%$=+#==) =
+  T.mkVariable tData_Complex 320060 320061 3 2 "==" Prelude.False
 
-a32v64readsPrec =
-  T.mkVariable tData_Complex 320064 3 1 "readsPrec" Prelude.False
+a32v64v32v67readsPrec =
+  T.mkVariable tData_Complex 320064 320067 3 1 "readsPrec" Prelude.False
 
-a32v70showsPrec =
-  T.mkVariable tData_Complex 320070 3 2 "showsPrec" Prelude.False
+a32v70v32v73showsPrec =
+  T.mkVariable tData_Complex 320070 320073 3 2 "showsPrec" Prelude.False
 
-arealPart = T.mkVariable tData_Complex 390001 3 1 "realPart" Prelude.False
+arealPart =
+  T.mkVariable tData_Complex 390001 390022 3 1 "realPart" Prelude.False
 
-aimagPart = T.mkVariable tData_Complex 400001 3 1 "imagPart" Prelude.False
+aimagPart =
+  T.mkVariable tData_Complex 400001 400022 3 1 "imagPart" Prelude.False
 
-aconjugate = T.mkVariable tData_Complex 430001 3 1 "conjugate" Prelude.False
+aconjugate =
+  T.mkVariable tData_Complex 430001 430028 3 1 "conjugate" Prelude.False
 
-amkPolar = T.mkVariable tData_Complex 460001 3 2 "mkPolar" Prelude.False
+amkPolar = T.mkVariable tData_Complex 460001 460050 3 2 "mkPolar" Prelude.False
 
-acis = T.mkVariable tData_Complex 490001 3 1 "cis" Prelude.False
+acis = T.mkVariable tData_Complex 490001 490042 3 1 "cis" Prelude.False
 
-apolar = T.mkVariable tData_Complex 520001 3 1 "polar" Prelude.False
+apolar = T.mkVariable tData_Complex 520001 520042 3 1 "polar" Prelude.False
 
-amagnitude = T.mkVariable tData_Complex 550001 3 1 "magnitude" Prelude.False
+amagnitude =
+  T.mkVariable tData_Complex 550001 580034 3 1 "magnitude" Prelude.False
 
-aphase = T.mkVariable tData_Complex 610001 3 1 "phase" Prelude.False
+aphase = T.mkVariable tData_Complex 610001 620028 3 1 "phase" Prelude.False
 
-(++>=#$+) = T.mkVariable tData_Complex 690012 26 2 "+" Prelude.False
+(++>=#$=+>=#$+) =
+  T.mkVariable tData_Complex 690012 690012 26 2 "+" Prelude.False
 
-(+^!=#$-) = T.mkVariable tData_Complex 700012 26 2 "-" Prelude.False
+(+^!=#$=^!=#$-) =
+  T.mkVariable tData_Complex 700012 700012 26 2 "-" Prelude.False
 
-(+^#=#$*) = T.mkVariable tData_Complex 710012 30 2 "*" Prelude.False
+(+^#=#$=^#=#$*) =
+  T.mkVariable tData_Complex 710012 710012 30 2 "*" Prelude.False
 
-a72v5negate = T.mkVariable tData_Complex 720005 3 1 "negate" Prelude.False
+a72v5v72v47negate =
+  T.mkVariable tData_Complex 720005 720047 3 1 "negate" Prelude.False
 
-a73v5abs = T.mkVariable tData_Complex 730005 3 1 "abs" Prelude.False
+a73v5v73v43abs =
+  T.mkVariable tData_Complex 730005 730043 3 1 "abs" Prelude.False
 
-a74v5signum = T.mkVariable tData_Complex 740005 3 1 "signum" Prelude.False
+a74v5v75v60signum =
+  T.mkVariable tData_Complex 740005 750060 3 1 "signum" Prelude.False
 
-a76v5fromInteger =
-  T.mkVariable tData_Complex 760005 3 1 "fromInteger" Prelude.False
+a76v5v76v45fromInteger =
+  T.mkVariable tData_Complex 760005 760045 3 1 "fromInteger" Prelude.False
 
-(+^>=#$/) = T.mkVariable tData_Complex 790012 30 2 "/" Prelude.False
+(+^>=#$=^>=#$/) =
+  T.mkVariable tData_Complex 790012 790012 30 2 "/" Prelude.False
 
-a85v5fromRational =
-  T.mkVariable tData_Complex 850005 3 1 "fromRational" Prelude.False
+a85v5v85v46fromRational =
+  T.mkVariable tData_Complex 850005 850046 3 1 "fromRational" Prelude.False
 
-a88v5pi = T.mkVariable tData_Complex 880005 3 0 "pi" Prelude.False
+a88v5v88v29pi = T.mkVariable tData_Complex 880005 880029 3 0 "pi" Prelude.False
 
-a89v5exp = T.mkVariable tData_Complex 890005 3 1 "exp" Prelude.False
+a89v5v90v40exp =
+  T.mkVariable tData_Complex 890005 900040 3 1 "exp" Prelude.False
 
-a91v5log = T.mkVariable tData_Complex 910005 3 1 "log" Prelude.False
+a91v5v91v50log =
+  T.mkVariable tData_Complex 910005 910050 3 1 "log" Prelude.False
 
-a93v5sqrt = T.mkVariable tData_Complex 930005 3 1 "sqrt" Prelude.False
+a93v5v97v67sqrt =
+  T.mkVariable tData_Complex 930005 970067 3 1 "sqrt" Prelude.False
 
-a99v5sin = T.mkVariable tData_Complex 990005 3 1 "sin" Prelude.False
+a99v5v99v54sin =
+  T.mkVariable tData_Complex 990005 990054 3 1 "sin" Prelude.False
 
-a100v5cos = T.mkVariable tData_Complex 1000005 3 1 "cos" Prelude.False
+a100v5v100v57cos =
+  T.mkVariable tData_Complex 1000005 1000057 3 1 "cos" Prelude.False
 
-a101v5tan = T.mkVariable tData_Complex 1010005 3 1 "tan" Prelude.False
+a101v5v105v42tan =
+  T.mkVariable tData_Complex 1010005 1050042 3 1 "tan" Prelude.False
 
-a107v5sinh = T.mkVariable tData_Complex 1070005 3 1 "sinh" Prelude.False
+a107v5v107v55sinh =
+  T.mkVariable tData_Complex 1070005 1070055 3 1 "sinh" Prelude.False
 
-a108v5cosh = T.mkVariable tData_Complex 1080005 3 1 "cosh" Prelude.False
+a108v5v108v54cosh =
+  T.mkVariable tData_Complex 1080005 1080054 3 1 "cosh" Prelude.False
 
-a109v5tanh = T.mkVariable tData_Complex 1090005 3 1 "tanh" Prelude.False
+a109v5v113v42tanh =
+  T.mkVariable tData_Complex 1090005 1130042 3 1 "tanh" Prelude.False
 
-a115v5asin = T.mkVariable tData_Complex 1150005 3 1 "asin" Prelude.False
+a115v5v116v70asin =
+  T.mkVariable tData_Complex 1150005 1160070 3 1 "asin" Prelude.False
 
-a117v5acos = T.mkVariable tData_Complex 1170005 3 1 "acos" Prelude.False
+a117v5v119v54acos =
+  T.mkVariable tData_Complex 1170005 1190054 3 1 "acos" Prelude.False
 
-a120v5atan = T.mkVariable tData_Complex 1200005 3 1 "atan" Prelude.False
+a120v5v121v68atan =
+  T.mkVariable tData_Complex 1200005 1210068 3 1 "atan" Prelude.False
 
-a123v5asinh = T.mkVariable tData_Complex 1230005 3 1 "asinh" Prelude.False
+a123v5v123v42asinh =
+  T.mkVariable tData_Complex 1230005 1230042 3 1 "asinh" Prelude.False
 
-a124v5acosh = T.mkVariable tData_Complex 1240005 3 1 "acosh" Prelude.False
+a124v5v124v55acosh =
+  T.mkVariable tData_Complex 1240005 1240055 3 1 "acosh" Prelude.False
 
-a125v5atanh = T.mkVariable tData_Complex 1250005 3 1 "atanh" Prelude.False
+a125v5v125v46atanh =
+  T.mkVariable tData_Complex 1250005 1250046 3 1 "atanh" Prelude.False
 
-a57v27k = T.mkVariable tData_Complex 570027 3 0 "k" Prelude.True
+a57v27v57v59k = T.mkVariable tData_Complex 570027 570059 3 0 "k" Prelude.True
 
-a58v27mk = T.mkVariable tData_Complex 580027 3 0 "mk" Prelude.True
+a58v27v58v34mk = T.mkVariable tData_Complex 580027 580034 3 0 "mk" Prelude.True
 
-a75v46r = T.mkVariable tData_Complex 750046 3 0 "r" Prelude.True
+a75v46v75v60r = T.mkVariable tData_Complex 750046 750060 3 0 "r" Prelude.True
 
-a80v34x'' = T.mkVariable tData_Complex 800034 3 0 "x''" Prelude.True
+a80v34v80v54x'' =
+  T.mkVariable tData_Complex 800034 800054 3 0 "x''" Prelude.True
 
-a81v34y'' = T.mkVariable tData_Complex 810034 3 0 "y''" Prelude.True
+a81v34v81v54y'' =
+  T.mkVariable tData_Complex 810034 810054 3 0 "y''" Prelude.True
 
-a82v34k = T.mkVariable tData_Complex 820034 3 0 "k" Prelude.True
+a82v34v82v71k = T.mkVariable tData_Complex 820034 820071 3 0 "k" Prelude.True
 
-a83v34d = T.mkVariable tData_Complex 830034 3 0 "d" Prelude.True
+a83v34v83v54d = T.mkVariable tData_Complex 830034 830054 3 0 "d" Prelude.True
 
-a90v29expx = T.mkVariable tData_Complex 900029 3 0 "expx" Prelude.True
+a90v29v90v40expx =
+  T.mkVariable tData_Complex 900029 900040 3 0 "expx" Prelude.True
 
-a95v30u = T.mkVariable tData_Complex 950030 3 0 "u" Prelude.True
+a95v30v95v30u = T.mkVariable tData_Complex 950030 950030 3 0 "u" Prelude.True
 
-a95v32v = T.mkVariable tData_Complex 950032 3 0 "v" Prelude.True
+a95v32v95v32v = T.mkVariable tData_Complex 950032 950032 3 0 "v" Prelude.True
 
-a96v29v' = T.mkVariable tData_Complex 960029 3 0 "v'" Prelude.True
+a96v29v96v49v' = T.mkVariable tData_Complex 960029 960049 3 0 "v'" Prelude.True
 
-a97v29u' = T.mkVariable tData_Complex 970029 3 0 "u'" Prelude.True
+a97v29v97v67u' = T.mkVariable tData_Complex 970029 970067 3 0 "u'" Prelude.True
 
-a102v29sinx = T.mkVariable tData_Complex 1020029 3 0 "sinx" Prelude.True
+a102v29v102v41sinx =
+  T.mkVariable tData_Complex 1020029 1020041 3 0 "sinx" Prelude.True
 
-a103v29cosx = T.mkVariable tData_Complex 1030029 3 0 "cosx" Prelude.True
+a103v29v103v41cosx =
+  T.mkVariable tData_Complex 1030029 1030041 3 0 "cosx" Prelude.True
 
-a104v29sinhy = T.mkVariable tData_Complex 1040029 3 0 "sinhy" Prelude.True
+a104v29v104v42sinhy =
+  T.mkVariable tData_Complex 1040029 1040042 3 0 "sinhy" Prelude.True
 
-a105v29coshy = T.mkVariable tData_Complex 1050029 3 0 "coshy" Prelude.True
+a105v29v105v42coshy =
+  T.mkVariable tData_Complex 1050029 1050042 3 0 "coshy" Prelude.True
 
-a110v29siny = T.mkVariable tData_Complex 1100029 3 0 "siny" Prelude.True
+a110v29v110v41siny =
+  T.mkVariable tData_Complex 1100029 1100041 3 0 "siny" Prelude.True
 
-a111v29cosy = T.mkVariable tData_Complex 1110029 3 0 "cosy" Prelude.True
+a111v29v111v41cosy =
+  T.mkVariable tData_Complex 1110029 1110041 3 0 "cosy" Prelude.True
 
-a112v29sinhx = T.mkVariable tData_Complex 1120029 3 0 "sinhx" Prelude.True
+a112v29v112v42sinhx =
+  T.mkVariable tData_Complex 1120029 1120042 3 0 "sinhx" Prelude.True
 
-a113v29coshx = T.mkVariable tData_Complex 1130029 3 0 "coshx" Prelude.True
+a113v29v113v42coshx =
+  T.mkVariable tData_Complex 1130029 1130042 3 0 "coshx" Prelude.True
 
-a116v31x' = T.mkVariable tData_Complex 1160031 3 0 "x'" Prelude.True
+a116v31v116v32x' =
+  T.mkVariable tData_Complex 1160031 1160032 3 0 "x'" Prelude.True
 
-a116v35y' = T.mkVariable tData_Complex 1160035 3 0 "y'" Prelude.True
+a116v35v116v36y' =
+  T.mkVariable tData_Complex 1160035 1160036 3 0 "y'" Prelude.True
 
-a118v30x'' = T.mkVariable tData_Complex 1180030 3 0 "x''" Prelude.True
+a118v30v118v32x'' =
+  T.mkVariable tData_Complex 1180030 1180032 3 0 "x''" Prelude.True
 
-a118v35y'' = T.mkVariable tData_Complex 1180035 3 0 "y''" Prelude.True
+a118v35v118v37y'' =
+  T.mkVariable tData_Complex 1180035 1180037 3 0 "y''" Prelude.True
 
-a119v30x' = T.mkVariable tData_Complex 1190030 3 0 "x'" Prelude.True
+a119v30v119v31x' =
+  T.mkVariable tData_Complex 1190030 1190031 3 0 "x'" Prelude.True
 
-a119v34y' = T.mkVariable tData_Complex 1190034 3 0 "y'" Prelude.True
+a119v34v119v35y' =
+  T.mkVariable tData_Complex 1190034 1190035 3 0 "y'" Prelude.True
 
-a121v30x' = T.mkVariable tData_Complex 1210030 3 0 "x'" Prelude.True
+a121v30v121v31x' =
+  T.mkVariable tData_Complex 1210030 1210031 3 0 "x'" Prelude.True
 
-a121v34y' = T.mkVariable tData_Complex 1210034 3 0 "y'" Prelude.True
+a121v34v121v35y' =
+  T.mkVariable tData_Complex 1210034 1210035 3 0 "y'" Prelude.True
