@@ -11,9 +11,11 @@ module Hat.Data.FiniteMap
     ,gintersectFM,aintersectFM,hintersectFM,gintersectFM_C,aintersectFM_C
     ,hintersectFM_C,gmapFM,amapFM,hmapFM,gfilterFM,afilterFM,hfilterFM) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 import Hat.Data.Maybe  (gisJust,aisJust,hisJust)
 
@@ -272,9 +274,9 @@ data FiniteMap key elt =
 instance T.WrapVal ((FiniteMap key elt))
   where
   
-  T.wrapVal pwrapVal (kwrapVal@EmptyFM) p =
+  wrapVal pwrapVal (kwrapVal@EmptyFM) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aEmptyFM)
-  T.wrapVal pwrapVal
+  wrapVal pwrapVal
     (kwrapVal@(Branch (T.R _ z1wrapVal) (T.R _ z2wrapVal) (T.R _ z3wrapVal)
         (T.R _ z4wrapVal) (T.R _ z5wrapVal))) p =
     T.R kwrapVal
@@ -319,20 +321,20 @@ haddToFM_C fcombiner (T.R (Branch fkey felt fsize ffm_l ffm_r) _) fnew_key
   fnew_elt p =
   T.uccase T.mkNoSrcPos p
     (let
-      v99v5v1 (T.R LT _) p =
+      v99v5v102v64v1 (T.R LT _) p =
         T.uwrapForward p
           (hmkBalBranch fkey felt
             (T.uwrapForward p (haddToFM_C fcombiner ffm_l fnew_key fnew_elt p))
             ffm_r p)
-      v99v5v1 (T.R GT _) p =
+      v99v5v102v64v1 (T.R GT _) p =
         T.uwrapForward p
           (hmkBalBranch fkey felt ffm_l
             (T.uwrapForward p (haddToFM_C fcombiner ffm_r fnew_key fnew_elt p))
             p)
-      v99v5v1 (T.R EQ _) p =
+      v99v5v102v64v1 (T.R EQ _) p =
         T.con5 T.mkNoSrcPos p Branch aBranch fnew_key
           (T.uap2 T.mkNoSrcPos p fcombiner felt fnew_elt) fsize ffm_l ffm_r
-      v99v5v1 _ p = T.fatal p in (v99v5v1))
+      v99v5v102v64v1 _ p = T.fatal p in (v99v5v102v64v1))
     (T.uap2 T.mkNoSrcPos p (gcompare T.mkNoSrcPos p) fnew_key fkey)
 haddToFM_C _ _ _ _ p = T.fatal p
 
@@ -352,9 +354,9 @@ haddListToFM_C fcombiner ffm fkey_elt_pairs p =
   T.uwrapForward p (hfoldl (gadd T.mkNoSrcPos p) ffm fkey_elt_pairs p)
   where
   
-  gadd padd p = T.ufun2 a108v5add padd p hadd
+  gadd padd p = T.ufun2 a108v5v108v56add padd p hadd
   
-  aadd = a108v5add
+  aadd = a108v5v108v56add
   
   hadd ffmap (T.R (T.Tuple2 fkey felt) _) p =
     T.uwrapForward p (haddToFM_C fcombiner ffmap fkey felt p)
@@ -367,16 +369,16 @@ hdelFromFM (T.R EmptyFM _) fdel_key p = gemptyFM T.mkNoSrcPos p
 hdelFromFM (T.R (Branch fkey felt fsize ffm_l ffm_r) _) fdel_key p =
   T.uccase T.mkNoSrcPos p
     (let
-      v112v5v1 (T.R GT _) p =
+      v112v5v115v31v1 (T.R GT _) p =
         T.uwrapForward p
           (hmkBalBranch fkey felt ffm_l
             (T.uwrapForward p (hdelFromFM ffm_r fdel_key p)) p)
-      v112v5v1 (T.R LT _) p =
+      v112v5v115v31v1 (T.R LT _) p =
         T.uwrapForward p
           (hmkBalBranch fkey felt
             (T.uwrapForward p (hdelFromFM ffm_l fdel_key p)) ffm_r p)
-      v112v5v1 (T.R EQ _) p = T.uwrapForward p (hglueBal ffm_l ffm_r p)
-      v112v5v1 _ p = T.fatal p in (v112v5v1))
+      v112v5v115v31v1 (T.R EQ _) p = T.uwrapForward p (hglueBal ffm_l ffm_r p)
+      v112v5v115v31v1 _ p = T.fatal p in (v112v5v115v31v1))
     (T.uap2 T.mkNoSrcPos p (gcompare T.mkNoSrcPos p) fdel_key fkey)
 hdelFromFM _ _ p = T.fatal p
 
@@ -400,26 +402,27 @@ hplusFM_C fcombiner ffm1 (T.R (Branch fsplit_key felt2 _ fleft fright) _) p =
   glts plts p = T.uconstUse plts p slts
   
   slts =
-    T.uconstDef p a126v5lts
+    T.uconstDef p a126v5v126v35lts
       (\ p -> T.uwrapForward p (hsplitLT ffm1 fsplit_key p))
   
   ggts pgts p = T.uconstUse pgts p sgts
   
   sgts =
-    T.uconstDef p a127v5gts
+    T.uconstDef p a127v5v127v35gts
       (\ p -> T.uwrapForward p (hsplitGT ffm1 fsplit_key p))
   
   gnew_elt pnew_elt p = T.uconstUse pnew_elt p snew_elt
   
   snew_elt =
-    T.uconstDef p a128v5new_elt
+    T.uconstDef p a128v5v130v47new_elt
       (\ p ->
         T.uccase T.mkNoSrcPos p
           (let
-            v128v15v1 (T.R Nothing _) p = T.projection T.mkNoSrcPos p felt2
-            v128v15v1 (T.R (Just felt1) _) p =
+            v128v15v130v47v1 (T.R Nothing _) p =
+              T.projection T.mkNoSrcPos p felt2
+            v128v15v130v47v1 (T.R (Just felt1) _) p =
               T.uap2 T.mkNoSrcPos p fcombiner felt1 felt2
-            v128v15v1 _ p = T.fatal p in (v128v15v1))
+            v128v15v130v47v1 _ p = T.fatal p in (v128v15v130v47v1))
           (T.uwrapForward p (hlookupFM ffm1 fsplit_key p)))
   
 hplusFM_C _ _ _ p = T.fatal p
@@ -438,13 +441,13 @@ hplusFM ffm1 (T.R (Branch fsplit_key felt1 _ fleft fright) _) p =
   glts plts p = T.uconstUse plts p slts
   
   slts =
-    T.uconstDef p a137v5lts
+    T.uconstDef p a137v5v137v35lts
       (\ p -> T.uwrapForward p (hsplitLT ffm1 fsplit_key p))
   
   ggts pgts p = T.uconstUse pgts p sgts
   
   sgts =
-    T.uconstDef p a138v5gts
+    T.uconstDef p a138v5v138v35gts
       (\ p -> T.uwrapForward p (hsplitGT ffm1 fsplit_key p))
   
 hplusFM _ _ p = T.fatal p
@@ -462,13 +465,13 @@ hminusFM ffm1 (T.R (Branch fsplit_key felt _ fleft fright) _) p =
   glts plts p = T.uconstUse plts p slts
   
   slts =
-    T.uconstDef p a146v5lts
+    T.uconstDef p a146v5v146v31lts
       (\ p -> T.uwrapForward p (hsplitLT ffm1 fsplit_key p))
   
   ggts pgts p = T.uconstUse pgts p sgts
   
   sgts =
-    T.uconstDef p a147v5gts
+    T.uconstDef p a147v5v147v31gts
       (\ p -> T.uwrapForward p (hsplitGT ffm1 fsplit_key p))
   
 hminusFM _ _ p = T.fatal p
@@ -509,30 +512,31 @@ hintersectFM_C fcombiner ffm1 (T.R (Branch fsplit_key felt2 _ fleft fright) _)
   glts plts p = T.uconstUse plts p slts
   
   slts =
-    T.uconstDef p a162v5lts
+    T.uconstDef p a162v5v162v31lts
       (\ p -> T.uwrapForward p (hsplitLT ffm1 fsplit_key p))
   
   ggts pgts p = T.uconstUse pgts p sgts
   
   sgts =
-    T.uconstDef p a163v5gts
+    T.uconstDef p a163v5v163v31gts
       (\ p -> T.uwrapForward p (hsplitGT ffm1 fsplit_key p))
   
   gmaybe_elt1 pmaybe_elt1 p = T.uconstUse pmaybe_elt1 p smaybe_elt1
   
   smaybe_elt1 =
-    T.uconstDef p a165v5maybe_elt1
+    T.uconstDef p a165v5v165v39maybe_elt1
       (\ p -> T.uwrapForward p (hlookupFM ffm1 fsplit_key p))
   
   gelt1 pelt1 p = T.uconstUse pelt1 p selt1
   
-  j166v5elt1 =
+  j166v5v166v13elt1 =
     case gmaybe_elt1 T.mkNoSrcPos p of
       T.R (Just felt1) kelt1 -> (kelt1,felt1)
       _ -> T.fatal p
   
   selt1 =
-    T.uconstDef p a166v10elt1 (\ _ -> case j166v5elt1 of (kelt1,felt1) -> felt1)
+    T.uconstDef p a166v10v166v13elt1
+      (\ _ -> case j166v5v166v13elt1 of (kelt1,felt1) -> felt1)
   
 hintersectFM_C _ _ _ p = T.fatal p
 
@@ -590,10 +594,12 @@ hlookupFM (T.R EmptyFM _) fkey p = T.con0 T.mkNoSrcPos p Nothing aNothing
 hlookupFM (T.R (Branch fkey felt _ ffm_l ffm_r) _) fkey_to_find p =
   T.uccase T.mkNoSrcPos p
     (let
-      v191v5v1 (T.R LT _) p = T.uwrapForward p (hlookupFM ffm_l fkey_to_find p)
-      v191v5v1 (T.R GT _) p = T.uwrapForward p (hlookupFM ffm_r fkey_to_find p)
-      v191v5v1 (T.R EQ _) p = T.con1 T.mkNoSrcPos p Just aJust felt
-      v191v5v1 _ p = T.fatal p in (v191v5v1))
+      v191v5v194v22v1 (T.R LT _) p =
+        T.uwrapForward p (hlookupFM ffm_l fkey_to_find p)
+      v191v5v194v22v1 (T.R GT _) p =
+        T.uwrapForward p (hlookupFM ffm_r fkey_to_find p)
+      v191v5v194v22v1 (T.R EQ _) p = T.con1 T.mkNoSrcPos p Just aJust felt
+      v191v5v194v22v1 _ p = T.fatal p in (v191v5v194v22v1))
     (T.uap2 T.mkNoSrcPos p (gcompare T.mkNoSrcPos p) fkey_to_find fkey)
 hlookupFM _ _ p = T.fatal p
 
@@ -602,9 +608,9 @@ gelemFM pelemFM p = T.ufun2 aelemFM pelemFM p helemFM
 helemFM fkey ffm p =
   T.uccase T.mkNoSrcPos p
     (let
-      v197v5v1 (T.R Nothing _) p = T.con0 T.mkNoSrcPos p False aFalse
-      v197v5v1 (T.R (Just felt) _) p = T.con0 T.mkNoSrcPos p True aTrue
-      v197v5v1 _ p = T.fatal p in (v197v5v1))
+      v197v5v197v66v1 (T.R Nothing _) p = T.con0 T.mkNoSrcPos p False aFalse
+      v197v5v197v66v1 (T.R (Just felt) _) p = T.con0 T.mkNoSrcPos p True aTrue
+      v197v5v197v66v1 _ p = T.fatal p in (v197v5v197v66v1))
     (T.uwrapForward p (hlookupFM ffm fkey p))
 
 glookupWithDefaultFM plookupWithDefaultFM p =
@@ -613,9 +619,9 @@ glookupWithDefaultFM plookupWithDefaultFM p =
 hlookupWithDefaultFM ffm fdeflt fkey p =
   T.uccase T.mkNoSrcPos p
     (let
-      v200v5v1 (T.R Nothing _) p = T.projection T.mkNoSrcPos p fdeflt
-      v200v5v1 (T.R (Just felt) _) p = T.projection T.mkNoSrcPos p felt
-      v200v5v1 _ p = T.fatal p in (v200v5v1))
+      v200v5v200v65v1 (T.R Nothing _) p = T.projection T.mkNoSrcPos p fdeflt
+      v200v5v200v65v1 (T.R (Just felt) _) p = T.projection T.mkNoSrcPos p felt
+      v200v5v200v65v1 _ p = T.fatal p in (v200v5v200v65v1))
     (T.uwrapForward p (hlookupFM ffm fkey p))
 
 gfmToList pfmToList p = T.ufun1 afmToList pfmToList p hfmToList
@@ -626,7 +632,9 @@ hfmToList ffm p =
       (T.ufun3 T.mkLambda T.mkNoSrcPos p
         (\ fkey felt frest p ->
           T.con2 T.mkNoSrcPos p T.Cons T.aCons
-            (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2 fkey felt) frest))
+            (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
+              (T.projection T.mkNoSrcPos p fkey)
+              (T.projection T.mkNoSrcPos p felt)) frest))
       (T.con0 T.mkNoSrcPos p T.List T.aList) ffm p)
 
 gkeysFM pkeysFM p = T.ufun1 akeysFM pkeysFM p hkeysFM
@@ -686,7 +694,7 @@ hmkBranch fwhich fkey felt ffm_l ffm_r p =
   let
     gresult presult p = T.uconstUse presult p sresult
     sresult =
-      T.uconstDef p a215v9result
+      T.uconstDef p a215v9v215v70result
         (\ p ->
           T.con5 T.mkNoSrcPos p Branch aBranch fkey felt
             (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p)
@@ -701,59 +709,62 @@ hmkBranch fwhich fkey felt ffm_l ffm_r p =
   gleft_ok pleft_ok p = T.uconstUse pleft_ok p sleft_ok
   
   sleft_ok =
-    T.uconstDef p a218v5left_ok
+    T.uconstDef p a218v5v222v63left_ok
       (\ p ->
         T.uccase T.mkNoSrcPos p
           (let
-            v219v9v1 (T.R EmptyFM _) p = T.con0 T.mkNoSrcPos p True aTrue
-            v219v9v1 (T.R (Branch fleft_key _ _ _ _) _) p =
+            v219v9v222v63v1 (T.R EmptyFM _) p = T.con0 T.mkNoSrcPos p True aTrue
+            v219v9v222v63v1 (T.R (Branch fleft_key _ _ _ _) _) p =
               let
                 gbiggest_left_key pbiggest_left_key p =
                   T.uconstUse pbiggest_left_key p sbiggest_left_key
                 sbiggest_left_key =
-                  T.uconstDef p a221v43biggest_left_key
+                  T.uconstDef p a221v43v221v78biggest_left_key
                     (\ p ->
                       T.uwrapForward p
                         (hfst (T.uwrapForward p (hfindMax ffm_l p)) p)) in
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !< p)
                   (gbiggest_left_key T.mkNoSrcPos p) fkey)
-            v219v9v1 _ p = T.fatal p in (v219v9v1)) ffm_l)
+            v219v9v222v63v1 _ p = T.fatal p in (v219v9v222v63v1)) ffm_l)
   
   gright_ok pright_ok p = T.uconstUse pright_ok p sright_ok
   
   sright_ok =
-    T.uconstDef p a223v5right_ok
+    T.uconstDef p a223v5v227v61right_ok
       (\ p ->
         T.uccase T.mkNoSrcPos p
           (let
-            v224v9v1 (T.R EmptyFM _) p = T.con0 T.mkNoSrcPos p True aTrue
-            v224v9v1 (T.R (Branch fright_key _ _ _ _) _) p =
+            v224v9v227v61v1 (T.R EmptyFM _) p = T.con0 T.mkNoSrcPos p True aTrue
+            v224v9v227v61v1 (T.R (Branch fright_key _ _ _ _) _) p =
               let
                 gsmallest_r_key psmallest_r_key p =
                   T.uconstUse psmallest_r_key p ssmallest_r_key
                 ssmallest_r_key =
-                  T.uconstDef p a226v43smallest_r_key
+                  T.uconstDef p a226v43v226v76smallest_r_key
                     (\ p ->
                       T.uwrapForward p
                         (hfst (T.uwrapForward p (hfindMin ffm_r p)) p)) in
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !< p) fkey
                   (gsmallest_r_key T.mkNoSrcPos p))
-            v224v9v1 _ p = T.fatal p in (v224v9v1)) ffm_r)
+            v224v9v227v61v1 _ p = T.fatal p in (v224v9v227v61v1)) ffm_r)
   
   gbalance_ok pbalance_ok p = T.uconstUse pbalance_ok p sbalance_ok
   
   sbalance_ok =
-    T.uconstDef p a228v5balance_ok (\ p -> T.con0 T.mkNoSrcPos p True aTrue)
+    T.uconstDef p a228v5v228v21balance_ok
+      (\ p -> T.con0 T.mkNoSrcPos p True aTrue)
   
   gleft_size pleft_size p = T.uconstUse pleft_size p sleft_size
   
   sleft_size =
-    T.uconstDef p a230v5left_size (\ p -> T.uwrapForward p (hsizeFM ffm_l p))
+    T.uconstDef p a230v5v230v28left_size
+      (\ p -> T.uwrapForward p (hsizeFM ffm_l p))
   
   gright_size pright_size p = T.uconstUse pright_size p sright_size
   
   sright_size =
-    T.uconstDef p a231v5right_size (\ p -> T.uwrapForward p (hsizeFM ffm_r p))
+    T.uconstDef p a231v5v231v28right_size
+      (\ p -> T.uwrapForward p (hsizeFM ffm_r p))
   
 
 gmkBalBranch ::
@@ -792,7 +803,7 @@ hmkBalBranch fkey felt ffm_L ffm_R p =
           (gsize_l T.mkNoSrcPos p)))
       (T.uccase T.mkNoSrcPos p
         (let
-          v242v5v1 (T.R (Branch _ _ _ ffm_rl ffm_rr) _) p =
+          v242v5v245v71v1 (T.R (Branch _ _ _ ffm_rl ffm_rr) _) p =
             T.ucguard
               (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !< p)
                 (T.uwrapForward p (hsizeFM ffm_rl p))
@@ -804,14 +815,14 @@ hmkBalBranch fkey felt ffm_L ffm_R p =
               (T.uwrapForward p (hsingle_L ffm_L ffm_R p))
               (T.ucguard (gotherwise T.mkNoSrcPos p)
                 (T.uwrapForward p (hdouble_L ffm_L ffm_R p)) (T.fatal p))
-          v242v5v1 _ p = T.fatal p in (v242v5v1)) ffm_R)
+          v242v5v245v71v1 _ p = T.fatal p in (v242v5v245v71v1)) ffm_R)
       (T.ucguard
         (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !> p) (gsize_l T.mkNoSrcPos p)
           (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !* p)
             (gsIZE_RATIO T.mkNoSrcPos p) (gsize_r T.mkNoSrcPos p)))
         (T.uccase T.mkNoSrcPos p
           (let
-            v249v5v1 (T.R (Branch _ _ _ ffm_ll ffm_lr) _) p =
+            v249v5v252v71v1 (T.R (Branch _ _ _ ffm_ll ffm_lr) _) p =
               T.ucguard
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !< p)
                   (T.uwrapForward p (hsizeFM ffm_lr p))
@@ -823,7 +834,7 @@ hmkBalBranch fkey felt ffm_L ffm_R p =
                 (T.uwrapForward p (hsingle_R ffm_L ffm_R p))
                 (T.ucguard (gotherwise T.mkNoSrcPos p)
                   (T.uwrapForward p (hdouble_R ffm_L ffm_R p)) (T.fatal p))
-            v249v5v1 _ p = T.fatal p in (v249v5v1)) ffm_L)
+            v249v5v252v71v1 _ p = T.fatal p in (v249v5v252v71v1)) ffm_L)
         (T.ucguard (gotherwise T.mkNoSrcPos p)
           (T.uwrapForward p
             (hmkBranch
@@ -836,16 +847,18 @@ hmkBalBranch fkey felt ffm_L ffm_R p =
   gsize_l psize_l p = T.uconstUse psize_l p ssize_l
   
   ssize_l =
-    T.uconstDef p a259v5size_l (\ p -> T.uwrapForward p (hsizeFM ffm_L p))
+    T.uconstDef p a259v5v259v26size_l
+      (\ p -> T.uwrapForward p (hsizeFM ffm_L p))
   
   gsize_r psize_r p = T.uconstUse psize_r p ssize_r
   
   ssize_r =
-    T.uconstDef p a260v5size_r (\ p -> T.uwrapForward p (hsizeFM ffm_R p))
+    T.uconstDef p a260v5v260v26size_r
+      (\ p -> T.uwrapForward p (hsizeFM ffm_R p))
   
-  gsingle_L psingle_L p = T.ufun2 a261v5single_L psingle_L p hsingle_L
+  gsingle_L psingle_L p = T.ufun2 a261v5v264v35single_L psingle_L p hsingle_L
   
-  asingle_L = a261v5single_L
+  asingle_L = a261v5v264v35single_L
   
   hsingle_L ffm_l (T.R (Branch fkey_r felt_r _ ffm_rl ffm_rr) _) p =
     T.uwrapForward p
@@ -860,9 +873,9 @@ hmkBalBranch fkey felt ffm_L ffm_R p =
         p)
   hsingle_L _ _ p = T.fatal p
   
-  gdouble_L pdouble_L p = T.ufun2 a265v5double_L pdouble_L p hdouble_L
+  gdouble_L pdouble_L p = T.ufun2 a265v5v270v75double_L pdouble_L p hdouble_L
   
-  adouble_L = a265v5double_L
+  adouble_L = a265v5v270v75double_L
   
   hdouble_L ffm_l
     (T.R
@@ -885,9 +898,9 @@ hmkBalBranch fkey felt ffm_L ffm_R p =
         p)
   hdouble_L _ _ p = T.fatal p
   
-  gsingle_R psingle_R p = T.ufun2 a271v5single_R psingle_R p hsingle_R
+  gsingle_R psingle_R p = T.ufun2 a271v5v273v69single_R psingle_R p hsingle_R
   
-  asingle_R = a271v5single_R
+  asingle_R = a271v5v273v69single_R
   
   hsingle_R (T.R (Branch fkey_l felt_l _ ffm_ll ffm_lr) _) ffm_r p =
     T.uwrapForward p
@@ -901,9 +914,9 @@ hmkBalBranch fkey felt ffm_L ffm_R p =
               (T.conInteger T.mkNoSrcPos p 9)) fkey felt ffm_lr ffm_r p)) p)
   hsingle_R _ _ p = T.fatal p
   
-  gdouble_R pdouble_R p = T.ufun2 a274v5double_R pdouble_R p hdouble_R
+  gdouble_R pdouble_R p = T.ufun2 a274v5v278v76double_R pdouble_R p hdouble_R
   
-  adouble_R = a274v5double_R
+  adouble_R = a274v5v278v76double_R
   
   hdouble_R
     (T.R
@@ -978,12 +991,14 @@ hmkVBalBranch fkey felt (ffm_l@(T.R (Branch fkey_l felt_l _ ffm_ll ffm_lr) _))
   gsize_l psize_l p = T.uconstUse psize_l p ssize_l
   
   ssize_l =
-    T.uconstDef p a300v5size_l (\ p -> T.uwrapForward p (hsizeFM ffm_l p))
+    T.uconstDef p a300v5v300v24size_l
+      (\ p -> T.uwrapForward p (hsizeFM ffm_l p))
   
   gsize_r psize_r p = T.uconstUse psize_r p ssize_r
   
   ssize_r =
-    T.uconstDef p a301v5size_r (\ p -> T.uwrapForward p (hsizeFM ffm_r p))
+    T.uconstDef p a301v5v301v24size_r
+      (\ p -> T.uwrapForward p (hsizeFM ffm_r p))
   
 hmkVBalBranch _ _ _ _ p = T.fatal p
 
@@ -1021,41 +1036,45 @@ hglueBal ffm1 ffm2 p =
   
   gmid_elt1 pmid_key1 p = T.uconstUse pmid_key1 p smid_elt1
   
-  j319v5mid_key1 =
+  j319v5v319v24mid_key1 =
     case T.uwrapForward p (hfindMax ffm1 p) of
       T.R (T.Tuple2 fmid_key1 fmid_elt1) kmid_key1 ->
         (kmid_key1,fmid_key1,fmid_elt1)
       _ -> T.fatal p
   
   smid_key1 =
-    T.uconstDef p a319v6mid_key1
+    T.uconstDef p a319v6v319v13mid_key1
       (\ _ ->
-        case j319v5mid_key1 of (kmid_key1,fmid_key1,fmid_elt1) -> fmid_key1)
+        case j319v5v319v24mid_key1 of
+          (kmid_key1,fmid_key1,fmid_elt1) -> fmid_key1)
   
   smid_elt1 =
-    T.uconstDef p a319v16mid_elt1
+    T.uconstDef p a319v16v319v23mid_elt1
       (\ _ ->
-        case j319v5mid_key1 of (kmid_key1,fmid_key1,fmid_elt1) -> fmid_elt1)
+        case j319v5v319v24mid_key1 of
+          (kmid_key1,fmid_key1,fmid_elt1) -> fmid_elt1)
   
   gmid_key2 pmid_key2 p = T.uconstUse pmid_key2 p smid_key2
   
   gmid_elt2 pmid_key2 p = T.uconstUse pmid_key2 p smid_elt2
   
-  j320v5mid_key2 =
+  j320v5v320v24mid_key2 =
     case T.uwrapForward p (hfindMin ffm2 p) of
       T.R (T.Tuple2 fmid_key2 fmid_elt2) kmid_key2 ->
         (kmid_key2,fmid_key2,fmid_elt2)
       _ -> T.fatal p
   
   smid_key2 =
-    T.uconstDef p a320v6mid_key2
+    T.uconstDef p a320v6v320v13mid_key2
       (\ _ ->
-        case j320v5mid_key2 of (kmid_key2,fmid_key2,fmid_elt2) -> fmid_key2)
+        case j320v5v320v24mid_key2 of
+          (kmid_key2,fmid_key2,fmid_elt2) -> fmid_key2)
   
   smid_elt2 =
-    T.uconstDef p a320v16mid_elt2
+    T.uconstDef p a320v16v320v23mid_elt2
       (\ _ ->
-        case j320v5mid_key2 of (kmid_key2,fmid_key2,fmid_elt2) -> fmid_elt2)
+        case j320v5v320v24mid_key2 of
+          (kmid_key2,fmid_key2,fmid_elt2) -> fmid_elt2)
   
 
 gglueVBal ::
@@ -1098,12 +1117,14 @@ hglueVBal (ffm_l@(T.R (Branch fkey_l felt_l _ ffm_ll ffm_lr) _))
   gsize_l psize_l p = T.uconstUse psize_l p ssize_l
   
   ssize_l =
-    T.uconstDef p a339v5size_l (\ p -> T.uwrapForward p (hsizeFM ffm_l p))
+    T.uconstDef p a339v5v339v24size_l
+      (\ p -> T.uwrapForward p (hsizeFM ffm_l p))
   
   gsize_r psize_r p = T.uconstUse psize_r p ssize_r
   
   ssize_r =
-    T.uconstDef p a340v5size_r (\ p -> T.uwrapForward p (hsizeFM ffm_r p))
+    T.uconstDef p a340v5v340v24size_r
+      (\ p -> T.uwrapForward p (hsizeFM ffm_r p))
   
 hglueVBal _ _ p = T.fatal p
 
@@ -1126,13 +1147,14 @@ hsplitLT (T.R EmptyFM _) fsplit_key p = gemptyFM T.mkNoSrcPos p
 hsplitLT (T.R (Branch fkey felt _ ffm_l ffm_r) _) fsplit_key p =
   T.uccase T.mkNoSrcPos p
     (let
-      v345v5v1 (T.R LT _) p = T.uwrapForward p (hsplitLT ffm_l fsplit_key p)
-      v345v5v1 (T.R GT _) p =
+      v345v5v348v18v1 (T.R LT _) p =
+        T.uwrapForward p (hsplitLT ffm_l fsplit_key p)
+      v345v5v348v18v1 (T.R GT _) p =
         T.uwrapForward p
           (hmkVBalBranch fkey felt ffm_l
             (T.uwrapForward p (hsplitLT ffm_r fsplit_key p)) p)
-      v345v5v1 (T.R EQ _) p = T.projection T.mkNoSrcPos p ffm_l
-      v345v5v1 _ p = T.fatal p in (v345v5v1))
+      v345v5v348v18v1 (T.R EQ _) p = T.projection T.mkNoSrcPos p ffm_l
+      v345v5v348v18v1 _ p = T.fatal p in (v345v5v348v18v1))
     (T.uap2 T.mkNoSrcPos p (gcompare T.mkNoSrcPos p) fsplit_key fkey)
 hsplitLT _ _ p = T.fatal p
 
@@ -1142,13 +1164,14 @@ hsplitGT (T.R EmptyFM _) fsplit_key p = gemptyFM T.mkNoSrcPos p
 hsplitGT (T.R (Branch fkey felt _ ffm_l ffm_r) _) fsplit_key p =
   T.uccase T.mkNoSrcPos p
     (let
-      v351v5v1 (T.R GT _) p = T.uwrapForward p (hsplitGT ffm_r fsplit_key p)
-      v351v5v1 (T.R LT _) p =
+      v351v5v354v18v1 (T.R GT _) p =
+        T.uwrapForward p (hsplitGT ffm_r fsplit_key p)
+      v351v5v354v18v1 (T.R LT _) p =
         T.uwrapForward p
           (hmkVBalBranch fkey felt
             (T.uwrapForward p (hsplitGT ffm_l fsplit_key p)) ffm_r p)
-      v351v5v1 (T.R EQ _) p = T.projection T.mkNoSrcPos p ffm_r
-      v351v5v1 _ p = T.fatal p in (v351v5v1))
+      v351v5v354v18v1 (T.R EQ _) p = T.projection T.mkNoSrcPos p ffm_r
+      v351v5v354v18v1 _ p = T.fatal p in (v351v5v354v18v1))
     (T.uap2 T.mkNoSrcPos p (gcompare T.mkNoSrcPos p) fsplit_key fkey)
 hsplitGT _ _ p = T.fatal p
 
@@ -1214,7 +1237,7 @@ instance (Eq key,Eq elt) => Eq ((FiniteMap key elt))
   where
   
   (!==) (%==) p =
-    T.ufun2 (+%^+=@==) (%==) p (*==)
+    T.ufun2 (+%^+=@=%^+=>==) (%==) p (*==)
     where
     
     (*==) ffm_1 ffm_2 p =
@@ -1231,174 +1254,222 @@ instance (Eq key,Eq elt) => Eq ((FiniteMap key elt))
 
 tData_FiniteMap = T.mkModule "Data.FiniteMap" "Data/FiniteMap.hs" Prelude.False
 
-aEmptyFM = T.mkConstructor tData_FiniteMap 860005 3 0 "EmptyFM"
+aEmptyFM = T.mkConstructor tData_FiniteMap 860005 860011 3 0 "EmptyFM"
 
-aBranch = T.mkConstructor tData_FiniteMap 870005 3 5 "Branch"
+aBranch = T.mkConstructor tData_FiniteMap 870005 870010 3 5 "Branch"
 
-aemptyFM = T.mkVariable tData_FiniteMap 920001 3 0 "emptyFM" Prelude.False
+aemptyFM =
+  T.mkVariable tData_FiniteMap 920001 920017 3 0 "emptyFM" Prelude.False
 
-aunitFM = T.mkVariable tData_FiniteMap 930001 3 2 "unitFM" Prelude.False
+aunitFM = T.mkVariable tData_FiniteMap 930001 930049 3 2 "unitFM" Prelude.False
 
-alistToFM = T.mkVariable tData_FiniteMap 940001 3 0 "listToFM" Prelude.False
+alistToFM =
+  T.mkVariable tData_FiniteMap 940001 940030 3 0 "listToFM" Prelude.False
 
-aaddToFM = T.mkVariable tData_FiniteMap 950001 3 3 "addToFM" Prelude.False
+aaddToFM =
+  T.mkVariable tData_FiniteMap 950001 950060 3 3 "addToFM" Prelude.False
 
-aaddToFM_C = T.mkVariable tData_FiniteMap 970001 3 4 "addToFM_C" Prelude.False
+aaddToFM_C =
+  T.mkVariable tData_FiniteMap 970001 1020064 3 4 "addToFM_C" Prelude.False
 
 aaddListToFM =
-  T.mkVariable tData_FiniteMap 1040001 3 2 "addListToFM" Prelude.False
+  T.mkVariable tData_FiniteMap 1040001 1040080 3 2 "addListToFM" Prelude.False
 
 aaddListToFM_C =
-  T.mkVariable tData_FiniteMap 1050001 3 3 "addListToFM_C" Prelude.False
+  T.mkVariable tData_FiniteMap 1050001 1080056 3 3 "addListToFM_C" Prelude.False
 
-adelFromFM = T.mkVariable tData_FiniteMap 1100001 3 2 "delFromFM" Prelude.False
+adelFromFM =
+  T.mkVariable tData_FiniteMap 1100001 1150031 3 2 "delFromFM" Prelude.False
 
 adelListFromFM =
-  T.mkVariable tData_FiniteMap 1170001 3 2 "delListFromFM" Prelude.False
+  T.mkVariable tData_FiniteMap 1170001 1170047 3 2 "delListFromFM" Prelude.False
 
-aplusFM_C = T.mkVariable tData_FiniteMap 1190001 3 3 "plusFM_C" Prelude.False
+aplusFM_C =
+  T.mkVariable tData_FiniteMap 1190001 1300047 3 3 "plusFM_C" Prelude.False
 
-aplusFM = T.mkVariable tData_FiniteMap 1320001 3 2 "plusFM" Prelude.False
+aplusFM =
+  T.mkVariable tData_FiniteMap 1320001 1380035 3 2 "plusFM" Prelude.False
 
-aminusFM = T.mkVariable tData_FiniteMap 1400001 3 2 "minusFM" Prelude.False
+aminusFM =
+  T.mkVariable tData_FiniteMap 1400001 1470031 3 2 "minusFM" Prelude.False
 
 aintersectFM =
-  T.mkVariable tData_FiniteMap 1490001 3 2 "intersectFM" Prelude.False
+  T.mkVariable tData_FiniteMap 1490001 1490067 3 2 "intersectFM" Prelude.False
 
 aintersectFM_C =
-  T.mkVariable tData_FiniteMap 1510001 3 3 "intersectFM_C" Prelude.False
+  T.mkVariable tData_FiniteMap 1510001 1660027 3 3 "intersectFM_C" Prelude.False
 
-afoldFM = T.mkVariable tData_FiniteMap 1680001 3 3 "foldFM" Prelude.False
+afoldFM =
+  T.mkVariable tData_FiniteMap 1680001 1700047 3 3 "foldFM" Prelude.False
 
-amapFM = T.mkVariable tData_FiniteMap 1720001 3 2 "mapFM" Prelude.False
+amapFM = T.mkVariable tData_FiniteMap 1720001 1740060 3 2 "mapFM" Prelude.False
 
-afilterFM = T.mkVariable tData_FiniteMap 1760001 3 2 "filterFM" Prelude.False
+afilterFM =
+  T.mkVariable tData_FiniteMap 1760001 1820047 3 2 "filterFM" Prelude.False
 
-asizeFM = T.mkVariable tData_FiniteMap 1840001 3 1 "sizeFM" Prelude.False
+asizeFM =
+  T.mkVariable tData_FiniteMap 1840001 1850035 3 1 "sizeFM" Prelude.False
 
-aisEmptyFM = T.mkVariable tData_FiniteMap 1870001 3 1 "isEmptyFM" Prelude.False
+aisEmptyFM =
+  T.mkVariable tData_FiniteMap 1870001 1870029 3 1 "isEmptyFM" Prelude.False
 
-alookupFM = T.mkVariable tData_FiniteMap 1890001 3 2 "lookupFM" Prelude.False
+alookupFM =
+  T.mkVariable tData_FiniteMap 1890001 1940022 3 2 "lookupFM" Prelude.False
 
-aelemFM = T.mkVariable tData_FiniteMap 1960006 3 2 "elemFM" Prelude.False
+aelemFM =
+  T.mkVariable tData_FiniteMap 1960006 1960011 3 2 "elemFM" Prelude.False
 
 alookupWithDefaultFM =
-  T.mkVariable tData_FiniteMap 1990001 3 3 "lookupWithDefaultFM" Prelude.False
+  T.mkVariable tData_FiniteMap 1990001 2000065 3 3 "lookupWithDefaultFM"
+    Prelude.False
 
-afmToList = T.mkVariable tData_FiniteMap 2020001 3 1 "fmToList" Prelude.False
+afmToList =
+  T.mkVariable tData_FiniteMap 2020001 2020063 3 1 "fmToList" Prelude.False
 
-akeysFM = T.mkVariable tData_FiniteMap 2030001 3 1 "keysFM" Prelude.False
+akeysFM =
+  T.mkVariable tData_FiniteMap 2030001 2030063 3 1 "keysFM" Prelude.False
 
-aeltsFM = T.mkVariable tData_FiniteMap 2040001 3 1 "eltsFM" Prelude.False
+aeltsFM =
+  T.mkVariable tData_FiniteMap 2040001 2040063 3 1 "eltsFM" Prelude.False
 
 asIZE_RATIO =
-  T.mkVariable tData_FiniteMap 2070001 3 0 "sIZE_RATIO" Prelude.False
+  T.mkVariable tData_FiniteMap 2070001 2070014 3 0 "sIZE_RATIO" Prelude.False
 
-amkBranch = T.mkVariable tData_FiniteMap 2140001 3 5 "mkBranch" Prelude.False
+amkBranch =
+  T.mkVariable tData_FiniteMap 2140001 2310028 3 5 "mkBranch" Prelude.False
 
 amkBalBranch =
-  T.mkVariable tData_FiniteMap 2370001 3 4 "mkBalBranch" Prelude.False
+  T.mkVariable tData_FiniteMap 2370001 2780076 3 4 "mkBalBranch" Prelude.False
 
 amkVBalBranch =
-  T.mkVariable tData_FiniteMap 2850001 3 4 "mkVBalBranch" Prelude.False
+  T.mkVariable tData_FiniteMap 2850001 3010024 3 4 "mkVBalBranch" Prelude.False
 
-aglueBal = T.mkVariable tData_FiniteMap 3070001 3 2 "glueBal" Prelude.False
+aglueBal =
+  T.mkVariable tData_FiniteMap 3070001 3200038 3 2 "glueBal" Prelude.False
 
-aglueVBal = T.mkVariable tData_FiniteMap 3260001 3 2 "glueVBal" Prelude.False
+aglueVBal =
+  T.mkVariable tData_FiniteMap 3260001 3400024 3 2 "glueVBal" Prelude.False
 
-asplitLT = T.mkVariable tData_FiniteMap 3430001 3 2 "splitLT" Prelude.False
+asplitLT =
+  T.mkVariable tData_FiniteMap 3430001 3480018 3 2 "splitLT" Prelude.False
 
-asplitGT = T.mkVariable tData_FiniteMap 3490001 3 2 "splitGT" Prelude.False
+asplitGT =
+  T.mkVariable tData_FiniteMap 3490001 3540018 3 2 "splitGT" Prelude.False
 
-afindMin = T.mkVariable tData_FiniteMap 3570001 3 1 "findMin" Prelude.False
+afindMin =
+  T.mkVariable tData_FiniteMap 3570001 3580051 3 1 "findMin" Prelude.False
 
-adeleteMin = T.mkVariable tData_FiniteMap 3610001 3 1 "deleteMin" Prelude.False
+adeleteMin =
+  T.mkVariable tData_FiniteMap 3610001 3630077 3 1 "deleteMin" Prelude.False
 
-afindMax = T.mkVariable tData_FiniteMap 3660001 3 1 "findMax" Prelude.False
+afindMax =
+  T.mkVariable tData_FiniteMap 3660001 3670051 3 1 "findMax" Prelude.False
 
-adeleteMax = T.mkVariable tData_FiniteMap 3700001 3 1 "deleteMax" Prelude.False
+adeleteMax =
+  T.mkVariable tData_FiniteMap 3700001 3720071 3 1 "deleteMax" Prelude.False
 
-(+%^+=@==) = T.mkVariable tData_FiniteMap 3760008 16 2 "==" Prelude.False
+(+%^+=@=%^+=>==) =
+  T.mkVariable tData_FiniteMap 3760008 3760009 16 2 "==" Prelude.False
 
-a108v5add = T.mkVariable tData_FiniteMap 1080005 3 2 "add" Prelude.True
+a108v5v108v56add =
+  T.mkVariable tData_FiniteMap 1080005 1080056 3 2 "add" Prelude.True
 
-a126v5lts = T.mkVariable tData_FiniteMap 1260005 3 0 "lts" Prelude.True
+a126v5v126v35lts =
+  T.mkVariable tData_FiniteMap 1260005 1260035 3 0 "lts" Prelude.True
 
-a127v5gts = T.mkVariable tData_FiniteMap 1270005 3 0 "gts" Prelude.True
+a127v5v127v35gts =
+  T.mkVariable tData_FiniteMap 1270005 1270035 3 0 "gts" Prelude.True
 
-a128v5new_elt = T.mkVariable tData_FiniteMap 1280005 3 0 "new_elt" Prelude.True
+a128v5v130v47new_elt =
+  T.mkVariable tData_FiniteMap 1280005 1300047 3 0 "new_elt" Prelude.True
 
-a137v5lts = T.mkVariable tData_FiniteMap 1370005 3 0 "lts" Prelude.True
+a137v5v137v35lts =
+  T.mkVariable tData_FiniteMap 1370005 1370035 3 0 "lts" Prelude.True
 
-a138v5gts = T.mkVariable tData_FiniteMap 1380005 3 0 "gts" Prelude.True
+a138v5v138v35gts =
+  T.mkVariable tData_FiniteMap 1380005 1380035 3 0 "gts" Prelude.True
 
-a146v5lts = T.mkVariable tData_FiniteMap 1460005 3 0 "lts" Prelude.True
+a146v5v146v31lts =
+  T.mkVariable tData_FiniteMap 1460005 1460031 3 0 "lts" Prelude.True
 
-a147v5gts = T.mkVariable tData_FiniteMap 1470005 3 0 "gts" Prelude.True
+a147v5v147v31gts =
+  T.mkVariable tData_FiniteMap 1470005 1470031 3 0 "gts" Prelude.True
 
-a162v5lts = T.mkVariable tData_FiniteMap 1620005 3 0 "lts" Prelude.True
+a162v5v162v31lts =
+  T.mkVariable tData_FiniteMap 1620005 1620031 3 0 "lts" Prelude.True
 
-a163v5gts = T.mkVariable tData_FiniteMap 1630005 3 0 "gts" Prelude.True
+a163v5v163v31gts =
+  T.mkVariable tData_FiniteMap 1630005 1630031 3 0 "gts" Prelude.True
 
-a165v5maybe_elt1 =
-  T.mkVariable tData_FiniteMap 1650005 3 0 "maybe_elt1" Prelude.True
+a165v5v165v39maybe_elt1 =
+  T.mkVariable tData_FiniteMap 1650005 1650039 3 0 "maybe_elt1" Prelude.True
 
-a166v10elt1 = T.mkVariable tData_FiniteMap 1660010 3 0 "elt1" Prelude.True
+a166v10v166v13elt1 =
+  T.mkVariable tData_FiniteMap 1660010 1660013 3 0 "elt1" Prelude.True
 
-a218v5left_ok = T.mkVariable tData_FiniteMap 2180005 3 0 "left_ok" Prelude.True
+a218v5v222v63left_ok =
+  T.mkVariable tData_FiniteMap 2180005 2220063 3 0 "left_ok" Prelude.True
 
-a223v5right_ok =
-  T.mkVariable tData_FiniteMap 2230005 3 0 "right_ok" Prelude.True
+a223v5v227v61right_ok =
+  T.mkVariable tData_FiniteMap 2230005 2270061 3 0 "right_ok" Prelude.True
 
-a228v5balance_ok =
-  T.mkVariable tData_FiniteMap 2280005 3 0 "balance_ok" Prelude.True
+a228v5v228v21balance_ok =
+  T.mkVariable tData_FiniteMap 2280005 2280021 3 0 "balance_ok" Prelude.True
 
-a230v5left_size =
-  T.mkVariable tData_FiniteMap 2300005 3 0 "left_size" Prelude.True
+a230v5v230v28left_size =
+  T.mkVariable tData_FiniteMap 2300005 2300028 3 0 "left_size" Prelude.True
 
-a231v5right_size =
-  T.mkVariable tData_FiniteMap 2310005 3 0 "right_size" Prelude.True
+a231v5v231v28right_size =
+  T.mkVariable tData_FiniteMap 2310005 2310028 3 0 "right_size" Prelude.True
 
-a221v43biggest_left_key =
-  T.mkVariable tData_FiniteMap 2210043 3 0 "biggest_left_key" Prelude.True
+a221v43v221v78biggest_left_key =
+  T.mkVariable tData_FiniteMap 2210043 2210078 3 0 "biggest_left_key"
+    Prelude.True
 
-a226v43smallest_r_key =
-  T.mkVariable tData_FiniteMap 2260043 3 0 "smallest_r_key" Prelude.True
+a226v43v226v76smallest_r_key =
+  T.mkVariable tData_FiniteMap 2260043 2260076 3 0 "smallest_r_key" Prelude.True
 
-a215v9result = T.mkVariable tData_FiniteMap 2150009 3 0 "result" Prelude.True
+a215v9v215v70result =
+  T.mkVariable tData_FiniteMap 2150009 2150070 3 0 "result" Prelude.True
 
-a259v5size_l = T.mkVariable tData_FiniteMap 2590005 3 0 "size_l" Prelude.True
+a259v5v259v26size_l =
+  T.mkVariable tData_FiniteMap 2590005 2590026 3 0 "size_l" Prelude.True
 
-a260v5size_r = T.mkVariable tData_FiniteMap 2600005 3 0 "size_r" Prelude.True
+a260v5v260v26size_r =
+  T.mkVariable tData_FiniteMap 2600005 2600026 3 0 "size_r" Prelude.True
 
-a261v5single_L =
-  T.mkVariable tData_FiniteMap 2610005 3 2 "single_L" Prelude.True
+a261v5v264v35single_L =
+  T.mkVariable tData_FiniteMap 2610005 2640035 3 2 "single_L" Prelude.True
 
-a265v5double_L =
-  T.mkVariable tData_FiniteMap 2650005 3 2 "double_L" Prelude.True
+a265v5v270v75double_L =
+  T.mkVariable tData_FiniteMap 2650005 2700075 3 2 "double_L" Prelude.True
 
-a271v5single_R =
-  T.mkVariable tData_FiniteMap 2710005 3 2 "single_R" Prelude.True
+a271v5v273v69single_R =
+  T.mkVariable tData_FiniteMap 2710005 2730069 3 2 "single_R" Prelude.True
 
-a274v5double_R =
-  T.mkVariable tData_FiniteMap 2740005 3 2 "double_R" Prelude.True
+a274v5v278v76double_R =
+  T.mkVariable tData_FiniteMap 2740005 2780076 3 2 "double_R" Prelude.True
 
-a300v5size_l = T.mkVariable tData_FiniteMap 3000005 3 0 "size_l" Prelude.True
+a300v5v300v24size_l =
+  T.mkVariable tData_FiniteMap 3000005 3000024 3 0 "size_l" Prelude.True
 
-a301v5size_r = T.mkVariable tData_FiniteMap 3010005 3 0 "size_r" Prelude.True
+a301v5v301v24size_r =
+  T.mkVariable tData_FiniteMap 3010005 3010024 3 0 "size_r" Prelude.True
 
-a319v6mid_key1 =
-  T.mkVariable tData_FiniteMap 3190006 3 0 "mid_key1" Prelude.True
+a319v6v319v13mid_key1 =
+  T.mkVariable tData_FiniteMap 3190006 3190013 3 0 "mid_key1" Prelude.True
 
-a319v16mid_elt1 =
-  T.mkVariable tData_FiniteMap 3190016 3 0 "mid_elt1" Prelude.True
+a319v16v319v23mid_elt1 =
+  T.mkVariable tData_FiniteMap 3190016 3190023 3 0 "mid_elt1" Prelude.True
 
-a320v6mid_key2 =
-  T.mkVariable tData_FiniteMap 3200006 3 0 "mid_key2" Prelude.True
+a320v6v320v13mid_key2 =
+  T.mkVariable tData_FiniteMap 3200006 3200013 3 0 "mid_key2" Prelude.True
 
-a320v16mid_elt2 =
-  T.mkVariable tData_FiniteMap 3200016 3 0 "mid_elt2" Prelude.True
+a320v16v320v23mid_elt2 =
+  T.mkVariable tData_FiniteMap 3200016 3200023 3 0 "mid_elt2" Prelude.True
 
-a339v5size_l = T.mkVariable tData_FiniteMap 3390005 3 0 "size_l" Prelude.True
+a339v5v339v24size_l =
+  T.mkVariable tData_FiniteMap 3390005 3390024 3 0 "size_l" Prelude.True
 
-a340v5size_r = T.mkVariable tData_FiniteMap 3400005 3 0 "size_r" Prelude.True
+a340v5v340v24size_r =
+  T.mkVariable tData_FiniteMap 3400005 3400024 3 0 "size_r" Prelude.True
