@@ -2,9 +2,11 @@ module Hat.Data.Ix
   (Ix(grange,gindex,ginRange,grangeSize,srange,sindex,sinRange
       ,srangeSize)) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 import Hat.Ix 
 
