@@ -5,7 +5,7 @@
 
 module Hat.Hat
   (RefModule,RefSrcPos,RefExp,RefAtom
-  ,Pos,Fixity,Arity
+  ,Loc,Fixity,Arity
   ,R(R)
   ,tPrelude
   ,toId,fromId
@@ -46,6 +46,7 @@ module Hat.Hat
   ,mkApp1,mkApp2,mkApp3,mkApp4,mkApp5,mkApp6,mkApp7,mkApp8,mkApp9,mkApp10
   ,mkApp11,mkApp12,mkApp13,mkApp14,mkApp15
   ,mkLambda,mkVariable,mkConstructor,mkConstructorWFields,mkAbstract
+  ,mkDoLambda
   ) where
 
 import Prelude hiding (IO,fromInteger,toInteger,catch)
@@ -53,7 +54,9 @@ import qualified Prelude (IO,fromInteger,toInteger)
 import qualified IO (ioeGetErrorString)
 import Ratio (numerator,denominator)
 #if defined (__GLASGOW_HASKELL__) || defined(__HUGS__)
-import IOExts (unsafePerformIO,IORef,newIORef,readIORef,writeIORef)
+--import IOExts (unsafePerformIO,IORef,newIORef,readIORef,writeIORef)
+import System.IO.Unsafe (unsafePerformIO)
+import Data.IORef (IORef,newIORef,readIORef,writeIORef)
 #endif
 #if defined(__NHC__)
 #if __NHC__ >= 115
@@ -75,10 +78,14 @@ import NHC.FFI (Storable,Ptr,castPtr,withArray,CString,withCString)
 import FFI (Storable,Ptr,castPtr,withArray,CString,withCString) 
 #endif
 #endif
-#if defined (__GLASGOW_HASKELL__)
+#if defined(__GLASGOW_HASKELL__)
+#  if  __GLASGOW_HASKELL__ >= 600
+import Control.Exception (Exception,ioErrors,catch)  -- also catches black holes
+#  else
 import Exception (ioErrors,catch)  -- also catches black holes
+#  endif
 #endif
-#if defined(__NHC__) 
+#if defined(__NHC__)
 import Prelude (catch)
 #endif
 
@@ -101,7 +108,7 @@ newtype RefAtom = RA Int
 mkNoSrcPos :: RefSrcPos
 mkNoSrcPos = RSP 0
 
-type Pos = Int
+type Loc = Int  -- line * 10000 + column
 type Fixity = Int
 type Arity = Int
 
@@ -219,21 +226,21 @@ data Tuple15 a b c d e f g h i j k l m n o =
   Tuple15 (R a) (R b) (R c) (R d) (R e) (R f) (R g) (R h) (R i) (R j) (R k) 
     (R l) (R m) (R n) (R o)
 
-aTuple0 = mkConstructor tPrelude 0 3 0 ""
-aTuple2 = mkConstructor tPrelude 0 3 2 "," 
-aTuple3 = mkConstructor tPrelude 0 3 3 ",,"
-aTuple4 = mkConstructor tPrelude 0 3 4 ",,,"
-aTuple5 = mkConstructor tPrelude 0 3 5 ",,,,"
-aTuple6 = mkConstructor tPrelude 0 3 6 ",,,,,"
-aTuple7 = mkConstructor tPrelude 0 3 7 ",,,,,,"
-aTuple8 = mkConstructor tPrelude 0 3 8 ",,,,,,,"
-aTuple9 = mkConstructor tPrelude 0 3 9 ",,,,,,,,"
-aTuple10 = mkConstructor tPrelude 0 3 10 ",,,,,,,,,"
-aTuple11 = mkConstructor tPrelude 0 3 11 ",,,,,,,,,,"
-aTuple12 = mkConstructor tPrelude 0 3 12 ",,,,,,,,,,,"
-aTuple13 = mkConstructor tPrelude 0 3 13 ",,,,,,,,,,,,"
-aTuple14 = mkConstructor tPrelude 0 3 14 ",,,,,,,,,,,,,"
-aTuple15 = mkConstructor tPrelude 0 3 15 ",,,,,,,,,,,,,,"
+aTuple0 = mkConstructor tPrelude 0 0 3 0 ""
+aTuple2 = mkConstructor tPrelude 0 0 3 2 "," 
+aTuple3 = mkConstructor tPrelude 0 0 3 3 ",,"
+aTuple4 = mkConstructor tPrelude 0 0 3 4 ",,,"
+aTuple5 = mkConstructor tPrelude 0 0 3 5 ",,,,"
+aTuple6 = mkConstructor tPrelude 0 0 3 6 ",,,,,"
+aTuple7 = mkConstructor tPrelude 0 0 3 7 ",,,,,,"
+aTuple8 = mkConstructor tPrelude 0 0 3 8 ",,,,,,,"
+aTuple9 = mkConstructor tPrelude 0 0 3 9 ",,,,,,,,"
+aTuple10 = mkConstructor tPrelude 0 0 3 10 ",,,,,,,,,"
+aTuple11 = mkConstructor tPrelude 0 0 3 11 ",,,,,,,,,,"
+aTuple12 = mkConstructor tPrelude 0 0 3 12 ",,,,,,,,,,,"
+aTuple13 = mkConstructor tPrelude 0 0 3 13 ",,,,,,,,,,,,"
+aTuple14 = mkConstructor tPrelude 0 0 3 14 ",,,,,,,,,,,,,"
+aTuple15 = mkConstructor tPrelude 0 0 3 15 ",,,,,,,,,,,,,,"
 
 toTuple0 :: RefExp -> R Tuple0 -> ()
 toTuple0 h (R Tuple0 _) = ()
@@ -255,8 +262,8 @@ fromTuple2 f g h (x,y) =
 data List a = Cons (R a) (R (List a)) | List  
   -- type constructor and empty list constructor need to have same name,
   -- because transformation doesn't distinguish between the two
-aCons = mkConstructor tPrelude 0 21 2 ":"
-aList = mkConstructor tPrelude 0 3 0 "[]"
+aCons = mkConstructor tPrelude 0 0 21 2 ":"
+aList = mkConstructor tPrelude 0 0 3 0 "[]"
 
 -- ----------------------------------------------------------------------------
 -- Main combinator
@@ -269,10 +276,14 @@ traceIO filename gmain = do
 
 
 #if defined (__GLASGOW_HASKELL__)
+#  if __GLASGOW_HASKELL__ < 600
 exceptionHandler :: IOError -> Prelude.IO a
+#  else
+exceptionHandler :: Exception -> Prelude.IO a
+#  endif
 exceptionHandler exception = do
   hatAborted (case ioErrors exception of
-                Just ioError -> IO.ioeGetErrorString exception
+                Just ioError -> IO.ioeGetErrorString ioError	-- exception
                 Nothing -> show exception)
   return undefined
 #endif
@@ -625,8 +636,9 @@ fun1 var sr p f =
   in R (-- entValueUse rf sr `seq` 
          Fun (\t a -> 
                 if hidden t 
-                  then let r = mkApp1 p mkNoSrcPos rf (refExp a) 
-                       in  wrapResult mkNoSrcPos r (f a r)
+                  then let r = mkApp1 t mkNoSrcPos rf (refExp a) 
+                       in  recordChild t r `seq` 
+                           wrapResult mkNoSrcPos r (f a r)
                   else f a t))
        rf 
 
@@ -638,9 +650,10 @@ fun2 var sr p f =
          Fun (\t a -> 
            R (Fun (\t b ->
                      if hidden t 
-                       then let r = mkApp2 p mkNoSrcPos rf (refExp a) 
+                       then let r = mkApp2 t mkNoSrcPos rf (refExp a) 
                                       (refExp b)
-                            in  wrapResult mkNoSrcPos r (f a b r)
+                            in  recordChild t r `seq` 
+                                wrapResult mkNoSrcPos r (f a b r)
                        else f a b t))
              t))
        rf 
@@ -653,9 +666,10 @@ fun3 var sr p f =
        R (Fun (\t b ->
          R (Fun (\t c ->
            if hidden t 
-             then let r = mkApp3 p mkNoSrcPos rf (refExp a) 
+             then let r = mkApp3 t mkNoSrcPos rf (refExp a) 
                             (refExp b) (refExp c)
-                  in  wrapResult mkNoSrcPos r (f a b c r)
+                  in  recordChild t r `seq` 
+                      wrapResult mkNoSrcPos r (f a b c r)
              else f a b c t))
            t))
          t))
@@ -671,9 +685,10 @@ fun4 var sr p f =
          R (Fun (\t c ->
            R (Fun (\t d -> 
              if hidden t 
-               then let r = mkApp4 p mkNoSrcPos rf (refExp a) 
+               then let r = mkApp4 t mkNoSrcPos rf (refExp a) 
                               (refExp b) (refExp c) (refExp d)
-                    in  wrapResult mkNoSrcPos r (f a b c d r)
+                    in  recordChild t r `seq` 
+                        wrapResult mkNoSrcPos r (f a b c d r)
                else f a b c d t))
              t))
            t))
@@ -691,9 +706,10 @@ fun5 var sr p f =
            R (Fun (\t d -> 
              R (Fun (\t e -> 
                if hidden t 
-                 then let r = mkApp5 p mkNoSrcPos rf (refExp a) (refExp b) 
+                 then let r = mkApp5 t mkNoSrcPos rf (refExp a) (refExp b) 
                                 (refExp c) (refExp d) (refExp e)
-                      in  wrapResult mkNoSrcPos r (f a b c d e r)
+                      in  recordChild t r `seq` 
+                          wrapResult mkNoSrcPos r (f a b c d e r)
                  else f a b c d e t))
                t))
              t))
@@ -714,9 +730,10 @@ fun6 var sr p f =
              R (Fun (\t e -> 
                R (Fun (\t g ->
                  if hidden t 
-                   then let r = mkApp6 p mkNoSrcPos rf (refExp a) (refExp b) 
+                   then let r = mkApp6 t mkNoSrcPos rf (refExp a) (refExp b) 
                                   (refExp c) (refExp d) (refExp e) (refExp g)
-                        in  wrapResult mkNoSrcPos r (f a b c d e g r)
+                        in  recordChild t r `seq` 
+                            wrapResult mkNoSrcPos r (f a b c d e g r)
                    else f a b c d e g t))
                  t))
                t))
@@ -738,10 +755,11 @@ fun7 var sr p f =
                R (Fun (\t g ->
                  R (Fun (\t h ->
                    if hidden t 
-                     then let r = mkApp7 p mkNoSrcPos rf (refExp a) (refExp b) 
+                     then let r = mkApp7 t mkNoSrcPos rf (refExp a) (refExp b) 
                                     (refExp c) (refExp d) (refExp e) (refExp g)
                                     (refExp h)
-                          in  wrapResult mkNoSrcPos r (f a b c d e g h r)
+                          in  recordChild t r `seq` 
+                              wrapResult mkNoSrcPos r (f a b c d e g h r)
                      else f a b c d e g h t))
                    t))
                  t))
@@ -765,11 +783,12 @@ fun8 var sr p f =
                  R (Fun (\t h ->
                    R (Fun (\t i ->
                      if hidden t 
-                       then let r = mkApp8 p mkNoSrcPos rf (refExp a) 
+                       then let r = mkApp8 t mkNoSrcPos rf (refExp a) 
                                       (refExp b) (refExp c) (refExp d) 
                                       (refExp e) (refExp g) (refExp h)
                                       (refExp i)
-                            in  wrapResult mkNoSrcPos r (f a b c d e g h i r)
+                            in  recordChild t r `seq` 
+                                wrapResult mkNoSrcPos r (f a b c d e g h i r)
                        else f a b c d e g h i t))
                      t))
                    t))
@@ -797,11 +816,12 @@ fun9 var sr p f =
                    R (Fun (\t i ->
                      R (Fun (\t j ->
                        if hidden t 
-                         then let r = mkApp9 p mkNoSrcPos rf (refExp a) 
+                         then let r = mkApp9 t mkNoSrcPos rf (refExp a) 
                                         (refExp b) (refExp c) (refExp d) 
                                         (refExp e) (refExp g) (refExp h)
                                         (refExp i) (refExp j)
-                              in  wrapResult mkNoSrcPos r 
+                              in  recordChild t r `seq`
+                                  wrapResult mkNoSrcPos r 
                                     (f a b c d e g h i j r)
                          else f a b c d e g h i j t))
                        t))
@@ -832,11 +852,12 @@ fun10 var sr p f =
                      R (Fun (\t j ->
                        R (Fun (\t k ->
                        if hidden t 
-                         then let r = mkApp10 p mkNoSrcPos rf (refExp a) 
+                         then let r = mkApp10 t mkNoSrcPos rf (refExp a) 
                                         (refExp b) (refExp c) (refExp d) 
                                         (refExp e) (refExp g) (refExp h)
                                         (refExp i) (refExp j) (refExp k)
-                              in  wrapResult mkNoSrcPos r 
+                              in  recordChild t r `seq` 
+                                  wrapResult mkNoSrcPos r 
                                     (f a b c d e g h i j k r)
                          else f a b c d e g h i j k t))
                          t))
@@ -869,12 +890,13 @@ fun11 var sr p f =
                        R (Fun (\t k ->
                          R (Fun (\t l ->
                        if hidden t 
-                         then let r = mkApp11 p mkNoSrcPos rf (refExp a) 
+                         then let r = mkApp11 t mkNoSrcPos rf (refExp a) 
                                         (refExp b) (refExp c) (refExp d) 
                                         (refExp e) (refExp g) (refExp h)
                                         (refExp i) (refExp j) (refExp k)
                                         (refExp l)
-                              in  wrapResult mkNoSrcPos r 
+                              in  recordChild t r `seq` 
+                                  wrapResult mkNoSrcPos r 
                                     (f a b c d e g h i j k l r)
                          else f a b c d e g h i j k l t))
                            t))
@@ -909,12 +931,13 @@ fun12 var sr p f =
                          R (Fun (\t l ->
                            R (Fun (\t m ->
                        if hidden t 
-                         then let r = mkApp12 p mkNoSrcPos rf (refExp a) 
+                         then let r = mkApp12 t mkNoSrcPos rf (refExp a) 
                                         (refExp b) (refExp c) (refExp d) 
                                         (refExp e) (refExp g) (refExp h)
                                         (refExp i) (refExp j) (refExp k)
                                         (refExp l) (refExp m)
-                              in  wrapResult mkNoSrcPos r 
+                              in  recordChild t r `seq` 
+                                  wrapResult mkNoSrcPos r 
                                     (f a b c d e g h i j k l m r)
                          else f a b c d e g h i j k l m t))
                              t))
@@ -951,12 +974,13 @@ fun13 var sr p f =
                            R (Fun (\t m ->
                              R (Fun (\t n ->
                        if hidden t 
-                         then let r = mkApp13 p mkNoSrcPos rf (refExp a) 
+                         then let r = mkApp13 t mkNoSrcPos rf (refExp a) 
                                         (refExp b) (refExp c) (refExp d) 
                                         (refExp e) (refExp g) (refExp h)
                                         (refExp i) (refExp j) (refExp k)
                                         (refExp l) (refExp m) (refExp n)
-                              in  wrapResult mkNoSrcPos r 
+                              in  recordChild t r `seq` 
+                                  wrapResult mkNoSrcPos r 
                                     (f a b c d e g h i j k l m n r)
                          else f a b c d e g h i j k l m n t))
                                t))
@@ -995,13 +1019,14 @@ fun14 var sr p f =
                              R (Fun (\t n ->
                                R (Fun (\t o ->
                        if hidden t 
-                         then let r = mkApp14 p mkNoSrcPos rf (refExp a) 
+                         then let r = mkApp14 t mkNoSrcPos rf (refExp a) 
                                         (refExp b) (refExp c) (refExp d) 
                                         (refExp e) (refExp g) (refExp h)
                                         (refExp i) (refExp j) (refExp k)
                                         (refExp l) (refExp m) (refExp n)
                                         (refExp o)
-                              in  wrapResult mkNoSrcPos r 
+                              in  recordChild t r `seq` 
+                                  wrapResult mkNoSrcPos r 
                                     (f a b c d e g h i j k l m n o r)
                          else f a b c d e g h i j k l m n o t))
                                  t))
@@ -1042,13 +1067,14 @@ fun15 var sr p f =
                                R (Fun (\t o ->
                                  R (Fun (\t q ->
                        if hidden t 
-                         then let r = mkApp15 p mkNoSrcPos rf (refExp a) 
+                         then let r = mkApp15 t mkNoSrcPos rf (refExp a) 
                                         (refExp b) (refExp c) (refExp d) 
                                         (refExp e) (refExp g) (refExp h)
                                         (refExp i) (refExp j) (refExp k)
                                         (refExp l) (refExp m) (refExp n)
                                         (refExp o) (refExp q)
-                              in  wrapResult mkNoSrcPos r 
+                              in  recordChild t r `seq` 
+                                  wrapResult mkNoSrcPos r 
                                     (f a b c d e g h i j k l m n o q r)
                          else f a b c d e g h i j k l m n o q t))
                                    t))
@@ -1924,82 +1950,82 @@ mkResApp15 p sr f a1 a2 a3 a4 a5 a6 a7 a8 a9 a10 a11 a12 a13 a14 a15 =
 -- optimising combinators for application of functions of known arity
 
 -- traced function in traced or trusted application
-app1 :: RefSrcPos -> RefExp -> RefAtom 
+app1 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
      -> (R a -> RefExp -> R z) -> R a -> R z
-app1 sr p af hf a@(R _ ra) =
-  let rf = mkValueUse p sr af
+app1 sr srFun p af hf a@(R _ ra) =
+  let rf = mkValueUse p srFun af
       r = mkApp1 p sr rf ra
   in wrapResult sr r (hf a r)
 
-app2 :: RefSrcPos -> RefExp -> RefAtom 
+app2 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
      -> (R a -> R b -> RefExp -> R z) -> R a -> R b -> R z
-app2 sr p af hf a@(R _ ra) b@(R _ rb) =
-  let rf = mkValueUse p sr af
+app2 sr srFun p af hf a@(R _ ra) b@(R _ rb) =
+  let rf = mkValueUse p srFun af
       r = mkApp2 p sr rf ra rb
   in wrapResult sr r (hf a b r)
 
-app3 :: RefSrcPos -> RefExp -> RefAtom 
+app3 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
      -> (R a -> R b -> R c -> RefExp -> R z) -> R a -> R b -> R c -> R z
-app3 sr p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) =
-  let rf = mkValueUse p sr af
+app3 sr srFun p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) =
+  let rf = mkValueUse p srFun af
       r = mkApp3 p sr rf ra rb rc
   in wrapResult sr r (hf a b c r)
 
-app4 :: RefSrcPos -> RefExp -> RefAtom 
+app4 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
      -> (R a -> R b -> R c -> R d -> RefExp -> R z) 
      -> R a -> R b -> R c -> R d -> R z
-app4 sr p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) d@(R _ rd) =
-  let rf = mkValueUse p sr af
+app4 sr srFun p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) d@(R _ rd) =
+  let rf = mkValueUse p srFun af
       r = mkApp4 p sr rf ra rb rc rd
   in wrapResult sr r (hf a b c d r)
 
-app5 :: RefSrcPos -> RefExp -> RefAtom 
+app5 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
      -> (R a -> R b -> R c -> R d -> R e -> RefExp -> R z) 
      -> R a -> R b -> R c -> R d -> R e -> R z
-app5 sr p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) d@(R _ rd) e@(R _ re) =
-  let rf = mkValueUse p sr af
+app5 sr srFun p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) d@(R _ rd) e@(R _ re) =
+  let rf = mkValueUse p srFun af
       r = mkApp5 p sr rf ra rb rc rd re
   in wrapResult sr r (hf a b c d e r)
 
 -- trusted function in traced application
-uapp1 :: RefSrcPos -> RefExp -> RefAtom 
+uapp1 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
       -> (R a -> RefExp -> R z) -> R a -> R z
-uapp1 sr p af hf a@(R _ ra) =
-  let rf = mkValueUse p sr af
+uapp1 sr srFun p af hf a@(R _ ra) =
+  let rf = mkValueUse p srFun af
       r = mkApp1 p sr rf ra 
       h = mkHidden r
   in wrapResult sr r (wrapResult mkNoSrcPos h (hf a h))
 
-uapp2 :: RefSrcPos -> RefExp -> RefAtom 
+uapp2 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
       -> (R a -> R b -> RefExp -> R z) -> R a -> R b -> R z
-uapp2 sr p af hf a@(R _ ra) b@(R _ rb) =
-  let rf = mkValueUse p sr af
+uapp2 sr srFun p af hf a@(R _ ra) b@(R _ rb) =
+  let rf = mkValueUse p srFun af
       r = mkApp2 p sr rf ra rb
       h = mkHidden r
   in wrapResult sr r (wrapResult mkNoSrcPos h (hf a b h))
 
-uapp3 :: RefSrcPos -> RefExp -> RefAtom 
+uapp3 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
       -> (R a -> R b -> R c -> RefExp -> R z) -> R a -> R b -> R c -> R z
-uapp3 sr p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) =
-  let rf = mkValueUse p sr af
+uapp3 sr srFun p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) =
+  let rf = mkValueUse p srFun af
       r = mkApp3 p sr rf ra rb rc
       h = mkHidden r
   in wrapResult sr r (wrapResult mkNoSrcPos h (hf a b c h))
 
-uapp4 :: RefSrcPos -> RefExp -> RefAtom 
+uapp4 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
       -> (R a -> R b -> R c -> R d -> RefExp -> R z) 
       -> R a -> R b -> R c -> R d -> R z
-uapp4 sr p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) d@(R _ rd) =
-  let rf = mkValueUse p sr af
+uapp4 sr srFun p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) d@(R _ rd) =
+  let rf = mkValueUse p srFun af
       r = mkApp4 p sr rf ra rb rc rd
       h = mkHidden r
   in wrapResult sr r (wrapResult mkNoSrcPos h (hf a b c d h))
 
-uapp5 :: RefSrcPos -> RefExp -> RefAtom 
+uapp5 :: RefSrcPos -> RefSrcPos -> RefExp -> RefAtom 
       -> (R a -> R b -> R c -> R d -> R e -> RefExp -> R z) 
       -> R a -> R b -> R c -> R d -> R e -> R z
-uapp5 sr p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) d@(R _ rd) e@(R _ re) =
-  let rf = mkValueUse p sr af
+uapp5 sr srFun p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) d@(R _ rd) e@(R _ re) =
+  let rf = mkValueUse p srFun af
       r = mkApp5 p sr rf ra rb rc rd re
       h = mkHidden r
   in wrapResult sr r (wrapResult mkNoSrcPos h (hf a b c d e h))
@@ -2012,16 +2038,16 @@ uapp5 sr p af hf a@(R _ ra) b@(R _ rb) c@(R _ rc) d@(R _ rd) e@(R _ re) =
 openTrace :: String -> Prelude.IO ()
 openTrace progname = withCString progname openTrace'
 
-FOREIGN("hat_Open")
+FOREIGN("hat-c.h hat_Open")
   openTrace' :: CString -> Prelude.IO ()
 
-FOREIGN("hat_Close")
+FOREIGN("hat-c.h hat_Close")
   closeTrace :: Prelude.IO ()
 
 hatAborted :: String -> Prelude.IO ()
 hatAborted msg = withCString msg hatAbort'
 
-FOREIGN("hat_Abort")
+FOREIGN("hat-c.h hat_Abort")
   hatAbort' :: CString -> Prelude.IO ()
 
 errorTraceExit :: String -> RefExp -> Int -> ()
@@ -2029,19 +2055,19 @@ errorTraceExit msg ref no =
   unsafePerformIO $
     withCString msg (\msg'->errorTraceExit' msg' ref no)
 
-FOREIGN("hat_ErrorExit")
+FOREIGN("hat-c.h hat_ErrorExit")
   errorTraceExit' :: CString -> RefExp -> Int -> Prelude.IO ()
 
 outputTrace :: RefExp -> String -> Prelude.IO ()
 outputTrace trace output = withCString output (outputTrace' trace)
 
-FOREIGN("hat_OutputTrace")
+FOREIGN("hat-c.h hat_OutputTrace")
   outputTrace' :: RefExp -> CString -> Prelude.IO ()
 
-FOREIGN("hat_Hidden")
+FOREIGN("hat-c.h hat_Hidden")
   hidden :: RefExp -> Bool
 
-FOREIGN("mkRoot")
+FOREIGN("hat-c.h mkRoot")
   mkRoot :: RefExp
 
 mkModule :: String -> String -> Bool -> RefModule
@@ -2051,149 +2077,149 @@ mkModule modName srcFile traced =
     withCString srcFile (\srcFile' ->
       mkModule' modName' srcFile' traced))
 
-FOREIGN("mkModule")
+FOREIGN("hat-c.h mkModule")
   mkModule' :: CString -> CString -> Bool -> Prelude.IO RefModule
 
-FOREIGN("mkSrcPos")
-  mkSrcPos :: RefModule -> Pos -> RefSrcPos
+FOREIGN("hat-c.h mkSrcPos")
+  mkSrcPos :: RefModule -> Loc -> Loc -> RefSrcPos
 
 -- Exp nodes
 
-FOREIGN("mkResApp1")
+FOREIGN("hat-c.h mkResApp1")
   mkResApp1 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp1")
+FOREIGN("hat-c.h mkApp1")
   mkApp1 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp2")
+FOREIGN("hat-c.h mkApp2")
   mkApp2 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp3")
+FOREIGN("hat-c.h mkApp3")
   mkApp3 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
          -> RefExp
 
-FOREIGN("mkApp4")
+FOREIGN("hat-c.h mkApp4")
   mkApp4 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp
 
-FOREIGN("mkApp5")
+FOREIGN("hat-c.h mkApp5")
   mkApp5 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp6")
+FOREIGN("hat-c.h mkApp6")
   mkApp6 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp7")
+FOREIGN("hat-c.h mkApp7")
   mkApp7 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp8")
+FOREIGN("hat-c.h mkApp8")
   mkApp8 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp9")
+FOREIGN("hat-c.h mkApp9")
   mkApp9 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp10")
+FOREIGN("hat-c.h mkApp10")
   mkApp10 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
           -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
           -> RefExp
 
-FOREIGN("mkApp11")
+FOREIGN("hat-c.h mkApp11")
   mkApp11 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
           -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
           -> RefExp -> RefExp
 
-FOREIGN("mkApp12")
+FOREIGN("hat-c.h mkApp12")
   mkApp12 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
           -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
           -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp13")
+FOREIGN("hat-c.h mkApp13")
   mkApp13 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
           -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
           -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp14")
+FOREIGN("hat-c.h mkApp14")
   mkApp14 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
           -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
           -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkApp15")
+FOREIGN("hat-c.h mkApp15")
   mkApp15 :: RefExp -> RefSrcPos -> RefExp -> RefExp -> RefExp -> RefExp 
           -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
           -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp1")
+FOREIGN("hat-c.h mkValueApp1")
   mkValueApp1 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp
 
-FOREIGN("mkValueApp2")
+FOREIGN("hat-c.h mkValueApp2")
   mkValueApp2 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp3")
+FOREIGN("hat-c.h mkValueApp3")
   mkValueApp3 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp
 
-FOREIGN("mkValueApp4")
+FOREIGN("hat-c.h mkValueApp4")
   mkValueApp4 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp
 
-FOREIGN("mkValueApp5")
+FOREIGN("hat-c.h mkValueApp5")
   mkValueApp5 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp6")
+FOREIGN("hat-c.h mkValueApp6")
   mkValueApp6 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp7")
+FOREIGN("hat-c.h mkValueApp7")
   mkValueApp7 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp8")
+FOREIGN("hat-c.h mkValueApp8")
   mkValueApp8 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp9")
+FOREIGN("hat-c.h mkValueApp9")
   mkValueApp9 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp10")
+FOREIGN("hat-c.h mkValueApp10")
   mkValueApp10 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
          -> RefExp
 
-FOREIGN("mkValueApp11")
+FOREIGN("hat-c.h mkValueApp11")
   mkValueApp11 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
          -> RefExp -> RefExp
 
-FOREIGN("mkValueApp12")
+FOREIGN("hat-c.h mkValueApp12")
   mkValueApp12 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
          -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp13")
+FOREIGN("hat-c.h mkValueApp13")
   mkValueApp13 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
          -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp14")
+FOREIGN("hat-c.h mkValueApp14")
   mkValueApp14 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkValueApp15")
+FOREIGN("hat-c.h mkValueApp15")
   mkValueApp15 :: RefExp -> RefSrcPos -> RefAtom -> RefExp -> RefExp -> RefExp 
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
          -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp -> RefExp
 
-FOREIGN("mkChar")
+FOREIGN("hat-c.h mkChar")
   mkChar :: RefExp -> RefSrcPos -> Char -> RefExp
 
-FOREIGN("mkInt")
+FOREIGN("hat-c.h mkInt")
   mkInt :: RefExp -> RefSrcPos -> Int -> RefExp
 
 mkInteger :: RefExp -> RefSrcPos -> String -> RefExp
@@ -2201,10 +2227,10 @@ mkInteger parent use integer =
   unsafePerformIO $
     withCString integer (\integer' -> mkInteger' parent use integer')
 
-FOREIGN("mkInteger")
+FOREIGN("hat-c.h mkInteger")
   mkInteger' :: RefExp -> RefSrcPos -> CString -> Prelude.IO RefExp
 
-FOREIGN("mkRat")
+FOREIGN("hat-c.h mkRat")
   mkRat :: RefExp -> RefSrcPos -> Int -> Int -> RefExp
 
 mkRational :: RefExp -> RefSrcPos -> String -> String -> RefExp
@@ -2213,112 +2239,122 @@ mkRational parent use num denom =
     withCString num (\num' ->
     withCString denom (\denom' -> mkRational' parent use num' denom'))
 
-FOREIGN("mkRational")
+FOREIGN("hat-c.h mkRational")
   mkRational' :: RefExp -> RefSrcPos -> CString -> CString -> Prelude.IO RefExp
 
-FOREIGN("mkFloat")
+FOREIGN("hat-c.h mkFloat")
   mkFloat :: RefExp -> RefSrcPos -> Float -> RefExp
 
-FOREIGN("mkDouble")
+FOREIGN("hat-c.h mkDouble")
   mkDouble :: RefExp -> RefSrcPos -> Double -> RefExp
 
-FOREIGN("mkValueUse")
+FOREIGN("hat-c.h mkValueUse")
   mkValueUse :: RefExp -> RefSrcPos -> RefAtom -> RefExp
 
-FOREIGN("mkConstUse")
+FOREIGN("hat-c.h mkConstUse")
   mkConstUse :: RefExp -> RefSrcPos -> RefExp -> RefExp
 
-FOREIGN("mkConstDef")
+FOREIGN("hat-c.h mkConstDef")
   mkConstDef :: RefExp -> RefAtom -> RefExp
 
-FOREIGN("mkGuard")
+FOREIGN("hat-c.h mkGuard")
   mkGuard :: RefExp -> RefSrcPos -> RefExp -> RefExp
 
-FOREIGN("mkCase")
+FOREIGN("hat-c.h mkCase")
   mkCase :: RefExp -> RefSrcPos -> RefExp -> RefExp
 
-FOREIGN("mkIf")
+FOREIGN("hat-c.h mkIf")
   mkIf :: RefExp -> RefSrcPos -> RefExp -> RefExp
 
-FOREIGN("mkFieldUpdate1")
+FOREIGN("hat-c.h mkFieldUpdate1")
   mkFieldUpdate1 :: RefExp -> RefSrcPos -> RefExp -> RefAtom -> RefExp 
                  -> RefExp
 
-FOREIGN("mkFieldUpdate2")
+FOREIGN("hat-c.h mkFieldUpdate2")
   mkFieldUpdate2 :: RefExp -> RefSrcPos -> RefExp -> RefAtom -> RefExp 
                  -> RefAtom -> RefExp -> RefExp
 
-FOREIGN("mkFieldUpdate3")
+FOREIGN("hat-c.h mkFieldUpdate3")
   mkFieldUpdate3 :: RefExp -> RefSrcPos -> RefExp -> RefAtom -> RefExp 
                  -> RefAtom -> RefExp -> RefAtom -> RefExp -> RefExp
 
-FOREIGN("mkProjection")
+FOREIGN("hat-c.h mkProjection")
   mkProjection :: RefExp -> RefSrcPos -> RefExp -> RefExp
 
-FOREIGN("mkHidden")
+FOREIGN("hat-c.h mkHidden")
   mkHidden :: RefExp -> RefExp
 
-FOREIGN("mkForward")
+FOREIGN("hat-c.h mkForward")
   mkForward :: RefExp -> RefExp
 
-FOREIGN("mkDoStmt")
+FOREIGN("hat-c.h mkDoStmt")
   mkDoStmt :: RefExp -> RefExp
 
-FOREIGN("mkLambda")
+FOREIGN("hat-c.h mkLambda")
   mkLambda :: RefAtom
 
-mkVariable :: RefModule -> Pos -> Fixity -> Arity -> String -> Bool -> RefAtom
-mkVariable mod pos fixity arity name local =
-  unsafePerformIO $
-    withCString name (\name' -> mkVariable' mod pos fixity arity name' local)
+FOREIGN("hat-c.h mkDoLambda")
+  mkDoLambda :: RefAtom
 
-FOREIGN("mkVariable")
-  mkVariable' :: RefModule -> Pos -> Fixity -> Arity -> CString -> Bool 
+mkVariable :: RefModule -> Loc -> Loc -> Fixity -> Arity -> String 
+           -> Bool -> RefAtom
+mkVariable mod begin end fixity arity name local =
+  unsafePerformIO $
+    withCString name 
+      (\name' -> mkVariable' mod begin end fixity arity name' local)
+
+FOREIGN("hat-c.h mkVariable")
+  mkVariable' :: RefModule -> Loc -> Loc -> Fixity -> Arity -> CString -> Bool 
               -> Prelude.IO RefAtom
 
-mkConstructor :: RefModule -> Pos -> Fixity -> Arity -> String -> RefAtom
-mkConstructor mod pos fixity arity name =
+mkConstructor :: RefModule -> Loc -> Loc -> Fixity -> Arity 
+              -> String -> RefAtom
+mkConstructor mod begin end fixity arity name =
   unsafePerformIO $
-    withCString name (\name' -> mkConstructor' mod pos fixity arity name')
+    withCString name 
+      (\name' -> mkConstructor' mod begin end fixity arity name')
 
-FOREIGN("mkConstructor")
-  mkConstructor' :: RefModule -> Pos -> Fixity -> Arity -> CString 
+FOREIGN("hat-c.h mkConstructor")
+  mkConstructor' :: RefModule -> Loc -> Loc -> Fixity -> Arity -> CString 
                  -> Prelude.IO RefAtom
 
-mkConstructorWFields :: RefModule -> Pos -> Fixity -> Arity -> String
-                        -> [RefAtom] -> RefAtom
-mkConstructorWFields mod pos fixity arity name labels =
+mkConstructorWFields :: RefModule -> Loc -> Loc -> Fixity -> Arity -> String
+                     -> [RefAtom] -> RefAtom
+mkConstructorWFields mod begin end fixity arity name labels =
   unsafePerformIO $
     withCString name (\name' ->
     withArray (map (\(RA i) -> i) labels) (\labels' ->
-      mkConstructorWFields' mod pos fixity arity name' labels'))
+      mkConstructorWFields' mod begin end fixity arity name' labels'))
 
-FOREIGN("mkConstructorWFields")
-  mkConstructorWFields' :: RefModule -> Pos -> Fixity -> Arity -> CString
-                        -> Ptr Int -> Prelude.IO RefAtom
+FOREIGN("hat-c.h mkConstructorWFields")
+  mkConstructorWFields' :: RefModule -> Loc -> Loc -> Fixity -> Arity 
+                        -> CString -> Ptr Int -> Prelude.IO RefAtom
 
 mkAbstract :: String -> RefAtom
 mkAbstract txt = unsafePerformIO $
   withCString txt (\txt' -> mkAbstract' txt')
 
-FOREIGN("mkAbstract")
+FOREIGN("hat-c.h mkAbstract")
   mkAbstract' :: CString -> Prelude.IO RefAtom
 
 -- Update node that it was entered
 
-FOREIGN("entResult")
+FOREIGN("hat-c.h entResult")
   entResult :: RefExp -> RefSrcPos -> ()
 
-FOREIGN("entForward")
+FOREIGN("hat-c.h entForward")
   entForward :: RefExp -> RefExp -> ()
 
 -- Update node with result
 
-FOREIGN("resResult")
+FOREIGN("hat-c.h resResult")
   resResult :: RefExp -> RefExp -> RefSrcPos -> ()
 
-FOREIGN("resForward")
+FOREIGN("hat-c.h resForward")
   resForward :: RefExp -> RefExp -> ()
 
+FOREIGN("hat-c.h recordChild")
+  recordChild :: RefExp -> RefExp -> ()
+
 -- ----------------------------------------------------------------------------
 -- End
