@@ -31,9 +31,11 @@ module Hat.IO
     ,greadFile,areadFile,hreadFile,gwriteFile,awriteFile,hwriteFile,gappendFile
     ,aappendFile,happendFile,greadIO,areadIO,hreadIO,greadLn) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 import Hat.Ix  (Ix(grange,gindex,ginRange,srange,sindex,sinRange))
 import Hat.PreludeBuiltinTypes 
@@ -73,7 +75,8 @@ instance Eq (Handle)
   
   (!==) (%==) p = T.uconstUse (%==) p (|==)
   
-  (|==) = T.uconstDef T.mkRoot (+&$=&==) (\ p -> gprimHandleEq T.mkNoSrcPos p)
+  (|==) =
+    T.uconstDef T.mkRoot (+&$=&=&$=$#==) (\ p -> gprimHandleEq T.mkNoSrcPos p)
   
 
 instance Show (Handle)
@@ -82,7 +85,7 @@ instance Show (Handle)
   gshowsPrec pshowsPrec p = T.uconstUse pshowsPrec p sshowsPrec
   
   sshowsPrec =
-    T.uconstDef T.mkRoot a45v3showsPrec
+    T.uconstDef T.mkRoot a45v3v45v33showsPrec
       (\ p -> gprimHandleShowsPrec T.mkNoSrcPos p)
   
 
@@ -104,7 +107,8 @@ instance Eq (HandlePosn)
   (!==) (%==) p = T.uconstUse (%==) p (|==)
   
   (|==) =
-    T.uconstDef T.mkRoot (+*%=&==) (\ p -> gprimHandlePosnEq T.mkNoSrcPos p)
+    T.uconstDef T.mkRoot (+*%=&=*%=$*==)
+      (\ p -> gprimHandlePosnEq T.mkNoSrcPos p)
   
 
 gstdin :: T.RefSrcPos -> T.RefExp -> T.R Handle
@@ -465,10 +469,11 @@ htry ff p =
   T.uwrapForward p
     (hcatch
       (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!>>= p) ff
-        (T.ufun1 T.mkLambda T.mkNoSrcPos p
+        (T.ufun1 T.mkDoLambda T.mkNoSrcPos p
           (\ fr p ->
             T.uap1 T.mkNoSrcPos p (greturn T.mkNoSrcPos p)
-              (T.con1 T.mkNoSrcPos p Right aRight fr))))
+              (T.con1 T.mkNoSrcPos p Right aRight
+                (T.projection T.mkNoSrcPos p fr)))))
       (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !. p) (greturn T.mkNoSrcPos p)
         (T.pa0 Left T.cn1 T.mkNoSrcPos p aLeft)) p)
 
@@ -486,21 +491,22 @@ gbracket pbracket p = T.ufun3 abracket pbracket p hbracket
 
 hbracket fbefore fafter fm p =
   T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!>>= p) fbefore
-    (T.ufun1 T.mkLambda T.mkNoSrcPos p
+    (T.ufun1 T.mkDoLambda T.mkNoSrcPos p
       (\ fx p ->
         T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!>>= p)
           (T.uwrapForward p (htry (T.uap1 T.mkNoSrcPos p fm fx) p))
-          (T.ufun1 T.mkLambda T.mkNoSrcPos p
+          (T.ufun1 T.mkDoLambda T.mkNoSrcPos p
             (\ frs p ->
               T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!>> p)
                 (T.uap1 T.mkNoSrcPos p fafter fx)
                 (T.uccase T.mkNoSrcPos p
                   (let
-                    v172v9v1 (T.R (Right fr) _) p =
+                    v172v9v174v31v1 (T.R (Right fr) _) p =
                       T.uap1 T.mkNoSrcPos p (greturn T.mkNoSrcPos p) fr
-                    v172v9v1 (T.R (Left fe) _) p =
+                    v172v9v174v31v1 (T.R (Left fe) _) p =
                       T.uwrapForward p (hioError fe p)
-                    v172v9v1 _ p = T.fatal p in (v172v9v1)) frs)))))
+                    v172v9v174v31v1 _ p = T.fatal p in (v172v9v174v31v1))
+                  frs)))))
 
 gbracket_ ::
   T.RefSrcPos ->
@@ -515,125 +521,134 @@ gbracket_ pbracket_ p = T.ufun3 abracket_ pbracket_ p hbracket_
 
 hbracket_ fbefore fafter fm p =
   T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!>>= p) fbefore
-    (T.ufun1 T.mkLambda T.mkNoSrcPos p
+    (T.ufun1 T.mkDoLambda T.mkNoSrcPos p
       (\ fx p ->
         T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!>>= p)
           (T.uwrapForward p (htry fm p))
-          (T.ufun1 T.mkLambda T.mkNoSrcPos p
+          (T.ufun1 T.mkDoLambda T.mkNoSrcPos p
             (\ frs p ->
               T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!>> p)
                 (T.uap1 T.mkNoSrcPos p fafter fx)
                 (T.uccase T.mkNoSrcPos p
                   (let
-                    v182v10v1 (T.R (Right fr) _) p =
+                    v182v10v184v32v1 (T.R (Right fr) _) p =
                       T.uap1 T.mkNoSrcPos p (greturn T.mkNoSrcPos p) fr
-                    v182v10v1 (T.R (Left fe) _) p =
+                    v182v10v184v32v1 (T.R (Left fe) _) p =
                       T.uwrapForward p (hioError fe p)
-                    v182v10v1 _ p = T.fatal p in (v182v10v1)) frs)))))
+                    v182v10v184v32v1 _ p = T.fatal p in (v182v10v184v32v1))
+                  frs)))))
 
 tIO = T.mkModule "IO" "IO.hs" Prelude.False
 
-aprimHandleEq = T.mkVariable tIO 360002 3 2 "primHandleEq" Prelude.False
+aprimHandleEq = T.mkVariable tIO 350001 360041 3 2 "primHandleEq" Prelude.False
 
 aprimHandleShowsPrec =
-  T.mkVariable tIO 390002 3 3 "primHandleShowsPrec" Prelude.False
+  T.mkVariable tIO 380001 390057 3 3 "primHandleShowsPrec" Prelude.False
 
-(+&$=&==) = T.mkVariable tIO 420004 16 0 "==" Prelude.False
+(+&$=&=&$=$#==) = T.mkVariable tIO 420004 420021 16 0 "==" Prelude.False
 
-a45v3showsPrec = T.mkVariable tIO 450003 3 0 "showsPrec" Prelude.False
+a45v3v45v33showsPrec =
+  T.mkVariable tIO 450003 450033 3 0 "showsPrec" Prelude.False
 
-aprimHandlePosnEq = T.mkVariable tIO 500002 3 2 "primHandlePosnEq" Prelude.False
+aprimHandlePosnEq =
+  T.mkVariable tIO 490001 500053 3 2 "primHandlePosnEq" Prelude.False
 
-(+*%=&==) = T.mkVariable tIO 530004 16 0 "==" Prelude.False
+(+*%=&=*%=$*==) = T.mkVariable tIO 530004 530025 16 0 "==" Prelude.False
 
-astdin = T.mkVariable tIO 650002 3 0 "stdin" Prelude.False
+astdin = T.mkVariable tIO 640001 650016 3 0 "stdin" Prelude.False
 
-astdout = T.mkVariable tIO 670002 3 0 "stdout" Prelude.False
+astdout = T.mkVariable tIO 660001 670017 3 0 "stdout" Prelude.False
 
-astderr = T.mkVariable tIO 690002 3 0 "stderr" Prelude.False
+astderr = T.mkVariable tIO 680001 690017 3 0 "stderr" Prelude.False
 
-aopenFile = T.mkVariable tIO 720002 3 2 "openFile" Prelude.False
+aopenFile = T.mkVariable tIO 710001 720048 3 2 "openFile" Prelude.False
 
-ahClose = T.mkVariable tIO 740002 3 1 "hClose" Prelude.False
+ahClose = T.mkVariable tIO 730001 740038 3 1 "hClose" Prelude.False
 
-ahFileSize = T.mkVariable tIO 770002 3 1 "hFileSize" Prelude.False
+ahFileSize = T.mkVariable tIO 760001 770038 3 1 "hFileSize" Prelude.False
 
-ahIsEOF = T.mkVariable tIO 790002 3 1 "hIsEOF" Prelude.False
+ahIsEOF = T.mkVariable tIO 780001 790038 3 1 "hIsEOF" Prelude.False
 
-aisEOF = T.mkVariable tIO 820001 3 0 "isEOF" Prelude.False
+aisEOF = T.mkVariable tIO 820001 820037 3 0 "isEOF" Prelude.False
 
-ahSetBuffering = T.mkVariable tIO 850002 3 2 "hSetBuffering" Prelude.False
+ahSetBuffering =
+  T.mkVariable tIO 840001 850053 3 2 "hSetBuffering" Prelude.False
 
-ahGetBuffering = T.mkVariable tIO 870002 3 1 "hGetBuffering" Prelude.False
+ahGetBuffering =
+  T.mkVariable tIO 860001 870039 3 1 "hGetBuffering" Prelude.False
 
-ahFlush = T.mkVariable tIO 890002 3 1 "hFlush" Prelude.False
+ahFlush = T.mkVariable tIO 880001 890038 3 1 "hFlush" Prelude.False
 
-ahGetPosn = T.mkVariable tIO 910002 3 1 "hGetPosn" Prelude.False
+ahGetPosn = T.mkVariable tIO 900001 910038 3 1 "hGetPosn" Prelude.False
 
-ahSetPosn = T.mkVariable tIO 930002 3 1 "hSetPosn" Prelude.False
+ahSetPosn = T.mkVariable tIO 920001 930042 3 1 "hSetPosn" Prelude.False
 
-ahSeek = T.mkVariable tIO 950002 3 3 "hSeek" Prelude.False
+ahSeek = T.mkVariable tIO 940001 950061 3 3 "hSeek" Prelude.False
 
-ahWaitForInput = T.mkVariable tIO 980002 3 2 "hWaitForInput" Prelude.False
+ahWaitForInput =
+  T.mkVariable tIO 970001 980043 3 2 "hWaitForInput" Prelude.False
 
-ahReady = T.mkVariable tIO 1010001 3 1 "hReady" Prelude.False
+ahReady = T.mkVariable tIO 1010001 1010028 3 1 "hReady" Prelude.False
 
-ahGetChar = T.mkVariable tIO 1040002 3 1 "hGetChar" Prelude.False
+ahGetChar = T.mkVariable tIO 1030001 1040038 3 1 "hGetChar" Prelude.False
 
-ahGetLine = T.mkVariable tIO 1060002 3 1 "hGetLine" Prelude.False
+ahGetLine = T.mkVariable tIO 1050001 1060038 3 1 "hGetLine" Prelude.False
 
-ahLookAhead = T.mkVariable tIO 1080002 3 1 "hLookAhead" Prelude.False
+ahLookAhead = T.mkVariable tIO 1070001 1080038 3 1 "hLookAhead" Prelude.False
 
-ahGetContents = T.mkVariable tIO 1100002 3 1 "hGetContents" Prelude.False
+ahGetContents =
+  T.mkVariable tIO 1090001 1100038 3 1 "hGetContents" Prelude.False
 
-ahPutChar = T.mkVariable tIO 1120002 3 2 "hPutChar" Prelude.False
+ahPutChar = T.mkVariable tIO 1110001 1120046 3 2 "hPutChar" Prelude.False
 
-ahPutStr = T.mkVariable tIO 1140002 3 2 "hPutStr" Prelude.False
+ahPutStr = T.mkVariable tIO 1130001 1140048 3 2 "hPutStr" Prelude.False
 
-ahPutStrLn = T.mkVariable tIO 1170001 3 2 "hPutStrLn" Prelude.False
+ahPutStrLn = T.mkVariable tIO 1170001 1180038 3 2 "hPutStrLn" Prelude.False
 
-ahPrint = T.mkVariable tIO 1210001 3 2 "hPrint" Prelude.False
+ahPrint = T.mkVariable tIO 1210001 1210032 3 2 "hPrint" Prelude.False
 
-ahIsOpen = T.mkVariable tIO 1240002 3 1 "hIsOpen" Prelude.False
+ahIsOpen = T.mkVariable tIO 1230001 1240038 3 1 "hIsOpen" Prelude.False
 
-ahIsClosed = T.mkVariable tIO 1260002 3 1 "hIsClosed" Prelude.False
+ahIsClosed = T.mkVariable tIO 1250001 1260038 3 1 "hIsClosed" Prelude.False
 
-ahIsReadable = T.mkVariable tIO 1280002 3 1 "hIsReadable" Prelude.False
+ahIsReadable = T.mkVariable tIO 1270001 1280038 3 1 "hIsReadable" Prelude.False
 
-ahIsWritable = T.mkVariable tIO 1300002 3 1 "hIsWritable" Prelude.False
+ahIsWritable = T.mkVariable tIO 1290001 1300038 3 1 "hIsWritable" Prelude.False
 
-ahIsSeekable = T.mkVariable tIO 1320002 3 1 "hIsSeekable" Prelude.False
+ahIsSeekable = T.mkVariable tIO 1310001 1320038 3 1 "hIsSeekable" Prelude.False
 
 aisAlreadyExistsError =
-  T.mkVariable tIO 1350002 3 1 "isAlreadyExistsError" Prelude.False
+  T.mkVariable tIO 1340001 1350041 3 1 "isAlreadyExistsError" Prelude.False
 
 aisDoesNotExistError =
-  T.mkVariable tIO 1370002 3 1 "isDoesNotExistError" Prelude.False
+  T.mkVariable tIO 1360001 1370041 3 1 "isDoesNotExistError" Prelude.False
 
 aisAlreadyInUseError =
-  T.mkVariable tIO 1390002 3 1 "isAlreadyInUseError" Prelude.False
+  T.mkVariable tIO 1380001 1390041 3 1 "isAlreadyInUseError" Prelude.False
 
-aisFullError = T.mkVariable tIO 1410002 3 1 "isFullError" Prelude.False
+aisFullError = T.mkVariable tIO 1400001 1410041 3 1 "isFullError" Prelude.False
 
-aisEOFError = T.mkVariable tIO 1430002 3 1 "isEOFError" Prelude.False
+aisEOFError = T.mkVariable tIO 1420001 1430041 3 1 "isEOFError" Prelude.False
 
 aisIllegalOperation =
-  T.mkVariable tIO 1450002 3 1 "isIllegalOperation" Prelude.False
+  T.mkVariable tIO 1440001 1450041 3 1 "isIllegalOperation" Prelude.False
 
 aisPermissionError =
-  T.mkVariable tIO 1470002 3 1 "isPermissionError" Prelude.False
+  T.mkVariable tIO 1460001 1470041 3 1 "isPermissionError" Prelude.False
 
-aisUserError = T.mkVariable tIO 1490002 3 1 "isUserError" Prelude.False
+aisUserError = T.mkVariable tIO 1480001 1490041 3 1 "isUserError" Prelude.False
 
 aioeGetErrorString =
-  T.mkVariable tIO 1520002 3 1 "ioeGetErrorString" Prelude.False
+  T.mkVariable tIO 1510001 1520043 3 1 "ioeGetErrorString" Prelude.False
 
-aioeGetHandle = T.mkVariable tIO 1540002 3 1 "ioeGetHandle" Prelude.False
+aioeGetHandle =
+  T.mkVariable tIO 1530001 1540042 3 1 "ioeGetHandle" Prelude.False
 
-aioeGetFileName = T.mkVariable tIO 1570002 3 1 "ioeGetFileName" Prelude.False
+aioeGetFileName =
+  T.mkVariable tIO 1560001 1570042 3 1 "ioeGetFileName" Prelude.False
 
-atry = T.mkVariable tIO 1630001 3 1 "try" Prelude.False
+atry = T.mkVariable tIO 1630001 1650038 3 1 "try" Prelude.False
 
-abracket = T.mkVariable tIO 1680001 3 3 "bracket" Prelude.False
+abracket = T.mkVariable tIO 1680001 1740031 3 3 "bracket" Prelude.False
 
-abracket_ = T.mkVariable tIO 1780001 3 3 "bracket_" Prelude.False
+abracket_ = T.mkVariable tIO 1780001 1840032 3 3 "bracket_" Prelude.False
