@@ -1,17 +1,21 @@
 module Hat.IOBuiltinTypes
   (IOMode(..),BufferMode(..),SeekMode(..),aReadMode,aWriteMode,aAppendMode
     ,aReadWriteMode,aNoBuffering,aLineBuffering,aBlockBuffering,aAbsoluteSeek
-    ,aRelativeSeek,aSeekFromEnd,(++=%$==),a6v36compare,a6v41range,a6v41index
-    ,a6v41inRange,a6v45minBound,a6v45maxBound,a6v54fromEnum,a6v54toEnum
-    ,a6v54enumFrom,a6v54enumFromThen,a6v60readsPrec,a6v66showsPrec,(+>=%$==)
-    ,a9v36compare,a9v41readsPrec,a9v47showsPrec,(+##=%$==),a11v36compare
-    ,a11v41range,a11v41index,a11v41inRange,a11v45minBound,a11v45maxBound
-    ,a11v54fromEnum,a11v54toEnum,a11v54enumFrom,a11v54enumFromThen
-    ,a11v60readsPrec,a11v66showsPrec) where
+    ,aRelativeSeek,aSeekFromEnd,(++=%$=+=%%==),a6v36v6v38compare,a6v41v6v42range
+    ,a6v41v6v42index,a6v41v6v42inRange,a6v45v6v51minBound,a6v45v6v51maxBound
+    ,a6v54v6v57fromEnum,a6v54v6v57toEnum,a6v54v6v57enumFrom
+    ,a6v54v6v57enumFromThen,a6v60v6v63readsPrec,a6v66v6v69showsPrec
+    ,(+>=%$=>=%%==),a9v36v9v38compare,a9v41v9v44readsPrec,a9v47v9v50showsPrec
+    ,(+##=%$=##=%%==),a11v36v11v38compare,a11v41v11v42range,a11v41v11v42index
+    ,a11v41v11v42inRange,a11v45v11v51minBound,a11v45v11v51maxBound
+    ,a11v54v11v57fromEnum,a11v54v11v57toEnum,a11v54v11v57enumFrom
+    ,a11v54v11v57enumFromThen,a11v60v11v63readsPrec,a11v66v11v69showsPrec) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 import Hat.Ix  (Ix(..))
 
@@ -20,13 +24,13 @@ data IOMode = ReadMode  | WriteMode  | AppendMode  | ReadWriteMode
 instance T.WrapVal (IOMode)
   where
   
-  T.wrapVal pwrapVal (kwrapVal@ReadMode) p =
+  wrapVal pwrapVal (kwrapVal@ReadMode) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aReadMode)
-  T.wrapVal pwrapVal (kwrapVal@WriteMode) p =
+  wrapVal pwrapVal (kwrapVal@WriteMode) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aWriteMode)
-  T.wrapVal pwrapVal (kwrapVal@AppendMode) p =
+  wrapVal pwrapVal (kwrapVal@AppendMode) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aAppendMode)
-  T.wrapVal pwrapVal (kwrapVal@ReadWriteMode) p =
+  wrapVal pwrapVal (kwrapVal@ReadWriteMode) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aReadWriteMode)
   
 
@@ -34,7 +38,7 @@ instance Eq (IOMode)
   where
   
   (!==) (%==) p =
-    T.ufun2 (++=%$==) (%==) p (*==)
+    T.ufun2 (++=%$=+=%%==) (%==) p (*==)
     where
     
     (*==) (T.R ReadMode _) (T.R ReadMode _) p =
@@ -53,7 +57,7 @@ instance Ord (IOMode)
   where
   
   gcompare pcompare p =
-    T.ufun2 a6v36compare pcompare p hcompare
+    T.ufun2 a6v36v6v38compare pcompare p hcompare
     where
     
     hcompare fy1 fy2 p =
@@ -63,9 +67,9 @@ instance Ord (IOMode)
       where
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a6v36localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a6v36v6v38localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a6v36localFromEnum
+      alocalFromEnum = a6v36v6v38localFromEnum
       
       hlocalFromEnum (T.R (ReadMode) _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -88,7 +92,7 @@ instance Ix (IOMode)
   where
   
   grange prange p =
-    T.ufun1 a6v41range prange p hrange
+    T.ufun1 a6v41v6v42range prange p hrange
     where
     
     hrange (T.R (T.Tuple2 fy1 fy2) _) p =
@@ -105,52 +109,55 @@ instance Ix (IOMode)
       hlocalToEnum :: (T.R Hat.Prelude.Int) -> T.RefExp -> T.R IOMode
       
       glocalToEnum plocalToEnum p =
-        T.ufun1 a6v41localToEnum plocalToEnum p hlocalToEnum
+        T.ufun1 a6v41v6v42localToEnum plocalToEnum p hlocalToEnum
       
-      alocalToEnum = a6v41localToEnum
+      alocalToEnum = a6v41v6v42localToEnum
       
-      hlocalToEnum fv6v41n p =
+      hlocalToEnum fv6v41v6v42n p =
         T.ucguard
-          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v41n
+          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v41v6v42n
             (T.uap1 T.mkNoSrcPos p
               (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-              (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1localToEnum fv6v41n p)
+              (T.conInteger T.mkNoSrcPos p 0))) (h p)
+          (y1localToEnum fv6v41v6v42n p)
         where
         
         h p = T.con0 T.mkNoSrcPos p ReadMode aReadMode
-        h p = y1localToEnum fv6v41n p
+        h p = y1localToEnum fv6v41v6v42n p
         
-      hlocalToEnum fv6v41n p = y1localToEnum fv6v41n p
+      hlocalToEnum fv6v41v6v42n p = y1localToEnum fv6v41v6v42n p
       
-      y1localToEnum fv6v41n p =
+      y1localToEnum fv6v41v6v42n p =
         T.ucguard
-          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v41n
+          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v41v6v42n
             (T.uap1 T.mkNoSrcPos p
               (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-              (T.conInteger T.mkNoSrcPos p 1))) (h p) (y2localToEnum fv6v41n p)
+              (T.conInteger T.mkNoSrcPos p 1))) (h p)
+          (y2localToEnum fv6v41v6v42n p)
         where
         
         h p = T.con0 T.mkNoSrcPos p WriteMode aWriteMode
-        h p = y2localToEnum fv6v41n p
+        h p = y2localToEnum fv6v41v6v42n p
         
-      y1localToEnum fv6v41n p = y2localToEnum fv6v41n p
+      y1localToEnum fv6v41v6v42n p = y2localToEnum fv6v41v6v42n p
       
-      y2localToEnum fv6v41n p =
+      y2localToEnum fv6v41v6v42n p =
         T.ucguard
-          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v41n
+          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v41v6v42n
             (T.uap1 T.mkNoSrcPos p
               (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-              (T.conInteger T.mkNoSrcPos p 2))) (h p) (y3localToEnum fv6v41n p)
+              (T.conInteger T.mkNoSrcPos p 2))) (h p)
+          (y3localToEnum fv6v41v6v42n p)
         where
         
         h p = T.con0 T.mkNoSrcPos p AppendMode aAppendMode
-        h p = y3localToEnum fv6v41n p
+        h p = y3localToEnum fv6v41v6v42n p
         
-      y2localToEnum fv6v41n p = y3localToEnum fv6v41n p
+      y2localToEnum fv6v41v6v42n p = y3localToEnum fv6v41v6v42n p
       
-      y3localToEnum fv6v41n p =
+      y3localToEnum fv6v41v6v42n p =
         T.ucguard
-          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v41n
+          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v41v6v42n
             (T.uap1 T.mkNoSrcPos p
               (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
               (T.conInteger T.mkNoSrcPos p 3))) (h p) (T.fatal p)
@@ -167,9 +174,9 @@ instance Ix (IOMode)
       hlocalFromEnum :: (T.R IOMode) -> T.RefExp -> T.R Hat.Prelude.Int
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a6v41localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a6v41v6v42localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a6v41localFromEnum
+      alocalFromEnum = a6v41v6v42localFromEnum
       
       hlocalFromEnum (T.R ReadMode _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -189,7 +196,7 @@ instance Ix (IOMode)
     
   
   gindex pindex p =
-    T.ufun2 a6v41index pindex p hindex
+    T.ufun2 a6v41v6v42index pindex p hindex
     where
     
     hindex (T.R (T.Tuple2 fy1 fy2) _) fy3 p =
@@ -204,9 +211,9 @@ instance Ix (IOMode)
       hlocalFromEnum :: (T.R IOMode) -> T.RefExp -> T.R Hat.Prelude.Int
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a6v41localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a6v41v6v42localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a6v41localFromEnum
+      alocalFromEnum = a6v41v6v42localFromEnum
       
       hlocalFromEnum (T.R ReadMode _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -226,7 +233,7 @@ instance Ix (IOMode)
     
   
   ginRange pinRange p =
-    T.ufun2 a6v41inRange pinRange p hinRange
+    T.ufun2 a6v41v6v42inRange pinRange p hinRange
     where
     
     hinRange (T.R (T.Tuple2 fy1 fy2) _) fy3 p =
@@ -243,9 +250,9 @@ instance Ix (IOMode)
       hlocalFromEnum :: (T.R IOMode) -> T.RefExp -> T.R Hat.Prelude.Int
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a6v41localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a6v41v6v42localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a6v41localFromEnum
+      alocalFromEnum = a6v41v6v42localFromEnum
       
       hlocalFromEnum (T.R ReadMode _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -271,13 +278,13 @@ instance Bounded (IOMode)
   gminBound pminBound p = T.uconstUse pminBound p sminBound
   
   sminBound =
-    T.uconstDef T.mkRoot a6v45minBound
+    T.uconstDef T.mkRoot a6v45v6v51minBound
       (\ p -> T.con0 T.mkNoSrcPos p ReadMode aReadMode)
   
   gmaxBound pmaxBound p = T.uconstUse pmaxBound p smaxBound
   
   smaxBound =
-    T.uconstDef T.mkRoot a6v45maxBound
+    T.uconstDef T.mkRoot a6v45v6v51maxBound
       (\ p -> T.con0 T.mkNoSrcPos p ReadWriteMode aReadWriteMode)
   
 
@@ -285,7 +292,7 @@ instance Enum (IOMode)
   where
   
   gfromEnum pfromEnum p =
-    T.ufun1 a6v54fromEnum pfromEnum p hfromEnum
+    T.ufun1 a6v54v6v57fromEnum pfromEnum p hfromEnum
     where
     
     hfromEnum (T.R ReadMode _) p =
@@ -304,56 +311,56 @@ instance Enum (IOMode)
     
   
   gtoEnum ptoEnum p =
-    T.ufun1 a6v54toEnum ptoEnum p htoEnum
+    T.ufun1 a6v54v6v57toEnum ptoEnum p htoEnum
     where
     
-    htoEnum fv6v54n p =
+    htoEnum fv6v54v6v57n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v54n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v54v6v57n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1toEnum fv6v54n p)
+            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1toEnum fv6v54v6v57n p)
       where
       
       h p = T.con0 T.mkNoSrcPos p ReadMode aReadMode
-      h p = y1toEnum fv6v54n p
+      h p = y1toEnum fv6v54v6v57n p
       
-    htoEnum fv6v54n p = y1toEnum fv6v54n p
+    htoEnum fv6v54v6v57n p = y1toEnum fv6v54v6v57n p
     
-    y1toEnum fv6v54n p =
+    y1toEnum fv6v54v6v57n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v54n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v54v6v57n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 1))) (h p) (y2toEnum fv6v54n p)
+            (T.conInteger T.mkNoSrcPos p 1))) (h p) (y2toEnum fv6v54v6v57n p)
       where
       
       h p = T.con0 T.mkNoSrcPos p WriteMode aWriteMode
-      h p = y2toEnum fv6v54n p
+      h p = y2toEnum fv6v54v6v57n p
       
-    y1toEnum fv6v54n p = y2toEnum fv6v54n p
+    y1toEnum fv6v54v6v57n p = y2toEnum fv6v54v6v57n p
     
-    y2toEnum fv6v54n p =
+    y2toEnum fv6v54v6v57n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v54n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v54v6v57n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 2))) (h p) (y3toEnum fv6v54n p)
+            (T.conInteger T.mkNoSrcPos p 2))) (h p) (y3toEnum fv6v54v6v57n p)
       where
       
       h p = T.con0 T.mkNoSrcPos p AppendMode aAppendMode
-      h p = y3toEnum fv6v54n p
+      h p = y3toEnum fv6v54v6v57n p
       
-    y2toEnum fv6v54n p = y3toEnum fv6v54n p
+    y2toEnum fv6v54v6v57n p = y3toEnum fv6v54v6v57n p
     
-    y3toEnum fv6v54n p =
+    y3toEnum fv6v54v6v57n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v54n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv6v54v6v57n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 3))) (h p) (y4toEnum fv6v54n p)
+            (T.conInteger T.mkNoSrcPos p 3))) (h p) (y4toEnum fv6v54v6v57n p)
       where
       
       h p = T.con0 T.mkNoSrcPos p ReadWriteMode aReadWriteMode
-      h p = y4toEnum fv6v54n p
+      h p = y4toEnum fv6v54v6v57n p
       
-    y3toEnum fv6v54n p = y4toEnum fv6v54n p
+    y3toEnum fv6v54v6v57n p = y4toEnum fv6v54v6v57n p
     
     y4toEnum _ p =
       T.uwrapForward p
@@ -362,7 +369,7 @@ instance Enum (IOMode)
     
   
   genumFrom penumFrom p =
-    T.ufun1 a6v54enumFrom penumFrom p henumFrom
+    T.ufun1 a6v54v6v57enumFrom penumFrom p henumFrom
     where
     
     henumFrom fy1 p =
@@ -371,7 +378,7 @@ instance Enum (IOMode)
     
   
   genumFromThen penumFromThen p =
-    T.ufun2 a6v54enumFromThen penumFromThen p henumFromThen
+    T.ufun2 a6v54v6v57enumFromThen penumFromThen p henumFromThen
     where
     
     henumFromThen fy1 fy2 p =
@@ -389,7 +396,7 @@ instance Read (IOMode)
   where
   
   greadsPrec preadsPrec p =
-    T.ufun1 a6v60readsPrec preadsPrec p hreadsPrec
+    T.ufun1 a6v60v6v63readsPrec preadsPrec p hreadsPrec
     where
     
     hreadsPrec fy1 p =
@@ -438,7 +445,7 @@ instance Show (IOMode)
   where
   
   gshowsPrec pshowsPrec p =
-    T.ufun2 a6v66showsPrec pshowsPrec p hshowsPrec
+    T.ufun2 a6v66v6v69showsPrec pshowsPrec p hshowsPrec
     where
     
     hshowsPrec fy1 (T.R (ReadMode) _) p =
@@ -463,11 +470,11 @@ data BufferMode =
 instance T.WrapVal (BufferMode)
   where
   
-  T.wrapVal pwrapVal (kwrapVal@NoBuffering) p =
+  wrapVal pwrapVal (kwrapVal@NoBuffering) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aNoBuffering)
-  T.wrapVal pwrapVal (kwrapVal@LineBuffering) p =
+  wrapVal pwrapVal (kwrapVal@LineBuffering) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aLineBuffering)
-  T.wrapVal pwrapVal (kwrapVal@(BlockBuffering (T.R _ z1wrapVal))) p =
+  wrapVal pwrapVal (kwrapVal@(BlockBuffering (T.R _ z1wrapVal))) p =
     T.R kwrapVal (T.mkValueApp1 p pwrapVal aBlockBuffering z1wrapVal)
   
 
@@ -475,7 +482,7 @@ instance Eq (BufferMode)
   where
   
   (!==) (%==) p =
-    T.ufun2 (+>=%$==) (%==) p (*==)
+    T.ufun2 (+>=%$=>=%%==) (%==) p (*==)
     where
     
     (*==) (T.R NoBuffering _) (T.R NoBuffering _) p =
@@ -492,7 +499,7 @@ instance Ord (BufferMode)
   where
   
   gcompare pcompare p =
-    T.ufun2 a9v36compare pcompare p hcompare
+    T.ufun2 a9v36v9v38compare pcompare p hcompare
     where
     
     hcompare (T.R (BlockBuffering fy3) _) (T.R (BlockBuffering fy4) _) p =
@@ -504,9 +511,9 @@ instance Ord (BufferMode)
       where
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a9v36localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a9v36v9v38localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a9v36localFromEnum
+      alocalFromEnum = a9v36v9v38localFromEnum
       
       hlocalFromEnum (T.R (NoBuffering) _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -526,7 +533,7 @@ instance Read (BufferMode)
   where
   
   greadsPrec preadsPrec p =
-    T.ufun1 a9v41readsPrec preadsPrec p hreadsPrec
+    T.ufun1 a9v41v9v44readsPrec preadsPrec p hreadsPrec
     where
     
     hreadsPrec fy1 p =
@@ -574,7 +581,7 @@ instance Show (BufferMode)
   where
   
   gshowsPrec pshowsPrec p =
-    T.ufun2 a9v47showsPrec pshowsPrec p hshowsPrec
+    T.ufun2 a9v47v9v50showsPrec pshowsPrec p hshowsPrec
     where
     
     hshowsPrec fy1 (T.R (NoBuffering) _) p =
@@ -606,11 +613,11 @@ data SeekMode = AbsoluteSeek  | RelativeSeek  | SeekFromEnd
 instance T.WrapVal (SeekMode)
   where
   
-  T.wrapVal pwrapVal (kwrapVal@AbsoluteSeek) p =
+  wrapVal pwrapVal (kwrapVal@AbsoluteSeek) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aAbsoluteSeek)
-  T.wrapVal pwrapVal (kwrapVal@RelativeSeek) p =
+  wrapVal pwrapVal (kwrapVal@RelativeSeek) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aRelativeSeek)
-  T.wrapVal pwrapVal (kwrapVal@SeekFromEnd) p =
+  wrapVal pwrapVal (kwrapVal@SeekFromEnd) p =
     T.R kwrapVal (T.mkValueUse p pwrapVal aSeekFromEnd)
   
 
@@ -618,7 +625,7 @@ instance Eq (SeekMode)
   where
   
   (!==) (%==) p =
-    T.ufun2 (+##=%$==) (%==) p (*==)
+    T.ufun2 (+##=%$=##=%%==) (%==) p (*==)
     where
     
     (*==) (T.R AbsoluteSeek _) (T.R AbsoluteSeek _) p =
@@ -635,7 +642,7 @@ instance Ord (SeekMode)
   where
   
   gcompare pcompare p =
-    T.ufun2 a11v36compare pcompare p hcompare
+    T.ufun2 a11v36v11v38compare pcompare p hcompare
     where
     
     hcompare fy1 fy2 p =
@@ -645,9 +652,9 @@ instance Ord (SeekMode)
       where
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a11v36localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a11v36v11v38localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a11v36localFromEnum
+      alocalFromEnum = a11v36v11v38localFromEnum
       
       hlocalFromEnum (T.R (AbsoluteSeek) _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -667,7 +674,7 @@ instance Ix (SeekMode)
   where
   
   grange prange p =
-    T.ufun1 a11v41range prange p hrange
+    T.ufun1 a11v41v11v42range prange p hrange
     where
     
     hrange (T.R (T.Tuple2 fy1 fy2) _) p =
@@ -684,39 +691,41 @@ instance Ix (SeekMode)
       hlocalToEnum :: (T.R Hat.Prelude.Int) -> T.RefExp -> T.R SeekMode
       
       glocalToEnum plocalToEnum p =
-        T.ufun1 a11v41localToEnum plocalToEnum p hlocalToEnum
+        T.ufun1 a11v41v11v42localToEnum plocalToEnum p hlocalToEnum
       
-      alocalToEnum = a11v41localToEnum
+      alocalToEnum = a11v41v11v42localToEnum
       
-      hlocalToEnum fv11v41n p =
+      hlocalToEnum fv11v41v11v42n p =
         T.ucguard
-          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v41n
+          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v41v11v42n
             (T.uap1 T.mkNoSrcPos p
               (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-              (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1localToEnum fv11v41n p)
+              (T.conInteger T.mkNoSrcPos p 0))) (h p)
+          (y1localToEnum fv11v41v11v42n p)
         where
         
         h p = T.con0 T.mkNoSrcPos p AbsoluteSeek aAbsoluteSeek
-        h p = y1localToEnum fv11v41n p
+        h p = y1localToEnum fv11v41v11v42n p
         
-      hlocalToEnum fv11v41n p = y1localToEnum fv11v41n p
+      hlocalToEnum fv11v41v11v42n p = y1localToEnum fv11v41v11v42n p
       
-      y1localToEnum fv11v41n p =
+      y1localToEnum fv11v41v11v42n p =
         T.ucguard
-          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v41n
+          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v41v11v42n
             (T.uap1 T.mkNoSrcPos p
               (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-              (T.conInteger T.mkNoSrcPos p 1))) (h p) (y2localToEnum fv11v41n p)
+              (T.conInteger T.mkNoSrcPos p 1))) (h p)
+          (y2localToEnum fv11v41v11v42n p)
         where
         
         h p = T.con0 T.mkNoSrcPos p RelativeSeek aRelativeSeek
-        h p = y2localToEnum fv11v41n p
+        h p = y2localToEnum fv11v41v11v42n p
         
-      y1localToEnum fv11v41n p = y2localToEnum fv11v41n p
+      y1localToEnum fv11v41v11v42n p = y2localToEnum fv11v41v11v42n p
       
-      y2localToEnum fv11v41n p =
+      y2localToEnum fv11v41v11v42n p =
         T.ucguard
-          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v41n
+          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v41v11v42n
             (T.uap1 T.mkNoSrcPos p
               (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
               (T.conInteger T.mkNoSrcPos p 2))) (h p) (T.fatal p)
@@ -733,9 +742,9 @@ instance Ix (SeekMode)
       hlocalFromEnum :: (T.R SeekMode) -> T.RefExp -> T.R Hat.Prelude.Int
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a11v41localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a11v41v11v42localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a11v41localFromEnum
+      alocalFromEnum = a11v41v11v42localFromEnum
       
       hlocalFromEnum (T.R AbsoluteSeek _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -752,7 +761,7 @@ instance Ix (SeekMode)
     
   
   gindex pindex p =
-    T.ufun2 a11v41index pindex p hindex
+    T.ufun2 a11v41v11v42index pindex p hindex
     where
     
     hindex (T.R (T.Tuple2 fy1 fy2) _) fy3 p =
@@ -767,9 +776,9 @@ instance Ix (SeekMode)
       hlocalFromEnum :: (T.R SeekMode) -> T.RefExp -> T.R Hat.Prelude.Int
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a11v41localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a11v41v11v42localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a11v41localFromEnum
+      alocalFromEnum = a11v41v11v42localFromEnum
       
       hlocalFromEnum (T.R AbsoluteSeek _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -786,7 +795,7 @@ instance Ix (SeekMode)
     
   
   ginRange pinRange p =
-    T.ufun2 a11v41inRange pinRange p hinRange
+    T.ufun2 a11v41v11v42inRange pinRange p hinRange
     where
     
     hinRange (T.R (T.Tuple2 fy1 fy2) _) fy3 p =
@@ -803,9 +812,9 @@ instance Ix (SeekMode)
       hlocalFromEnum :: (T.R SeekMode) -> T.RefExp -> T.R Hat.Prelude.Int
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a11v41localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a11v41v11v42localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a11v41localFromEnum
+      alocalFromEnum = a11v41v11v42localFromEnum
       
       hlocalFromEnum (T.R AbsoluteSeek _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -828,13 +837,13 @@ instance Bounded (SeekMode)
   gminBound pminBound p = T.uconstUse pminBound p sminBound
   
   sminBound =
-    T.uconstDef T.mkRoot a11v45minBound
+    T.uconstDef T.mkRoot a11v45v11v51minBound
       (\ p -> T.con0 T.mkNoSrcPos p AbsoluteSeek aAbsoluteSeek)
   
   gmaxBound pmaxBound p = T.uconstUse pmaxBound p smaxBound
   
   smaxBound =
-    T.uconstDef T.mkRoot a11v45maxBound
+    T.uconstDef T.mkRoot a11v45v11v51maxBound
       (\ p -> T.con0 T.mkNoSrcPos p SeekFromEnd aSeekFromEnd)
   
 
@@ -842,7 +851,7 @@ instance Enum (SeekMode)
   where
   
   gfromEnum pfromEnum p =
-    T.ufun1 a11v54fromEnum pfromEnum p hfromEnum
+    T.ufun1 a11v54v11v57fromEnum pfromEnum p hfromEnum
     where
     
     hfromEnum (T.R AbsoluteSeek _) p =
@@ -858,44 +867,44 @@ instance Enum (SeekMode)
     
   
   gtoEnum ptoEnum p =
-    T.ufun1 a11v54toEnum ptoEnum p htoEnum
+    T.ufun1 a11v54v11v57toEnum ptoEnum p htoEnum
     where
     
-    htoEnum fv11v54n p =
+    htoEnum fv11v54v11v57n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v54n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v54v11v57n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1toEnum fv11v54n p)
+            (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1toEnum fv11v54v11v57n p)
       where
       
       h p = T.con0 T.mkNoSrcPos p AbsoluteSeek aAbsoluteSeek
-      h p = y1toEnum fv11v54n p
+      h p = y1toEnum fv11v54v11v57n p
       
-    htoEnum fv11v54n p = y1toEnum fv11v54n p
+    htoEnum fv11v54v11v57n p = y1toEnum fv11v54v11v57n p
     
-    y1toEnum fv11v54n p =
+    y1toEnum fv11v54v11v57n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v54n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v54v11v57n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 1))) (h p) (y2toEnum fv11v54n p)
+            (T.conInteger T.mkNoSrcPos p 1))) (h p) (y2toEnum fv11v54v11v57n p)
       where
       
       h p = T.con0 T.mkNoSrcPos p RelativeSeek aRelativeSeek
-      h p = y2toEnum fv11v54n p
+      h p = y2toEnum fv11v54v11v57n p
       
-    y1toEnum fv11v54n p = y2toEnum fv11v54n p
+    y1toEnum fv11v54v11v57n p = y2toEnum fv11v54v11v57n p
     
-    y2toEnum fv11v54n p =
+    y2toEnum fv11v54v11v57n p =
       T.ucguard
-        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v54n
+        (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv11v54v11v57n
           (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-            (T.conInteger T.mkNoSrcPos p 2))) (h p) (y3toEnum fv11v54n p)
+            (T.conInteger T.mkNoSrcPos p 2))) (h p) (y3toEnum fv11v54v11v57n p)
       where
       
       h p = T.con0 T.mkNoSrcPos p SeekFromEnd aSeekFromEnd
-      h p = y3toEnum fv11v54n p
+      h p = y3toEnum fv11v54v11v57n p
       
-    y2toEnum fv11v54n p = y3toEnum fv11v54n p
+    y2toEnum fv11v54v11v57n p = y3toEnum fv11v54v11v57n p
     
     y3toEnum _ p =
       T.uwrapForward p
@@ -904,7 +913,7 @@ instance Enum (SeekMode)
     
   
   genumFrom penumFrom p =
-    T.ufun1 a11v54enumFrom penumFrom p henumFrom
+    T.ufun1 a11v54v11v57enumFrom penumFrom p henumFrom
     where
     
     henumFrom fy1 p =
@@ -913,7 +922,7 @@ instance Enum (SeekMode)
     
   
   genumFromThen penumFromThen p =
-    T.ufun2 a11v54enumFromThen penumFromThen p henumFromThen
+    T.ufun2 a11v54v11v57enumFromThen penumFromThen p henumFromThen
     where
     
     henumFromThen fy1 fy2 p =
@@ -931,7 +940,7 @@ instance Read (SeekMode)
   where
   
   greadsPrec preadsPrec p =
-    T.ufun1 a11v60readsPrec preadsPrec p hreadsPrec
+    T.ufun1 a11v60v11v63readsPrec preadsPrec p hreadsPrec
     where
     
     hreadsPrec fy1 p =
@@ -970,7 +979,7 @@ instance Show (SeekMode)
   where
   
   gshowsPrec pshowsPrec p =
-    T.ufun2 a11v66showsPrec pshowsPrec p hshowsPrec
+    T.ufun2 a11v66v11v69showsPrec pshowsPrec p hshowsPrec
     where
     
     hshowsPrec fy1 (T.R (AbsoluteSeek) _) p =
@@ -988,115 +997,132 @@ instance Show (SeekMode)
 
 tIOBuiltinTypes = T.mkModule "IOBuiltinTypes" "IOBuiltinTypes.hs" Prelude.False
 
-aReadMode = T.mkConstructor tIOBuiltinTypes 50021 3 0 "ReadMode"
+aReadMode = T.mkConstructor tIOBuiltinTypes 50021 50028 3 0 "ReadMode"
 
-aWriteMode = T.mkConstructor tIOBuiltinTypes 50032 3 0 "WriteMode"
+aWriteMode = T.mkConstructor tIOBuiltinTypes 50032 50040 3 0 "WriteMode"
 
-aAppendMode = T.mkConstructor tIOBuiltinTypes 50044 3 0 "AppendMode"
+aAppendMode = T.mkConstructor tIOBuiltinTypes 50044 50053 3 0 "AppendMode"
 
-aReadWriteMode = T.mkConstructor tIOBuiltinTypes 50057 3 0 "ReadWriteMode"
+aReadWriteMode = T.mkConstructor tIOBuiltinTypes 50057 50069 3 0 "ReadWriteMode"
 
-aNoBuffering = T.mkConstructor tIOBuiltinTypes 70021 3 0 "NoBuffering"
+aNoBuffering = T.mkConstructor tIOBuiltinTypes 70021 70031 3 0 "NoBuffering"
 
-aLineBuffering = T.mkConstructor tIOBuiltinTypes 70035 3 0 "LineBuffering"
+aLineBuffering = T.mkConstructor tIOBuiltinTypes 70035 70047 3 0 "LineBuffering"
 
-aBlockBuffering = T.mkConstructor tIOBuiltinTypes 80022 3 1 "BlockBuffering"
+aBlockBuffering =
+  T.mkConstructor tIOBuiltinTypes 80022 80035 3 1 "BlockBuffering"
 
-aAbsoluteSeek = T.mkConstructor tIOBuiltinTypes 100021 3 0 "AbsoluteSeek"
+aAbsoluteSeek = T.mkConstructor tIOBuiltinTypes 100021 100032 3 0 "AbsoluteSeek"
 
-aRelativeSeek = T.mkConstructor tIOBuiltinTypes 100036 3 0 "RelativeSeek"
+aRelativeSeek = T.mkConstructor tIOBuiltinTypes 100036 100047 3 0 "RelativeSeek"
 
-aSeekFromEnd = T.mkConstructor tIOBuiltinTypes 100051 3 0 "SeekFromEnd"
+aSeekFromEnd = T.mkConstructor tIOBuiltinTypes 100051 100061 3 0 "SeekFromEnd"
 
-(++=%$==) = T.mkVariable tIOBuiltinTypes 60032 3 2 "==" Prelude.False
+(++=%$=+=%%==) = T.mkVariable tIOBuiltinTypes 60032 60033 3 2 "==" Prelude.False
 
-a6v36compare = T.mkVariable tIOBuiltinTypes 60036 3 2 "compare" Prelude.False
+a6v36v6v38compare =
+  T.mkVariable tIOBuiltinTypes 60036 60038 3 2 "compare" Prelude.False
 
-a6v41range = T.mkVariable tIOBuiltinTypes 60041 3 1 "range" Prelude.False
+a6v41v6v42range =
+  T.mkVariable tIOBuiltinTypes 60041 60042 3 1 "range" Prelude.False
 
-a6v41index = T.mkVariable tIOBuiltinTypes 60041 3 2 "index" Prelude.False
+a6v41v6v42index =
+  T.mkVariable tIOBuiltinTypes 60041 60042 3 2 "index" Prelude.False
 
-a6v41inRange = T.mkVariable tIOBuiltinTypes 60041 3 2 "inRange" Prelude.False
+a6v41v6v42inRange =
+  T.mkVariable tIOBuiltinTypes 60041 60042 3 2 "inRange" Prelude.False
 
-a6v45minBound = T.mkVariable tIOBuiltinTypes 60045 3 0 "minBound" Prelude.False
+a6v45v6v51minBound =
+  T.mkVariable tIOBuiltinTypes 60045 60051 3 0 "minBound" Prelude.False
 
-a6v45maxBound = T.mkVariable tIOBuiltinTypes 60045 3 0 "maxBound" Prelude.False
+a6v45v6v51maxBound =
+  T.mkVariable tIOBuiltinTypes 60045 60051 3 0 "maxBound" Prelude.False
 
-a6v54fromEnum = T.mkVariable tIOBuiltinTypes 60054 3 1 "fromEnum" Prelude.False
+a6v54v6v57fromEnum =
+  T.mkVariable tIOBuiltinTypes 60054 60057 3 1 "fromEnum" Prelude.False
 
-a6v54toEnum = T.mkVariable tIOBuiltinTypes 60054 3 1 "toEnum" Prelude.False
+a6v54v6v57toEnum =
+  T.mkVariable tIOBuiltinTypes 60054 60057 3 1 "toEnum" Prelude.False
 
-a6v54enumFrom = T.mkVariable tIOBuiltinTypes 60054 3 1 "enumFrom" Prelude.False
+a6v54v6v57enumFrom =
+  T.mkVariable tIOBuiltinTypes 60054 60057 3 1 "enumFrom" Prelude.False
 
-a6v54enumFromThen =
-  T.mkVariable tIOBuiltinTypes 60054 3 2 "enumFromThen" Prelude.False
+a6v54v6v57enumFromThen =
+  T.mkVariable tIOBuiltinTypes 60054 60057 3 2 "enumFromThen" Prelude.False
 
-a6v60readsPrec =
-  T.mkVariable tIOBuiltinTypes 60060 3 1 "readsPrec" Prelude.False
+a6v60v6v63readsPrec =
+  T.mkVariable tIOBuiltinTypes 60060 60063 3 1 "readsPrec" Prelude.False
 
-a6v66showsPrec =
-  T.mkVariable tIOBuiltinTypes 60066 3 2 "showsPrec" Prelude.False
+a6v66v6v69showsPrec =
+  T.mkVariable tIOBuiltinTypes 60066 60069 3 2 "showsPrec" Prelude.False
 
-(+>=%$==) = T.mkVariable tIOBuiltinTypes 90032 3 2 "==" Prelude.False
+(+>=%$=>=%%==) = T.mkVariable tIOBuiltinTypes 90032 90033 3 2 "==" Prelude.False
 
-a9v36compare = T.mkVariable tIOBuiltinTypes 90036 3 2 "compare" Prelude.False
+a9v36v9v38compare =
+  T.mkVariable tIOBuiltinTypes 90036 90038 3 2 "compare" Prelude.False
 
-a9v41readsPrec =
-  T.mkVariable tIOBuiltinTypes 90041 3 1 "readsPrec" Prelude.False
+a9v41v9v44readsPrec =
+  T.mkVariable tIOBuiltinTypes 90041 90044 3 1 "readsPrec" Prelude.False
 
-a9v47showsPrec =
-  T.mkVariable tIOBuiltinTypes 90047 3 2 "showsPrec" Prelude.False
+a9v47v9v50showsPrec =
+  T.mkVariable tIOBuiltinTypes 90047 90050 3 2 "showsPrec" Prelude.False
 
-(+##=%$==) = T.mkVariable tIOBuiltinTypes 110032 3 2 "==" Prelude.False
+(+##=%$=##=%%==) =
+  T.mkVariable tIOBuiltinTypes 110032 110033 3 2 "==" Prelude.False
 
-a11v36compare = T.mkVariable tIOBuiltinTypes 110036 3 2 "compare" Prelude.False
+a11v36v11v38compare =
+  T.mkVariable tIOBuiltinTypes 110036 110038 3 2 "compare" Prelude.False
 
-a11v41range = T.mkVariable tIOBuiltinTypes 110041 3 1 "range" Prelude.False
+a11v41v11v42range =
+  T.mkVariable tIOBuiltinTypes 110041 110042 3 1 "range" Prelude.False
 
-a11v41index = T.mkVariable tIOBuiltinTypes 110041 3 2 "index" Prelude.False
+a11v41v11v42index =
+  T.mkVariable tIOBuiltinTypes 110041 110042 3 2 "index" Prelude.False
 
-a11v41inRange = T.mkVariable tIOBuiltinTypes 110041 3 2 "inRange" Prelude.False
+a11v41v11v42inRange =
+  T.mkVariable tIOBuiltinTypes 110041 110042 3 2 "inRange" Prelude.False
 
-a11v45minBound =
-  T.mkVariable tIOBuiltinTypes 110045 3 0 "minBound" Prelude.False
+a11v45v11v51minBound =
+  T.mkVariable tIOBuiltinTypes 110045 110051 3 0 "minBound" Prelude.False
 
-a11v45maxBound =
-  T.mkVariable tIOBuiltinTypes 110045 3 0 "maxBound" Prelude.False
+a11v45v11v51maxBound =
+  T.mkVariable tIOBuiltinTypes 110045 110051 3 0 "maxBound" Prelude.False
 
-a11v54fromEnum =
-  T.mkVariable tIOBuiltinTypes 110054 3 1 "fromEnum" Prelude.False
+a11v54v11v57fromEnum =
+  T.mkVariable tIOBuiltinTypes 110054 110057 3 1 "fromEnum" Prelude.False
 
-a11v54toEnum = T.mkVariable tIOBuiltinTypes 110054 3 1 "toEnum" Prelude.False
+a11v54v11v57toEnum =
+  T.mkVariable tIOBuiltinTypes 110054 110057 3 1 "toEnum" Prelude.False
 
-a11v54enumFrom =
-  T.mkVariable tIOBuiltinTypes 110054 3 1 "enumFrom" Prelude.False
+a11v54v11v57enumFrom =
+  T.mkVariable tIOBuiltinTypes 110054 110057 3 1 "enumFrom" Prelude.False
 
-a11v54enumFromThen =
-  T.mkVariable tIOBuiltinTypes 110054 3 2 "enumFromThen" Prelude.False
+a11v54v11v57enumFromThen =
+  T.mkVariable tIOBuiltinTypes 110054 110057 3 2 "enumFromThen" Prelude.False
 
-a11v60readsPrec =
-  T.mkVariable tIOBuiltinTypes 110060 3 1 "readsPrec" Prelude.False
+a11v60v11v63readsPrec =
+  T.mkVariable tIOBuiltinTypes 110060 110063 3 1 "readsPrec" Prelude.False
 
-a11v66showsPrec =
-  T.mkVariable tIOBuiltinTypes 110066 3 2 "showsPrec" Prelude.False
+a11v66v11v69showsPrec =
+  T.mkVariable tIOBuiltinTypes 110066 110069 3 2 "showsPrec" Prelude.False
 
-a6v36localFromEnum =
-  T.mkVariable tIOBuiltinTypes 60036 3 1 "localFromEnum" Prelude.True
+a6v36v6v38localFromEnum =
+  T.mkVariable tIOBuiltinTypes 60036 60038 3 1 "localFromEnum" Prelude.True
 
-a6v41localToEnum =
-  T.mkVariable tIOBuiltinTypes 60041 3 1 "localToEnum" Prelude.True
+a6v41v6v42localToEnum =
+  T.mkVariable tIOBuiltinTypes 60041 60042 3 1 "localToEnum" Prelude.True
 
-a6v41localFromEnum =
-  T.mkVariable tIOBuiltinTypes 60041 3 1 "localFromEnum" Prelude.True
+a6v41v6v42localFromEnum =
+  T.mkVariable tIOBuiltinTypes 60041 60042 3 1 "localFromEnum" Prelude.True
 
-a9v36localFromEnum =
-  T.mkVariable tIOBuiltinTypes 90036 3 1 "localFromEnum" Prelude.True
+a9v36v9v38localFromEnum =
+  T.mkVariable tIOBuiltinTypes 90036 90038 3 1 "localFromEnum" Prelude.True
 
-a11v36localFromEnum =
-  T.mkVariable tIOBuiltinTypes 110036 3 1 "localFromEnum" Prelude.True
+a11v36v11v38localFromEnum =
+  T.mkVariable tIOBuiltinTypes 110036 110038 3 1 "localFromEnum" Prelude.True
 
-a11v41localToEnum =
-  T.mkVariable tIOBuiltinTypes 110041 3 1 "localToEnum" Prelude.True
+a11v41v11v42localToEnum =
+  T.mkVariable tIOBuiltinTypes 110041 110042 3 1 "localToEnum" Prelude.True
 
-a11v41localFromEnum =
-  T.mkVariable tIOBuiltinTypes 110041 3 1 "localFromEnum" Prelude.True
+a11v41v11v42localFromEnum =
+  T.mkVariable tIOBuiltinTypes 110041 110042 3 1 "localFromEnum" Prelude.True
