@@ -5,9 +5,11 @@ module Hat.Locale
     ,awDays,amonths,aamPm,adateTimeFmt,adateFmt,atimeFmt,atime12Fmt,aTimeLocale
     ,gdefaultTimeLocale) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 
 data TimeLocale =
@@ -20,7 +22,7 @@ data TimeLocale =
 instance T.WrapVal (TimeLocale)
   where
   
-  T.wrapVal pwrapVal
+  wrapVal pwrapVal
     (kwrapVal@(TimeLocale (T.R _ z1wrapVal) (T.R _ z2wrapVal) (T.R _ z3wrapVal)
         (T.R _ z4wrapVal) (T.R _ z5wrapVal) (T.R _ z6wrapVal)
         (T.R _ z7wrapVal))) p =
@@ -63,7 +65,7 @@ instance Eq (TimeLocale)
   where
   
   (!==) (%==) p =
-    T.ufun2 (+##=$#==) (%==) p (*==)
+    T.ufun2 (+##=$#=##=$$==) (%==) p (*==)
     where
     
     (*==) (T.R (TimeLocale fy1 fy2 fy3 fy4 fy5 fy6 fy7) _)
@@ -102,56 +104,60 @@ instance Ord (TimeLocale)
   where
   
   gcompare pcompare p =
-    T.ufun2 a11v25compare pcompare p hcompare
+    T.ufun2 a11v25v11v27compare pcompare p hcompare
     where
     
     hcompare (T.R (TimeLocale fy3 fy4 fy5 fy6 fy7 fy8 fy9) _)
       (T.R (TimeLocale fy10 fy11 fy12 fy13 fy14 fy15 fy16) _) p =
       T.uccase T.mkNoSrcPos p
         (let
-          v11v25v1 (T.R Hat.Prelude.EQ _) p =
+          v11v25v11v27v1 (T.R Hat.Prelude.EQ _) p =
             T.uccase T.mkNoSrcPos p
               (let
-                v11v25v1 (T.R Hat.Prelude.EQ _) p =
+                v11v25v11v27v1 (T.R Hat.Prelude.EQ _) p =
                   T.uccase T.mkNoSrcPos p
                     (let
-                      v11v25v1 (T.R Hat.Prelude.EQ _) p =
+                      v11v25v11v27v1 (T.R Hat.Prelude.EQ _) p =
                         T.uccase T.mkNoSrcPos p
                           (let
-                            v11v25v1 (T.R Hat.Prelude.EQ _) p =
+                            v11v25v11v27v1 (T.R Hat.Prelude.EQ _) p =
                               T.uccase T.mkNoSrcPos p
                                 (let
-                                  v11v25v1 (T.R Hat.Prelude.EQ _) p =
+                                  v11v25v11v27v1 (T.R Hat.Prelude.EQ _) p =
                                     T.uccase T.mkNoSrcPos p
                                       (let
-                                        v11v25v1 (T.R Hat.Prelude.EQ _) p =
+                                        v11v25v11v27v1 (T.R Hat.Prelude.EQ _)
+                                          p =
                                           T.uap2 T.mkNoSrcPos p
                                             (Hat.Prelude.gcompare T.mkNoSrcPos
                                               p) fy9 fy16
-                                        v11v25v1 fy1 p =
+                                        v11v25v11v27v1 fy1 p =
                                           T.projection T.mkNoSrcPos p fy1 in
-                                        (v11v25v1))
+                                        (v11v25v11v27v1))
                                       (T.uap2 T.mkNoSrcPos p
                                         (Hat.Prelude.gcompare T.mkNoSrcPos p)
                                         fy8 fy15)
-                                  v11v25v1 fy1 p =
+                                  v11v25v11v27v1 fy1 p =
                                     T.projection T.mkNoSrcPos p fy1 in
-                                  (v11v25v1))
+                                  (v11v25v11v27v1))
                                 (T.uap2 T.mkNoSrcPos p
                                   (Hat.Prelude.gcompare T.mkNoSrcPos p) fy7
                                   fy14)
-                            v11v25v1 fy1 p = T.projection T.mkNoSrcPos p fy1 in
-                            (v11v25v1))
+                            v11v25v11v27v1 fy1 p =
+                              T.projection T.mkNoSrcPos p fy1 in
+                            (v11v25v11v27v1))
                           (T.uap2 T.mkNoSrcPos p
                             (Hat.Prelude.gcompare T.mkNoSrcPos p) fy6 fy13)
-                      v11v25v1 fy1 p = T.projection T.mkNoSrcPos p fy1 in
-                      (v11v25v1))
+                      v11v25v11v27v1 fy1 p = T.projection T.mkNoSrcPos p fy1 in
+                      (v11v25v11v27v1))
                     (T.uap2 T.mkNoSrcPos p (Hat.Prelude.gcompare T.mkNoSrcPos p)
                       fy5 fy12)
-                v11v25v1 fy1 p = T.projection T.mkNoSrcPos p fy1 in (v11v25v1))
+                v11v25v11v27v1 fy1 p = T.projection T.mkNoSrcPos p fy1 in
+                (v11v25v11v27v1))
               (T.uap2 T.mkNoSrcPos p (Hat.Prelude.gcompare T.mkNoSrcPos p) fy4
                 fy11)
-          v11v25v1 fy1 p = T.projection T.mkNoSrcPos p fy1 in (v11v25v1))
+          v11v25v11v27v1 fy1 p = T.projection T.mkNoSrcPos p fy1 in
+          (v11v25v11v27v1))
         (T.uap2 T.mkNoSrcPos p (Hat.Prelude.gcompare T.mkNoSrcPos p) fy3 fy10)
     hcompare fy1 fy2 p =
       T.uap2 T.mkNoSrcPos p (Hat.Prelude.gcompare T.mkNoSrcPos p)
@@ -160,9 +166,9 @@ instance Ord (TimeLocale)
       where
       
       glocalFromEnum plocalFromEnum p =
-        T.ufun1 a11v25localFromEnum plocalFromEnum p hlocalFromEnum
+        T.ufun1 a11v25v11v27localFromEnum plocalFromEnum p hlocalFromEnum
       
-      alocalFromEnum = a11v25localFromEnum
+      alocalFromEnum = a11v25v11v27localFromEnum
       
       hlocalFromEnum (T.R (TimeLocale _ _ _ _ _ _ _) _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -176,7 +182,7 @@ instance Show (TimeLocale)
   where
   
   gshowsPrec pshowsPrec p =
-    T.ufun2 a11v30showsPrec pshowsPrec p hshowsPrec
+    T.ufun2 a11v30v11v33showsPrec pshowsPrec p hshowsPrec
     where
     
     hshowsPrec fy1 (T.R (TimeLocale fy2 fy3 fy4 fy5 fy6 fy7 fy8) _) p =
@@ -397,41 +403,37 @@ sdefaultTimeLocale =
 tLocale = T.mkModule "Locale" "Locale.hs" Prelude.False
 
 aTimeLocale =
-  T.mkConstructorWFields tLocale 50019 3 7 "TimeLocale"
+  T.mkConstructorWFields tLocale 50019 50028 3 7 "TimeLocale"
     (awDays
-      Prelude.:
+      :
       (amonths
-        Prelude.:
-        (aamPm
-          Prelude.:
-          (adateTimeFmt
-            Prelude.:
-            (adateFmt
-              Prelude.:
-              (atimeFmt Prelude.: (atime12Fmt Prelude.: Prelude.[])))))))
+        :
+        (aamPm : (adateTimeFmt : (adateFmt : (atimeFmt : (atime12Fmt : [])))))))
 
-awDays = T.mkVariable tLocale 60009 3 1 "wDays" Prelude.False
+awDays = T.mkVariable tLocale 60009 60013 3 1 "wDays" Prelude.False
 
-amonths = T.mkVariable tLocale 70009 3 1 "months" Prelude.False
+amonths = T.mkVariable tLocale 70009 70014 3 1 "months" Prelude.False
 
-aamPm = T.mkVariable tLocale 80009 3 1 "amPm" Prelude.False
+aamPm = T.mkVariable tLocale 80009 80012 3 1 "amPm" Prelude.False
 
-adateTimeFmt = T.mkVariable tLocale 90009 3 1 "dateTimeFmt" Prelude.False
+adateTimeFmt = T.mkVariable tLocale 90009 90019 3 1 "dateTimeFmt" Prelude.False
 
-adateFmt = T.mkVariable tLocale 90022 3 1 "dateFmt" Prelude.False
+adateFmt = T.mkVariable tLocale 90022 90028 3 1 "dateFmt" Prelude.False
 
-atimeFmt = T.mkVariable tLocale 100011 3 1 "timeFmt" Prelude.False
+atimeFmt = T.mkVariable tLocale 100011 100017 3 1 "timeFmt" Prelude.False
 
-atime12Fmt = T.mkVariable tLocale 100020 3 1 "time12Fmt" Prelude.False
+atime12Fmt = T.mkVariable tLocale 100020 100028 3 1 "time12Fmt" Prelude.False
 
-(+##=$#==) = T.mkVariable tLocale 110021 3 2 "==" Prelude.False
+(+##=$#=##=$$==) = T.mkVariable tLocale 110021 110022 3 2 "==" Prelude.False
 
-a11v25compare = T.mkVariable tLocale 110025 3 2 "compare" Prelude.False
+a11v25v11v27compare =
+  T.mkVariable tLocale 110025 110027 3 2 "compare" Prelude.False
 
-a11v30showsPrec = T.mkVariable tLocale 110030 3 2 "showsPrec" Prelude.False
+a11v30v11v33showsPrec =
+  T.mkVariable tLocale 110030 110033 3 2 "showsPrec" Prelude.False
 
 adefaultTimeLocale =
-  T.mkVariable tLocale 140001 3 0 "defaultTimeLocale" Prelude.False
+  T.mkVariable tLocale 140001 310017 3 0 "defaultTimeLocale" Prelude.False
 
-a11v25localFromEnum =
-  T.mkVariable tLocale 110025 3 1 "localFromEnum" Prelude.True
+a11v25v11v27localFromEnum =
+  T.mkVariable tLocale 110025 110027 3 1 "localFromEnum" Prelude.True
