@@ -5,9 +5,11 @@ module Hat.Random
     ,ggetStdRandom,agetStdRandom,hgetStdRandom,ggetStdGen,gsetStdGen,asetStdGen
     ,hsetStdGen,gnewStdGen) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 import Hat.PreludeBuiltinTypes 
 import Hat.RandomBuiltin 
@@ -24,7 +26,7 @@ class RandomGen g
   gsplit :: T.RefSrcPos -> T.RefExp -> T.R (T.Fun g (T.Tuple2 g g))
   
   ggenRange pgenRange p =
-    T.ufun1 a24v3genRange pgenRange p hgenRange
+    T.ufun1 a24v3v24v34genRange pgenRange p hgenRange
     where
     
     hgenRange fg p =
@@ -45,17 +47,19 @@ instance RandomGen (StdGen)
   ggenRange pgenRange p = T.uconstUse pgenRange p sgenRange
   
   sgenRange =
-    T.uconstDef T.mkRoot a31v3genRange
+    T.uconstDef T.mkRoot a31v3v31v31genRange
       (\ p -> gprimStdGenGenRange T.mkNoSrcPos p)
   
   gnext pnext p = T.uconstUse pnext p snext
   
-  snext = T.uconstDef T.mkRoot a32v3next (\ p -> gprimStdGenNext T.mkNoSrcPos p)
+  snext =
+    T.uconstDef T.mkRoot a32v3v32v23next (\ p -> gprimStdGenNext T.mkNoSrcPos p)
   
   gsplit psplit p = T.uconstUse psplit p ssplit
   
   ssplit =
-    T.uconstDef T.mkRoot a33v3split (\ p -> gprimStdGenSplit T.mkNoSrcPos p)
+    T.uconstDef T.mkRoot a33v3v33v25split
+      (\ p -> gprimStdGenSplit T.mkNoSrcPos p)
   
 
 instance Read (StdGen)
@@ -64,7 +68,7 @@ instance Read (StdGen)
   greadsPrec preadsPrec p = T.uconstUse preadsPrec p sreadsPrec
   
   sreadsPrec =
-    T.uconstDef T.mkRoot a36v3readsPrec
+    T.uconstDef T.mkRoot a36v3v36v33readsPrec
       (\ p -> gprimStdGenReadsPrec T.mkNoSrcPos p)
   
 
@@ -74,7 +78,7 @@ instance Show (StdGen)
   gshowsPrec pshowsPrec p = T.uconstUse pshowsPrec p sshowsPrec
   
   sshowsPrec =
-    T.uconstDef T.mkRoot a39v3showsPrec
+    T.uconstDef T.mkRoot a39v3v39v33showsPrec
       (\ p -> gprimStdGenShowsPrec T.mkNoSrcPos p)
   
 
@@ -164,7 +168,7 @@ class Random a
   grandomIO :: T.RefSrcPos -> T.RefExp -> T.R (IO a)
   
   grandoms prandoms p =
-    T.ufun1 a68v3randoms prandoms p hrandoms
+    T.ufun1 a68v3v70v21randoms prandoms p hrandoms
     where
     
     hrandoms fg p =
@@ -176,19 +180,23 @@ class Random a
       
       gg' px p = T.uconstUse px p sg'
       
-      j70v5x =
+      j70v5v70v10x =
         case T.uap1 T.mkNoSrcPos p (grandom T.mkNoSrcPos p) fg of
           T.R (T.Tuple2 fx fg') kx -> (kx,fx,fg')
           _ -> T.fatal p
       
-      sx = T.uconstDef p a70v6x (\ _ -> case j70v5x of (kx,fx,fg') -> fx)
+      sx =
+        T.uconstDef p a70v6v70v6x
+          (\ _ -> case j70v5v70v10x of (kx,fx,fg') -> fx)
       
-      sg' = T.uconstDef p a70v8g' (\ _ -> case j70v5x of (kx,fx,fg') -> fg')
+      sg' =
+        T.uconstDef p a70v8v70v9g'
+          (\ _ -> case j70v5v70v10x of (kx,fx,fg') -> fg')
       
     
   
   grandomRs prandomRs p =
-    T.ufun2 a71v3randomRs prandomRs p hrandomRs
+    T.ufun2 a71v3v73v28randomRs prandomRs p hrandomRs
     where
     
     hrandomRs frange fg p =
@@ -201,25 +209,29 @@ class Random a
       
       gg' px p = T.uconstUse px p sg'
       
-      j73v5x =
+      j73v5v73v10x =
         case T.uap2 T.mkNoSrcPos p (grandomR T.mkNoSrcPos p) frange fg of
           T.R (T.Tuple2 fx fg') kx -> (kx,fx,fg')
           _ -> T.fatal p
       
-      sx = T.uconstDef p a73v6x (\ _ -> case j73v5x of (kx,fx,fg') -> fx)
+      sx =
+        T.uconstDef p a73v6v73v6x
+          (\ _ -> case j73v5v73v10x of (kx,fx,fg') -> fx)
       
-      sg' = T.uconstDef p a73v8g' (\ _ -> case j73v5x of (kx,fx,fg') -> fg')
+      sg' =
+        T.uconstDef p a73v8v73v9g'
+          (\ _ -> case j73v5v73v10x of (kx,fx,fg') -> fg')
       
     
   
   grandomIO prandomIO p = T.uconstUse prandomIO p srandomIO
   
   srandomIO =
-    T.uconstDef T.mkRoot a74v3randomIO
+    T.uconstDef T.mkRoot a74v3v74v39randomIO
       (\ p -> T.uwrapForward p (hgetStdRandom (grandom T.mkNoSrcPos p) p))
   
   grandomRIO prandomRIO p =
-    T.ufun1 a75v3randomRIO prandomRIO p hrandomRIO
+    T.ufun1 a75v3v75v47randomRIO prandomRIO p hrandomRIO
     where
     
     hrandomRIO frange p =
@@ -245,7 +257,7 @@ instance Random (Int)
   where
   
   grandomR prandomR p =
-    T.ufun2 a78v3randomR prandomR p hrandomR
+    T.ufun2 a78v3v78v66randomR prandomR p hrandomR
     where
     
     hrandomR (T.R (T.Tuple2 fa fb) _) fg p =
@@ -258,7 +270,7 @@ instance Random (Int)
     
   
   grandom prandom p =
-    T.ufun1 a79v3random prandom p hrandom
+    T.ufun1 a79v3v79v49random prandom p hrandom
     where
     
     hrandom fg p =
@@ -272,14 +284,14 @@ instance Random (Integer)
   where
   
   grandomR prandomR p =
-    T.ufun2 a82v3randomR prandomR p hrandomR
+    T.ufun2 a82v3v82v43randomR prandomR p hrandomR
     where
     
     hrandomR fival fg p = T.uwrapForward p (hrandomIvalInteger fival fg p)
     
   
   grandom prandom p =
-    T.ufun1 a83v3random prandom p hrandom
+    T.ufun1 a83v3v83v83random prandom p hrandom
     where
     
     hrandom fg p =
@@ -296,7 +308,7 @@ instance Random (Float)
   where
   
   grandom prandom p =
-    T.ufun1 a86v3random prandom p hrandom
+    T.ufun1 a86v3v86v63random prandom p hrandom
     where
     
     hrandom fg p =
@@ -314,7 +326,7 @@ instance Random (Float)
     
   
   grandomR prandomR p =
-    T.ufun2 a87v3randomR prandomR p hrandomR
+    T.ufun2 a87v3v87v78randomR prandomR p hrandomR
     where
     
     hrandomR (T.R (T.Tuple2 fa fb) _) fg p =
@@ -332,7 +344,7 @@ instance Random (Double)
   where
   
   grandomR prandomR p =
-    T.ufun2 a90v3randomR prandomR p hrandomR
+    T.ufun2 a90v3v90v45randomR prandomR p hrandomR
     where
     
     hrandomR fival fg p =
@@ -340,7 +352,7 @@ instance Random (Double)
     
   
   grandom prandom p =
-    T.ufun1 a91v3random prandom p hrandom
+    T.ufun1 a91v3v91v42random prandom p hrandom
     where
     
     hrandom fg p =
@@ -358,16 +370,16 @@ instance Random (Bool)
   where
   
   grandomR prandomR p =
-    T.ufun2 a94v3randomR prandomR p hrandomR
+    T.ufun2 a94v3v105v22randomR prandomR p hrandomR
     where
     
     hrandomR (T.R (T.Tuple2 fa fb) _) fg p =
       T.uccase T.mkNoSrcPos p
         (let
-          v95v5v1 (T.R (T.Tuple2 fx fg) _) p =
+          v95v5v97v33v1 (T.R (T.Tuple2 fx fg) _) p =
             T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
               (T.uwrapForward p (hint2Bool fx p)) fg
-          v95v5v1 _ p = T.fatal p in (v95v5v1))
+          v95v5v97v33v1 _ p = T.fatal p in (v95v5v97v33v1))
         (T.uwrapForward p
           (hrandomIvalInteger
             (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
@@ -380,9 +392,9 @@ instance Random (Bool)
       hbool2Integer :: (T.R Bool) -> T.RefExp -> T.R Integer
       
       gbool2Integer pbool2Integer p =
-        T.ufun1 a100v5bool2Integer pbool2Integer p hbool2Integer
+        T.ufun1 a100v5v101v26bool2Integer pbool2Integer p hbool2Integer
       
-      abool2Integer = a100v5bool2Integer
+      abool2Integer = a100v5v101v26bool2Integer
       
       hbool2Integer (T.R False _) p =
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
@@ -396,22 +408,25 @@ instance Random (Bool)
       
       hint2Bool :: (T.R Int) -> T.RefExp -> T.R Bool
       
-      gint2Bool pint2Bool p = T.ufun1 a104v5int2Bool pint2Bool p hint2Bool
+      gint2Bool pint2Bool p =
+        T.ufun1 a104v5v105v22int2Bool pint2Bool p hint2Bool
       
-      aint2Bool = a104v5int2Bool
+      aint2Bool = a104v5v105v22int2Bool
       
-      hint2Bool fv104v14n p =
+      hint2Bool fv104v14v104v14n p =
         T.ucguard
-          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv104v14n
+          (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p)
+            fv104v14v104v14n
             (T.uap1 T.mkNoSrcPos p
               (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
-              (T.conInteger T.mkNoSrcPos p 0))) (h p) (y1int2Bool fv104v14n p)
+              (T.conInteger T.mkNoSrcPos p 0))) (h p)
+          (y1int2Bool fv104v14v104v14n p)
         where
         
         h p = T.con0 T.mkNoSrcPos p False aFalse
-        h p = y1int2Bool fv104v14n p
+        h p = y1int2Bool fv104v14v104v14n p
         
-      hint2Bool fv104v14n p = y1int2Bool fv104v14n p
+      hint2Bool fv104v14v104v14n p = y1int2Bool fv104v14v104v14n p
       
       y1int2Bool _ p = T.con0 T.mkNoSrcPos p True aTrue
       
@@ -419,7 +434,7 @@ instance Random (Bool)
     
   
   grandom prandom p =
-    T.ufun1 a107v3random prandom p hrandom
+    T.ufun1 a107v3v107v42random prandom p hrandom
     where
     
     hrandom fg p =
@@ -433,16 +448,16 @@ instance Random (Char)
   where
   
   grandomR prandomR p =
-    T.ufun2 a110v3randomR prandomR p hrandomR
+    T.ufun2 a110v3v112v27randomR prandomR p hrandomR
     where
     
     hrandomR (T.R (T.Tuple2 fa fb) _) fg p =
       T.uccase T.mkNoSrcPos p
         (let
-          v111v7v1 (T.R (T.Tuple2 fx fg) _) p =
+          v111v7v112v27v1 (T.R (T.Tuple2 fx fg) _) p =
             T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
               (T.uap1 T.mkNoSrcPos p (gchr T.mkNoSrcPos p) fx) fg
-          v111v7v1 _ p = T.fatal p in (v111v7v1))
+          v111v7v112v27v1 _ p = T.fatal p in (v111v7v112v27v1))
         (T.uwrapForward p
           (hrandomIvalInteger
             (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
@@ -454,7 +469,7 @@ instance Random (Char)
     
   
   grandom prandom p =
-    T.ufun1 a113v3random prandom p hrandom
+    T.ufun1 a113v3v113v42random prandom p hrandom
     where
     
     hrandom fg p =
@@ -479,18 +494,20 @@ grandomIvalInteger prandomIvalInteger p =
 hrandomIvalInteger (T.R (T.Tuple2 fl fh) _) frng p =
   T.ucguard (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !> p) fl fh)
     (T.uwrapForward p
-      (hrandomIvalInteger (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2 fh fl) frng
-        p))
+      (hrandomIvalInteger
+        (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
+          (T.projection T.mkNoSrcPos p fh) (T.projection T.mkNoSrcPos p fl))
+        frng p))
     (T.ucguard (gotherwise T.mkNoSrcPos p)
       (T.uccase T.mkNoSrcPos p
         (let
-          v120v16v1 (T.R (T.Tuple2 fv frng') _) p =
+          v120v16v120v83v1 (T.R (T.Tuple2 fv frng') _) p =
             T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
               (T.uap1 T.mkNoSrcPos p (gfromInteger T.mkNoSrcPos p)
                 (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p) fl
                   (T.uap2 T.mkNoSrcPos p (gmod T.mkNoSrcPos p) fv
                     (gk T.mkNoSrcPos p)))) frng'
-          v120v16v1 _ p = T.fatal p in (v120v16v1))
+          v120v16v120v83v1 _ p = T.fatal p in (v120v16v120v83v1))
         (T.uwrapForward p
           (hf (gn T.mkNoSrcPos p)
             (T.uap1 T.mkNoSrcPos p
@@ -501,7 +518,7 @@ hrandomIvalInteger (T.R (T.Tuple2 fl fh) _) frng p =
   gk pk p = T.uconstUse pk p sk
   
   sk =
-    T.uconstDef p a122v8k
+    T.uconstDef p a122v8v122v20k
       (\ p ->
         T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p)
           (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !- p) fh fl)
@@ -511,7 +528,7 @@ hrandomIvalInteger (T.R (T.Tuple2 fl fh) _) frng p =
   gb pb p = T.uconstUse pb p sb
   
   sb =
-    T.uconstDef p a123v8b
+    T.uconstDef p a123v8v123v21b
       (\ p ->
         T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
           (T.conInteger T.mkNoSrcPos p 2147483561))
@@ -519,38 +536,41 @@ hrandomIvalInteger (T.R (T.Tuple2 fl fh) _) frng p =
   gn pn p = T.uconstUse pn p sn
   
   sn =
-    T.uconstDef p a124v8n
+    T.uconstDef p a124v8v124v23n
       (\ p ->
         T.uwrapForward p (hiLogBase (gb T.mkNoSrcPos p) (gk T.mkNoSrcPos p) p))
   
-  gf pf p = T.ufun3 a126v8f pf p hf
+  gf pf p = T.ufun3 a126v8v131v42f pf p hf
   
-  af = a126v8f
+  af = a126v8v131v42f
   
-  hf fv126v10n facc fg p =
+  hf fv126v10v126v10n facc fg p =
     T.ucguard
-      (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv126v10n
+      (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos Hat.Prelude.!== p) fv126v10v126v10n
         (T.uap1 T.mkNoSrcPos p (Hat.PreludeBasic.gfromInteger T.mkNoSrcPos p)
           (T.conInteger T.mkNoSrcPos p 0))) (h facc fg p)
-      (y1f fv126v10n facc fg p)
+      (y1f fv126v10v126v10n facc fg p)
     where
     
     h facc fg p = T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2 facc fg
-    h _ _ p = y1f fv126v10n facc fg p
+    h _ _ p = y1f fv126v10v126v10n facc fg p
     
-  hf fv126v10n facc fg p = y1f fv126v10n facc fg p
+  hf fv126v10v126v10n facc fg p = y1f fv126v10v126v10n facc fg p
   
   y1f fn facc fg p =
     let
       gx px p = T.uconstUse px p sx
       gg' px p = T.uconstUse px p sg'
-      j129v12x =
+      j129v12v129v17x =
         case T.uap1 T.mkNoSrcPos p (gnext T.mkNoSrcPos p) fg of
           T.R (T.Tuple2 fx fg') kx -> (kx,fx,fg')
           _ -> T.fatal p
-      sx = T.uconstDef p a129v13x (\ _ -> case j129v12x of (kx,fx,fg') -> fx)
-      sg' = T.uconstDef p a129v15g' (\ _ -> case j129v12x of (kx,fx,fg') -> fg')
-      in
+      sx =
+        T.uconstDef p a129v13v129v13x
+          (\ _ -> case j129v12v129v17x of (kx,fx,fg') -> fx)
+      sg' =
+        T.uconstDef p a129v15v129v16g'
+          (\ _ -> case j129v12v129v17x of (kx,fx,fg') -> fg') in
       (T.uwrapForward p
         (hf
           (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !- p) fn
@@ -584,16 +604,18 @@ grandomIvalDouble prandomIvalDouble p =
 hrandomIvalDouble (T.R (T.Tuple2 fl fh) _) ffromDouble frng p =
   T.ucguard (T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !> p) fl fh)
     (T.uwrapForward p
-      (hrandomIvalDouble (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2 fh fl)
+      (hrandomIvalDouble
+        (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
+          (T.projection T.mkNoSrcPos p fh) (T.projection T.mkNoSrcPos p fl))
         ffromDouble frng p))
     (T.ucguard (gotherwise T.mkNoSrcPos p)
       (T.uccase T.mkNoSrcPos p
         (let
-          v137v8v1 (T.R (T.Tuple2 fx frng') _) p =
+          v137v8v145v28v1 (T.R (T.Tuple2 fx frng') _) p =
             let
               gscaled_x pscaled_x p = T.uconstUse pscaled_x p sscaled_x
               sscaled_x =
-                T.uconstDef p a140v14scaled_x
+                T.uconstDef p a140v14v143v36scaled_x
                   (\ p ->
                     T.uap2 T.mkNoSrcPos p (T.mkNoSrcPos !+ p)
                       (T.uap1 T.mkNoSrcPos p ffromDouble
@@ -612,7 +634,7 @@ hrandomIvalDouble (T.R (T.Tuple2 fl fh) _) ffromDouble frng p =
                           (fx :: T.R Int)))) in
               (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
                 (gscaled_x T.mkNoSrcPos p) frng')
-          v137v8v1 _ p = T.fatal p in (v137v8v1))
+          v137v8v145v28v1 _ p = T.fatal p in (v137v8v145v28v1))
         (T.uwrapForward p
           (hrandomIvalInteger
             (T.con2 T.mkNoSrcPos p T.Tuple2 T.aTuple2
@@ -705,109 +727,133 @@ hgetStdRandom z1getStdRandom kgetStdRandom =
 
 tRandom = T.mkModule "Random" "Random.hs" Prelude.False
 
-a24v3genRange = T.mkVariable tRandom 240003 3 1 "genRange" Prelude.False
+a24v3v24v34genRange =
+  T.mkVariable tRandom 240003 240034 3 1 "genRange" Prelude.False
 
-a31v3genRange = T.mkVariable tRandom 310003 3 0 "genRange" Prelude.False
+a31v3v31v31genRange =
+  T.mkVariable tRandom 310003 310031 3 0 "genRange" Prelude.False
 
-a32v3next = T.mkVariable tRandom 320003 3 0 "next" Prelude.False
+a32v3v32v23next = T.mkVariable tRandom 320003 320023 3 0 "next" Prelude.False
 
-a33v3split = T.mkVariable tRandom 330003 3 0 "split" Prelude.False
+a33v3v33v25split = T.mkVariable tRandom 330003 330025 3 0 "split" Prelude.False
 
-a36v3readsPrec = T.mkVariable tRandom 360003 3 0 "readsPrec" Prelude.False
+a36v3v36v33readsPrec =
+  T.mkVariable tRandom 360003 360033 3 0 "readsPrec" Prelude.False
 
-a39v3showsPrec = T.mkVariable tRandom 390003 3 0 "showsPrec" Prelude.False
+a39v3v39v33showsPrec =
+  T.mkVariable tRandom 390003 390033 3 0 "showsPrec" Prelude.False
 
 aprimStdGenGenRange =
-  T.mkVariable tRandom 420002 3 1 "primStdGenGenRange" Prelude.False
+  T.mkVariable tRandom 410001 420041 3 1 "primStdGenGenRange" Prelude.False
 
-aprimStdGenNext = T.mkVariable tRandom 440002 3 1 "primStdGenNext" Prelude.False
+aprimStdGenNext =
+  T.mkVariable tRandom 430001 440040 3 1 "primStdGenNext" Prelude.False
 
 aprimStdGenSplit =
-  T.mkVariable tRandom 460002 3 1 "primStdGenSplit" Prelude.False
+  T.mkVariable tRandom 450001 460044 3 1 "primStdGenSplit" Prelude.False
 
 aprimStdGenReadsPrec =
-  T.mkVariable tRandom 480002 3 2 "primStdGenReadsPrec" Prelude.False
+  T.mkVariable tRandom 470001 480042 3 2 "primStdGenReadsPrec" Prelude.False
 
 aprimStdGenShowsPrec =
-  T.mkVariable tRandom 500002 3 3 "primStdGenShowsPrec" Prelude.False
+  T.mkVariable tRandom 490001 500057 3 3 "primStdGenShowsPrec" Prelude.False
 
-amkStdGen = T.mkVariable tRandom 540002 3 1 "mkStdGen" Prelude.False
+amkStdGen = T.mkVariable tRandom 530001 540026 3 1 "mkStdGen" Prelude.False
 
-a68v3randoms = T.mkVariable tRandom 680003 3 1 "randoms" Prelude.False
+a68v3v70v21randoms =
+  T.mkVariable tRandom 680003 700021 3 1 "randoms" Prelude.False
 
-a71v3randomRs = T.mkVariable tRandom 710003 3 2 "randomRs" Prelude.False
+a71v3v73v28randomRs =
+  T.mkVariable tRandom 710003 730028 3 2 "randomRs" Prelude.False
 
-a74v3randomIO = T.mkVariable tRandom 740003 3 0 "randomIO" Prelude.False
+a74v3v74v39randomIO =
+  T.mkVariable tRandom 740003 740039 3 0 "randomIO" Prelude.False
 
-a75v3randomRIO = T.mkVariable tRandom 750003 3 1 "randomRIO" Prelude.False
+a75v3v75v47randomRIO =
+  T.mkVariable tRandom 750003 750047 3 1 "randomRIO" Prelude.False
 
-a78v3randomR = T.mkVariable tRandom 780003 3 2 "randomR" Prelude.False
+a78v3v78v66randomR =
+  T.mkVariable tRandom 780003 780066 3 2 "randomR" Prelude.False
 
-a79v3random = T.mkVariable tRandom 790003 3 1 "random" Prelude.False
+a79v3v79v49random =
+  T.mkVariable tRandom 790003 790049 3 1 "random" Prelude.False
 
-a82v3randomR = T.mkVariable tRandom 820003 3 2 "randomR" Prelude.False
+a82v3v82v43randomR =
+  T.mkVariable tRandom 820003 820043 3 2 "randomR" Prelude.False
 
-a83v3random = T.mkVariable tRandom 830003 3 1 "random" Prelude.False
+a83v3v83v83random =
+  T.mkVariable tRandom 830003 830083 3 1 "random" Prelude.False
 
-a86v3random = T.mkVariable tRandom 860003 3 1 "random" Prelude.False
+a86v3v86v63random =
+  T.mkVariable tRandom 860003 860063 3 1 "random" Prelude.False
 
-a87v3randomR = T.mkVariable tRandom 870003 3 2 "randomR" Prelude.False
+a87v3v87v78randomR =
+  T.mkVariable tRandom 870003 870078 3 2 "randomR" Prelude.False
 
-a90v3randomR = T.mkVariable tRandom 900003 3 2 "randomR" Prelude.False
+a90v3v90v45randomR =
+  T.mkVariable tRandom 900003 900045 3 2 "randomR" Prelude.False
 
-a91v3random = T.mkVariable tRandom 910003 3 1 "random" Prelude.False
+a91v3v91v42random =
+  T.mkVariable tRandom 910003 910042 3 1 "random" Prelude.False
 
-a94v3randomR = T.mkVariable tRandom 940003 3 2 "randomR" Prelude.False
+a94v3v105v22randomR =
+  T.mkVariable tRandom 940003 1050022 3 2 "randomR" Prelude.False
 
-a107v3random = T.mkVariable tRandom 1070003 3 1 "random" Prelude.False
+a107v3v107v42random =
+  T.mkVariable tRandom 1070003 1070042 3 1 "random" Prelude.False
 
-a110v3randomR = T.mkVariable tRandom 1100003 3 2 "randomR" Prelude.False
+a110v3v112v27randomR =
+  T.mkVariable tRandom 1100003 1120027 3 2 "randomR" Prelude.False
 
-a113v3random = T.mkVariable tRandom 1130003 3 1 "random" Prelude.False
+a113v3v113v42random =
+  T.mkVariable tRandom 1130003 1130042 3 1 "random" Prelude.False
 
 arandomIvalInteger =
-  T.mkVariable tRandom 1180001 3 2 "randomIvalInteger" Prelude.False
+  T.mkVariable tRandom 1180001 1310042 3 2 "randomIvalInteger" Prelude.False
 
 arandomIvalDouble =
-  T.mkVariable tRandom 1340001 3 3 "randomIvalDouble" Prelude.False
+  T.mkVariable tRandom 1340001 1450028 3 3 "randomIvalDouble" Prelude.False
 
-afromInt = T.mkVariable tRandom 1480001 3 0 "fromInt" Prelude.False
+afromInt = T.mkVariable tRandom 1480001 1480033 3 0 "fromInt" Prelude.False
 
-aintRange = T.mkVariable tRandom 1510001 3 0 "intRange" Prelude.False
+aintRange = T.mkVariable tRandom 1510001 1510064 3 0 "intRange" Prelude.False
 
-aiLogBase = T.mkVariable tRandom 1540001 3 2 "iLogBase" Prelude.False
+aiLogBase = T.mkVariable tRandom 1540001 1540061 3 2 "iLogBase" Prelude.False
 
-anewStdGen = T.mkVariable tRandom 1590002 3 0 "newStdGen" Prelude.False
+anewStdGen = T.mkVariable tRandom 1580001 1590019 3 0 "newStdGen" Prelude.False
 
-asetStdGen = T.mkVariable tRandom 1610002 3 1 "setStdGen" Prelude.False
+asetStdGen = T.mkVariable tRandom 1600001 1610029 3 1 "setStdGen" Prelude.False
 
-agetStdGen = T.mkVariable tRandom 1630002 3 0 "getStdGen" Prelude.False
+agetStdGen = T.mkVariable tRandom 1620001 1630019 3 0 "getStdGen" Prelude.False
 
-agetStdRandom = T.mkVariable tRandom 1650002 3 1 "getStdRandom" Prelude.False
+agetStdRandom =
+  T.mkVariable tRandom 1640001 1650046 3 1 "getStdRandom" Prelude.False
 
-a70v6x = T.mkVariable tRandom 700006 3 0 "x" Prelude.True
+a70v6v70v6x = T.mkVariable tRandom 700006 700006 3 0 "x" Prelude.True
 
-a70v8g' = T.mkVariable tRandom 700008 3 0 "g'" Prelude.True
+a70v8v70v9g' = T.mkVariable tRandom 700008 700009 3 0 "g'" Prelude.True
 
-a73v6x = T.mkVariable tRandom 730006 3 0 "x" Prelude.True
+a73v6v73v6x = T.mkVariable tRandom 730006 730006 3 0 "x" Prelude.True
 
-a73v8g' = T.mkVariable tRandom 730008 3 0 "g'" Prelude.True
+a73v8v73v9g' = T.mkVariable tRandom 730008 730009 3 0 "g'" Prelude.True
 
-a100v5bool2Integer =
-  T.mkVariable tRandom 1000005 3 1 "bool2Integer" Prelude.True
+a100v5v101v26bool2Integer =
+  T.mkVariable tRandom 1000005 1010026 3 1 "bool2Integer" Prelude.True
 
-a104v5int2Bool = T.mkVariable tRandom 1040005 3 1 "int2Bool" Prelude.True
+a104v5v105v22int2Bool =
+  T.mkVariable tRandom 1040005 1050022 3 1 "int2Bool" Prelude.True
 
-a122v8k = T.mkVariable tRandom 1220008 3 0 "k" Prelude.True
+a122v8v122v20k = T.mkVariable tRandom 1220008 1220020 3 0 "k" Prelude.True
 
-a123v8b = T.mkVariable tRandom 1230008 3 0 "b" Prelude.True
+a123v8v123v21b = T.mkVariable tRandom 1230008 1230021 3 0 "b" Prelude.True
 
-a124v8n = T.mkVariable tRandom 1240008 3 0 "n" Prelude.True
+a124v8v124v23n = T.mkVariable tRandom 1240008 1240023 3 0 "n" Prelude.True
 
-a126v8f = T.mkVariable tRandom 1260008 3 3 "f" Prelude.True
+a126v8v131v42f = T.mkVariable tRandom 1260008 1310042 3 3 "f" Prelude.True
 
-a129v13x = T.mkVariable tRandom 1290013 3 0 "x" Prelude.True
+a129v13v129v13x = T.mkVariable tRandom 1290013 1290013 3 0 "x" Prelude.True
 
-a129v15g' = T.mkVariable tRandom 1290015 3 0 "g'" Prelude.True
+a129v15v129v16g' = T.mkVariable tRandom 1290015 1290016 3 0 "g'" Prelude.True
 
-a140v14scaled_x = T.mkVariable tRandom 1400014 3 0 "scaled_x" Prelude.True
+a140v14v143v36scaled_x =
+  T.mkVariable tRandom 1400014 1430036 3 0 "scaled_x" Prelude.True
