@@ -3,9 +3,11 @@ module Hat.Ratio
     ,gdenominator,adenominator,hdenominator,gapproxRational,aapproxRational
     ,happroxRational) where
 
-import qualified Prelude 
-import qualified Hat.Hat as T 
 import qualified Hat.PreludeBasic 
+import qualified Prelude 
+import Hat.Hack 
+import qualified Hat.Hat as T 
+import Hat.Hat  (WrapVal(wrapVal))
 import Hat.Prelude 
 import Hat.PreludeBasic 
 
