@@ -32,6 +32,8 @@ static void updateFileOffset (FileOffset at, FileOffset entry);
 static int stringSize(char *s);
 static void writeString(char *s);
 static void dumpBuffers();
+static void initHiddenChildrenSet();
+static void dumpHiddenChildrenSet();
 
 /* global variables */
 
@@ -41,9 +43,9 @@ typedef char byte;
 static FILE *HatFile, *HatOutput, *HatBridge;
 static FileOffset HatCounter = HeaderSize;
 static FileOffset LastExp = 0;
-static bool traceOpen = False;
-static bool atTraceEnd = True;
-static bool controlC = False;
+static Bool traceOpen = False;
+static Bool atTraceEnd = True;
+static Bool controlC = False;
 static FileOffset hiddenMask;
 static FileOffset unevaluated;
 static FileOffset entered;
@@ -104,12 +106,13 @@ hat_Open(char *progname)
     strcat(filename,".hat");		/* the .hat file holds the archive */
     HatFile = fopen(filename,"w");	/* of redex trails */
     p = ftell(HatFile);                 /* should be 0 */
-    fprintf(HatFile,"Hat%s",VERSION);	/* initialise file */
+    fprintf(HatFile,"Hat%s",FILEVERSION);	/* initialise file */
     fputc(0,HatFile);
     fwrite(&p,sizeof(FileOffset),1,HatFile);
     fwrite(&p,sizeof(FileOffset),1,HatFile);
 
     initBuffers();
+    initHiddenChildrenSet();
 
     strcpy(filename,progname);		/* the .output file is a copy of */
     strcat(filename,".hat.output");	/* stdout */
@@ -135,6 +138,7 @@ void hat_Close(void)
 {
   if (!traceOpen) 
     return;
+  dumpHiddenChildrenSet();
   dumpBuffers();
   hat_dumpBuffer();
   fclose(HatFile);
@@ -216,7 +220,7 @@ void hat_OutputTrace(FileOffset trace, char *output)
   }
 }
 
-bool
+Bool
 hat_Hidden(FileOffset node)
 {
   return (!((node & hiddenMask) == node));
@@ -232,7 +236,7 @@ mkRoot()
 /* Write trace nodes */
 
 FileOffset
-mkModule(char *modname, char *srcfile, bool traced)
+mkModule(char *modname, char *srcfile, Bool traced)
 {
   FileOffset fo;
 
@@ -245,14 +249,15 @@ mkModule(char *modname, char *srcfile, bool traced)
 }
 
 FileOffset
-mkSrcPos(FileOffset moduleTraceInfo,int pos)
+mkSrcPos(FileOffset moduleTraceInfo,int begin,int end)
 {
   FileOffset fo;
 
-  fo = writeTag(SrcPos,1+sizeof(FileOffset)+sizeof(int));
+  fo = writeTag(SrcPos,1+sizeof(FileOffset)+2*sizeof(int));
   writeFileOffset(moduleTraceInfo);
-  writeInt(pos);
-  HIDE(fprintf(stderr,"\tmkSrcPos -> 0x%x\n",fo);)
+  writeInt(begin);
+  writeInt(end);
+  HIDE(fprintf(stderr,"\tmkSrcPos %d %d -> 0x%x\n",begin,end,fo);)
   return fo;
 }
 
@@ -1380,11 +1385,10 @@ mkProjection(FileOffset parent,FileOffset use,FileOffset exp)
 FileOffset
 mkHidden(FileOffset parent)
 {
-  FileOffset fo;
-
-  fo = writeTag(ExpHidden,1 + 2*sizeof(FileOffset));
+  FileOffset fo = writeTag(ExpHidden,1 + 3*sizeof(FileOffset));
   writeFileOffset(parent);
   writeFileOffset(unevaluated);
+  writeFileOffset((FileOffset)NULL);  /* initially empty child list */
   HIDE(fprintf(stderr,"\tmkHidden 0x%x 0x%x -> 0x%x\n",parent,Unevaluated,fo);)
   return (fo|~hiddenMask);
 }
@@ -1418,58 +1422,66 @@ mkLambda()
 {   return (htonl(Lambda)); }
 
 FileOffset
-mkVariable(FileOffset module,int pos,int fixity,int arity,char *name
-	  ,bool local)
+mkDoLambda()
+{   return (htonl(DoLambda)); }
+
+FileOffset
+mkVariable(FileOffset module,int begin,int end,int fixity,int arity,char *name
+	  ,Bool local)
 {
   FileOffset fo;
 
   fo = writeTag(AtomVariable | (local?LocalDef:0)
-               ,1+sizeof(FileOffset)+sizeof(int)+2*sizeof(byte)
+               ,1+sizeof(FileOffset)+2*sizeof(int)+2*sizeof(byte)
                 +stringSize(name));
   writeFileOffset(module);
-  writeInt(pos);
+  writeInt(begin);
+  writeInt(end);
   writeByte(fixity);
   writeByte(arity);
   writeString(name);
-  HIDE(fprintf(stderr,"\tmkVariable 0x%x %d %d %d %s %d -> 0x%x\n",module,pos,fixity,arity,name,local,fo);)
+  HIDE(fprintf(stderr,"\tmkVariable 0x%x %d %d %d %d %s %d -> 0x%x\n",module,begin,end,fixity,arity,name,local,fo);)
   return fo;
 }
 
 FileOffset
-mkConstructor(FileOffset module,int pos,int fixity,int arity,char *name)
+mkConstructor(FileOffset module,int begin,int end,int fixity,int arity
+             ,char *name)
 {
   FileOffset fo;
 
   fo = writeTag(AtomConstructor
-               ,1+sizeof(FileOffset)+sizeof(int)+2*sizeof(byte)
+               ,1+sizeof(FileOffset)+2*sizeof(int)+2*sizeof(byte)
                 +stringSize(name));
   writeFileOffset(module);
-  writeInt(pos);
+  writeInt(begin);
+  writeInt(end);
   writeByte(fixity);
   writeByte(arity);
   writeString(name);
-  HIDE(fprintf(stderr,"\tmkConstructor 0x%x %d %d %d %s %d -> 0x%x\n",module,pos,fixity,arity,name,fo);)
+  HIDE(fprintf(stderr,"\tmkConstructor 0x%x %d %d %d %d %s %d -> 0x%x\n",module,begin,end,fixity,arity,name,fo);)
   return fo;
 }
 
 FileOffset
-mkConstructorWFields(FileOffset module,int pos,int fixity,int arity,char *name
-	            ,FileOffset labels[])
+mkConstructorWFields(FileOffset module,int begin,int end,int fixity,int arity
+                    ,char *name,FileOffset labels[])
 {
   int i;
   FileOffset fo;
 
   fo = writeTag(AtomConstructor|HasFields
-	       ,1+(1+arity)*sizeof(FileOffset)+sizeof(int)+2*sizeof(byte)
+	       ,1+(1+arity)*sizeof(FileOffset)+2*sizeof(int)+2*sizeof(byte)
                 +stringSize(name));
   writeFileOffset(module);
-  writeInt(pos);
+  writeInt(begin);
+  writeInt(end);
   writeByte(fixity);
   writeByte(arity);
   writeString(name);
   for (i=0;i<arity;i++) 
     writeFileOffset(labels[i]);
-  HIDE(fprintf(stderr,"\tmkConstructorWFields 0x%x %d %d %d %s -> 0x%x\n",module,pos,fixity,arity,name,fo);)
+  HIDE(fprintf(stderr,"\tmkConstructorWFields 0x%x %d %d %d %d %s -> 0x%x\n",module,begin,end,fixity,arity,name,fo);)
   return fo;
 }
 
@@ -1485,6 +1497,19 @@ mkAbstract(char *description)
   return fo;
 }
 
+FileOffset
+mkListCons(FileOffset elem,FileOffset tail)
+{
+  FileOffset fo;
+
+  fo = writeTag(ListCons,1 + 2*sizeof(FileOffset));
+  writeFileOffset(elem);
+  writeFileOffset(tail);
+  HIDE(fprintf(stderr,"\tmkListCons 0x%x 0x%x -> 0x%x\n",elem,tail,fo);)
+  return fo;
+}
+
+
 /* Update node that it was entered */
 
 void entResult(FileOffset node,FileOffset use)
@@ -1670,6 +1695,7 @@ void hat_putInBuffer(FileOffset at, FileOffset new) {
 }
 
 void hat_dumpBuffer() {
+
   int i;
 
   for (i=0;i<buffertop;i++) {
@@ -1696,7 +1722,7 @@ void hat_reduce(FileOffset at, FileOffset offset, FileOffset entry)
 /* Buffering scheme for sequential writing with updates */
 
 #define WriteBufferNo 4  /* minimum: 1 */
-#define WriteBufferSize 20000  /* minimum: size of largest node */
+#define WriteBufferSize 40000  /* minimum: size of largest node */
 
 typedef struct writeBuffer {
   FileOffset begin;  /* in host byte order */
@@ -1797,6 +1823,7 @@ updateFileOffset (FileOffset at, FileOffset entry)
   }
 }
 
+/* size of string when stored in hat file (including length information) */
 static int
 stringSize(char *s)
 {
@@ -1834,3 +1861,105 @@ dumpBuffers()
   }
 }
 
+
+/* Store children of Hidden nodes in linked list of ListCons nodes 
+ * The child pointer of every Hidden node unequal NULL is kept in 
+ * a HiddenChildren buffer. That buffer is written at the end.
+ * The buffer serves both for *reading* the current and writing a 
+ * new child pointer.
+ * This avoids having to read the trace; the assumption is that
+ * there are few Hidden nodes with children, hence the buffer is small.
+ */
+
+typedef struct hcEntry {
+  FileOffset at;
+  FileOffset child;
+} HCEntry;
+
+#define HCPartSize 8000
+
+typedef struct hcPart {
+  HCEntry entry[HCPartSize];
+  struct hcPart *previous;
+  int top;
+} HCPart;
+
+
+static HCPart *hiddenChildrenSet = NULL ;
+
+static void initHiddenChildrenSet()
+{
+  hiddenChildrenSet = malloc(sizeof(HCPart));
+  if (hiddenChildrenSet == NULL) {
+    fprintf(stderr,"No space for Hat Children set.");
+    exit (-1);
+  }
+  hiddenChildrenSet->top = 0;
+  hiddenChildrenSet->previous = NULL;
+}
+
+/* Can be invoked any time. The hiddenChildrenSet data structure keeps alive. */
+static void dumpHiddenChildrenSet() 
+{
+  int i;
+  HCPart *curHC = hiddenChildrenSet;
+  
+  while (curHC != NULL) {
+    for (i=0;i<curHC->top;i++) {
+      updateFileOffset(curHC->entry[i].at,curHC->entry[i].child);
+    }
+    curHC = curHC->previous;
+  }
+}
+
+void recordChild(FileOffset hidden,FileOffset child)
+{
+  /* basically: read child pointer of Hidden node, write ListCons with
+   * child and read old child pointer, overwrite child pointer of Hidden
+   * node.
+   * The difficulty is that the child pointer is (hopefully) in one of
+   * several buffers, avoiding file read and write access.
+   */
+  FileOffset listTail,newCons;
+  int i;
+  HCPart *curHC;
+  FileOffset at = ntohl(hidden&hiddenMask) + 1 + 2*sizeof(FileOffset);
+
+  for (curHC = hiddenChildrenSet; curHC != NULL; curHC = curHC->previous)
+    for (i = 0; i<curHC->top;i++)
+      if (curHC->entry[i].at == at)
+        goto found;
+
+  /* not found in set, need to add an entry */
+
+  if (hiddenChildrenSet->top<HCPartSize) {
+    /* add in first part */
+    curHC = hiddenChildrenSet;
+    i = curHC->top++;
+  } else {
+    /* add another part at front */
+    curHC = malloc(sizeof(HCPart));
+    if (curHC == NULL) {
+      fprintf(stderr,"No space for Hat Children set.");
+      exit (-1);
+    }
+    curHC->previous = hiddenChildrenSet; 
+    hiddenChildrenSet = curHC;
+    curHC->top = 1;
+    i = 0;
+  }
+  curHC->entry[i].at = at;
+  curHC->entry[i].child = (FileOffset)NULL;
+
+ found:
+
+  /* read current child tail of Hidden node */
+  listTail = curHC->entry[i].child;
+ 
+  /* create extended children list */
+  newCons = mkListCons(child,listTail);
+    
+  /* overwrite child tail pointer of Hidden node */
+  curHC->entry[i].child = newCons;
+}
+
