@@ -1,6 +1,6 @@
 import Maybe
 import List		(sort,isPrefixOf,isSuffixOf,intersperse)
-import Monad		(when)
+import Monad		(when,liftM)
 import System
 import Char		(isDigit,digitToInt,isUpper,toUpper,isAlphaNum,isSpace)
 import IO		(hFlush,stdout,hSetBuffering,BufferMode(..))
@@ -9,11 +9,11 @@ import FFIExtensions	(withCString)
 import Observe		(ObserveResult(..)
 			,isFound,isNotFound,isInterrupted,fromFound
 			,newObservation,newObservationSrc)
-import Ident		(Ident(..),collateIdents,getAllIdents,sortIdents
-			,showInfo)
+import Ident		(Ident(..),getIdentAt)
+import Idents		(collateIdents,getAllIdents,sortIdents,showInfo)
 import LowLevel		(FileNode(..),nil,openHatFile,getResult,hatVersionNumber
-			,getSrcRef,srcRefFile,srcRefLine,srcRefCol
-			,getDefnRef,defnFile,defnLine,defnCol)
+			,getSrcRef,getDefnRef)
+import SrcRef		(SrcRef(..),readSrcRef,defnSrcRef)
 import SExp		(SExp(..),Label,prettyEquation,prettyExpression
 			,fileNode2SExp,funId,label,QName(..))
 import HighlightStyle	(cursorUp,cleareol,highlight,Highlight(..),Colour(..)
@@ -22,17 +22,12 @@ import Trie		(Trie,Search(..),emptyTrie,match)
 import TExp		(linearise)
 import Pattern		(topMatchPat,parsePat,lexPat)
 import CmdLine		(initialize,cmdline)
-
-
-spawnDetectCmd  = "xterm -e hat-detect "
-spawnObserveCmd = "xterm -e hat-observe "
-spawnTrailCmd   = "xterm -e hat-trail "
-
-hatView :: FilePath -> Int -> Int -> String
-hatView f y x  = "xterm -e hat-view "++f++" "++show y++" "++show x++" &"
-
-
-shortHelpText  = ":? for help, :q to quit"
+import CommonUI		(hatObserve,hatTrail,hatAnim,hatDetect,hatExplore
+                        ,hatView
+			,shortHelpText
+			,Keep(..),Options(..),initialOptions
+			,OptionCmd(..),optionCmd,onOff,number,safeReadInt
+			,optionsUpdate,showOption)
 
 ----
 data InteractiveState =
@@ -42,35 +37,21 @@ data InteractiveState =
 	, equationsPerPage :: Int
 	, hatFile       :: String
 	, currentPos	:: Int
-	, cutoffdepth	:: Int
-	, unevalMode	:: Bool
-	, stringSugar	:: Bool
-	, listSugar	:: Bool
-	, recursiveMode	:: Bool
 	, showRHS	:: Bool
-	, equations	:: Bool
-	, showQual	:: Bool
-	, filterMode    :: Keep
-	, screenWidth	:: Int
+	, options	:: Options
+	, screenWidth   :: Int
 	, symbolTable   :: ([Ident],[Ident],[Ident])
 	}
-initialState :: InteractiveState
-initialState = State
+initialState :: FilePath -> Int -> InteractiveState
+initialState file width = State
 	{ lastObserved = []
 	, more = False
 	, equationsPerPage = 10
-	, hatFile = ""
+	, hatFile = file
 	, currentPos = 0
-	, cutoffdepth = 10
-	, unevalMode = False
-	, stringSugar = True
-	, listSugar = True
-	, recursiveMode = True
-	, showQual = False
 	, showRHS = True
-	, equations = True
-	, filterMode = Unique
-	, screenWidth = 80
+	, options = initialOptions
+	, screenWidth = width
 	, symbolTable = ([],[],[])
 	}
 ----
@@ -91,7 +72,7 @@ main = do
     (width,_) <- getTerminalSize
     CmdLine.initialize	-- for readline functionality
     let hatFile = rectify (arguments!!0)
-        state   = initialState { screenWidth=width, hatFile=hatFile }
+        state   = initialState hatFile width
     prog <- System.getProgName
     withCString prog (\p-> withCString hatFile (openHatFile p))
     putStrLn (highlight [Foreground Red] "Loading symbol table...")
@@ -112,14 +93,12 @@ main = do
 
 ----
 showObservation :: InteractiveState -> Int -> FileNode -> IO ()
-showObservation state i node = do
+showObservation state@(State{options=opts}) i node = do
   putStr (cursorUp ++ cleareol)	-- delay relies on line-buffered output
-  putStrLn (display (screenWidth state) (cutoffdepth state)
-                    (unevalMode state) (stringSugar state)
-                    (listSugar state) (showQual state) node)
+  putStrLn (display (screenWidth state) opts node)
   where
-  display | equations state && showRHS state = prettyEquation (show i++" ") ""
-          | otherwise                        = prettyExpression (show i++" ")
+  display | equations opts && showRHS state = prettyEquation (show i++" ") ""
+          | otherwise                       = prettyExpression (show i++" ")
 
 
 -- arguments to showObservationList are:
@@ -138,20 +117,19 @@ showObservationList state i max (e:r) = do
 
 makeObserveSrc :: [String] -> ObserveResult
 makeObserveSrc [mod,line,col] | all isDigit line && all isDigit col =
-    newObservationSrc mod (fromJust (stringToInt line))
-                          (fromJust (stringToInt col))
+    newObservationSrc mod (safeReadInt 0 line) (safeReadInt 0 col)
 makeObserveSrc _  | otherwise = NotFound
 
 
 makeObserve :: InteractiveState -> QName -> Maybe QName -> SExp () -> Int
                -> ObserveResult
-makeObserve state ident1 ident2 pat arity =
-    let observed = newObservation ident1 ident2 (recursiveMode state) arity in
+makeObserve State{options=opts} ident1 ident2 pat arity =
+    let observed = newObservation ident1 ident2 (recursiveMode opts) arity in
     if not (isFound observed) then observed
     else ( Found
          . map fromEqn
          . filter (\o-> topMatchPat o pat)
-         . (case filterMode state of
+         . (case filterMode opts of
               All -> id
               _   -> uniqueify)
          . map toSEqn
@@ -226,61 +204,14 @@ getEquationNumber n lastObserved =
 
 ----
 setState :: Mode -> InteractiveState -> InteractiveState
-setState (Uneval b)     state = state {unevalMode=b}
-setState (Strings b)    state = state {stringSugar=b}
-setState (Lists b)      state = state {listSugar=b}
-setState (Recursive b)  state = state {recursiveMode=b}
-setState (Qualify b)    state = state {showQual=b}
-setState (Equations b)  state = state {equations=b}
-setState (Filter k)     state = state {filterMode=k}
-setState (CutOff n)     state = state {cutoffdepth=max n 1}
-setState (Deeper n)     state = state {cutoffdepth=cutoffdepth state + n}
-setState (Shallower n) state = state {cutoffdepth=max (cutoffdepth state - n) 1}
-setState (Group n)      state = state {equationsPerPage=max n 1}
+setState (O o)     state = state {options=optionsUpdate o (options state)}
+setState (Group n) state = state {equationsPerPage=max n 1}
 
-showState (Uneval _)    state = "  unevaluated exprs shown in full ("
-                                ++ highlight [Underscore] "uneval"
-                                ++ ") "
-                                ++ showOnOff (unevalMode state)
-showState (Strings _)   state = "  string sugar ("
-                                ++ highlight [Underscore] "strSugar"
-                                ++ ") "
-                                ++ showOnOff (stringSugar state)
-showState (Lists _)     state = "  list sugar ("
-                                ++ highlight [Underscore] "listSugar"
-                                ++ ") "
-                                ++ showOnOff (listSugar state)
-showState (Recursive _) state = "  "++highlight [Underscore] "recursive"
-                                ++ " calls are shown "
-                                ++ showOnOff (recursiveMode state)
-showState (Qualify _) state   = "  identifier names are shown "
-                                ++ highlight [Underscore] "qualified"
-                                ++ " "++showOnOff (showQual state)
-showState (Equations _) state = "  right-hand-side of "
-                                ++ highlight [Underscore] "equations"
-                                ++ " are shown "
-                                ++ showOnOff (equations state)
-showState (Filter _)    state = "  show ("
-				++ concat
-				     (intersperse ","
-                                       (map keep [All,Unique,MostGeneralSoFar
-				                 ,MostGeneral]))
-                                ++ ") equations"
-  where
-    keep k | k==filterMode state = highlight [Underscore] (show k)
-           | otherwise           = show k
-showState (CutOff _) state    = "  expression "
-                                ++ highlight [Underscore] "cutoff"
-                                ++ " depth = "++ show (cutoffdepth state)
-showState (Deeper _) state    = showState (CutOff 0) state
-showState (Shallower _) state = showState (CutOff 0) state
-showState (Group _)  state    = "  equations per "
-                                ++ highlight [Underscore] "group"
-                                ++ " = "++ show (equationsPerPage state)
-
-showOnOff :: Bool -> String
-showOnOff True  = "[on]"
-showOnOff False = "[off]"
+showState :: Mode -> InteractiveState -> String
+showState (O o)     state  = showOption o (options state)
+showState (Group _) state  = "  "++ highlight [Underscore] "group"
+                             ++ "\t\t" ++ show (equationsPerPage state)++"\t"
+                             ++ "number of equations per group"
 
 ----
 doCommand :: Cmd -> InteractiveState -> IO()
@@ -307,7 +238,7 @@ doCommand (InfoC mod) state =
 	interactive state
 doCommand Resize state =
      do (width,_) <- getTerminalSize
-        interactive state {screenWidth=width}
+        interactive (state{screenWidth=width})
 doCommand Count state =
      do when (more state)
              (putStrLn "One moment, this may take a while...")
@@ -317,8 +248,9 @@ doCommand Count state =
 
 doCommand Status state =
      do mapM_ (\m-> putStrLn (showState m state))
-              [ Uneval True, Strings True, Lists True, Recursive True
-              , Qualify True, Equations True, Filter All, Group 0, CutOff 0 ]
+              [ O (Uneval True), O (Strings True), O (Lists True)
+              , O (Recursive True), O (Qualify True), O (Equations True)
+              , Group 0, O (CutOff 0), O (Filter All) ]
         interactive state
 doCommand (Set mode) state =
      do let state' = setState mode state
@@ -332,18 +264,15 @@ doCommand (Source n) state =
      do node <- getEquationNumber n (lastObserved state)
         let sr = getSrcRef (fromJust node)
 	when (isJust node && sr /= LowLevel.nil)
-             (do System.system (hatView (srcRefFile sr)
-                                        (srcRefLine sr)
-                                        (srcRefCol sr))
+             (do System.system (hatView (readSrcRef sr))
                  return ())
 	interactive state
 doCommand (Definition n) state =
      do node <- getEquationNumber n (lastObserved state)
-        let sr = getDefnRef (fromJust node)
+        let atom = getDefnRef (fromJust node)
 	when (isJust node)
-             (do System.system (hatView (defnFile sr)
-                                        (defnLine sr)
-                                        (defnCol sr))
+             (do defnSR <- liftM defnSrcRef (getIdentAt atom)
+                 System.system (hatView defnSR)
                  return ())
 	interactive state
 
@@ -411,33 +340,36 @@ startExternalTool tool state =
   let file = hatFile state
       checkOK errcode s = when (errcode/=ExitSuccess)
                                (putStrLn ("ERROR: Unable to start "++s++"."))
-      esc []        = []
-      esc ('"':xs) = '\\': '"': esc xs
-      esc ('$':xs) = '\\': '$': esc xs
-      esc ('`':xs) = '\\': '`': esc xs
-      esc (x:xs)    = x: esc xs
   in case tool of
       Choose -> do
-          putStrLn "choose :observe, :detect, or :trail"
+          putStrLn "choose :observe, :detect, :trail, :animate, or :explore"
       Observe pat -> do
-          errcode <- system (spawnObserveCmd++file
-                             ++(if null pat then "&"
-                                else " \""++esc pat++"\" &"))
+          errcode <- system (hatObserve file pat)
           checkOK errcode "hat-observe"
       Trail num -> do
           node <- getEquationNumber num (lastObserved state)
           if isJust node then do
-              let id = show (int (fromJust node))
-              errcode <- system (spawnTrailCmd++file++" -remote "++id++" &")
+              errcode <- system (hatTrail file (fromJust node))
               checkOK errcode "hat-trail"
             else putStrLn ("ERROR: Equation "++show num++" not available.")
+      Anim num -> do
+          node <- getEquationNumber num (lastObserved state)
+          if isJust node then do
+              errcode <- system (hatAnim file (fromJust node))
+              checkOK errcode "hat-anim"
+            else putStrLn ("ERROR: Equation "++show num++" not available.")
       Detect num -> do
           node <- getEquationNumber num (lastObserved state)
           if isJust node then do
-              let id = show (int (fromJust node))
-              errcode <- system (spawnDetectCmd++file++" -remote "++id++" &")
+              errcode <- system (hatDetect file (fromJust node))
               checkOK errcode "hat-detect"
             else putStrLn ("ERROR: Equation "++show num++" not available.")
+      Explore num -> do
+          node <- getEquationNumber num (lastObserved state)
+          if isJust node then do
+              errcode <- system (hatExplore file (fromJust node))
+              checkOK errcode "hat-explore"
+            else putStrLn ("ERROR: Equation "++show num++" not available.")
 
 
 interactiveHelp s = do
@@ -445,18 +377,20 @@ interactiveHelp s = do
               else detailedHelp s)
   where
     basicHelptext = "\
-\------------ : help <cmd> for more detail ---------------------------------\n\
+\------------ :help <cmd> for more detail ----------------------------------\n\
 \ <query>            observe named function (:help query for pattern syntax)\n\
 \ <RETURN>           show more observations (if available)\n\
 \ :info              see a list of all observable functions\n\
 \ :Info              see a list of all observable data constructors\n\
 \ :detect <n>        start hat-detect on equation <n>\n\
 \ :trail <n>         start hat-trail browser on equation <n>\n\
+\ :animate <n>       start hat-anim on equation <n>\n\
+\ :explore <n>       start hat-explore on equation <n>\n\
 \ :observe [query]   start hat-observe in a new window with the new query\n\
 \ :source <n>        show the source application for equation <n>\n\
 \ :Source <n>        show the source definition for identifier in eqn <n>\n\
 \ :set               show all current mode settings\n\
-\ :set <flag>        change one mode setting\n\
+\ :set <flag>        change one mode setting (:help set for more detail)\n\
 \ :+[n]              short-cut to increase cutoff depth by <n> (default 1)\n\
 \ :-[n]              short-cut to decrease cutoff depth by <n> (default 1)\n\
 \ :resize            detect new window size for pretty-printing\n\
@@ -511,9 +445,16 @@ interactiveHelp s = do
 \                  cutoff <n>           cut-off depth for deeply nested exprs"
       | s `isPrefixOf` "detect"  = "\
 \ :detect <n>        start hat-detect on equation <n>\n\
-\    hat-detect is not currently available.  It is a browser that asks\n\
+\    hat-detect is a browser that asks\n\
 \    questions about whether certain equations are correct or incorrect,\n\
 \    using an automatic method to locate the source of a bug."
+      | s `isPrefixOf` "explore"  = "\
+\ :explore <n>        start hat-explore on equation <n>\n\
+\    hat-detect allows source-based browsing."
+      | s `isPrefixOf` "animate"  = "\
+\ :animate <n>       start hat-anim on equation <n>\n\
+\    hat-anim is new and potentially buggy.  It is a browser that animates\n\
+\    the reduction of an expression to its final value."
       | s `isPrefixOf` "trail"  = "\
 \ :trail <n>         start hat-trail browser on equation <n>\n\
 \    hat-trail is an interactive browser that permits exploration backwards\n\
@@ -555,11 +496,9 @@ data Cmd  = Pattern String | Location [String]
           | Help String | Quit | Unknown
           | StartTool Tool | Source Int | Definition Int
           | Status | Set Mode | Shell String | Resize
-data Tool = Trail Int | Observe String | Detect Int | Choose
-data Mode = Uneval Bool | Strings Bool | Lists Bool | Recursive Bool
-          | CutOff Int  | Deeper Int   | Shallower Int | Qualify Bool
-          | Group Int   | Filter Keep  | Equations Bool
-data Keep = All | Unique | MostGeneralSoFar | MostGeneral deriving Eq
+data Tool = Trail Int | Anim Int | Observe String | Detect Int | Explore Int 
+          | Choose
+data Mode = Group Int | O OptionCmd
 
 getCommand :: String -> IO Cmd
 getCommand prompt = do
@@ -575,8 +514,12 @@ getCommand prompt = do
               | cmd `isPrefixOf` "location" -> return (Location ss)
               | cmd `isPrefixOf` "detect"   ->
 				return (number (StartTool . Detect) ss 0)
+              | cmd `isPrefixOf` "explore"   ->
+				return (number (StartTool . Explore) ss 0)
               | cmd `isPrefixOf` "trail"    ->
 				return (number (StartTool . Trail) ss 0)
+              | cmd `isPrefixOf` "animate"    ->
+				return (number (StartTool . Anim) ss 0)
               | cmd `isPrefixOf` "observe"  ->
 				return (StartTool (Observe (unwords ss)))
               | cmd `isPrefixOf` "source"   -> return (number Source ss 0)
@@ -586,67 +529,18 @@ getCommand prompt = do
               | cmd `isPrefixOf` "resize"   -> return Resize
               | cmd `isPrefixOf` "set"      ->
                   case ss of
-                      [] -> return Status
-                      (m:sss)
-                          | m `isPrefixOf` "uneval" ->
-					return (onOff Uneval sss)
-                          | m `isPrefixOf` "strSugar" ->
-					return (onOff Strings sss)
-                          | m `isPrefixOf` "listSugar" ->
-					return (onOff Lists sss)
-                          | m `isPrefixOf` "recursive" ->
-					return (onOff Recursive sss)
-                          | m `isPrefixOf` "all" ->
-					return (Set (Filter All))
-                          | m `isPrefixOf` "unique" ->
-					return (Set (Filter Unique))
-                          | m `isPrefixOf` "generalise" ->
-					return (Set (Filter MostGeneralSoFar))
-                          | m `isPrefixOf` "mostgeneral" ->
-					return (Set (Filter MostGeneral))
-                          | m `isPrefixOf` "cutoff" ->
-					return (Set (number CutOff sss 10))
-                          | m `isPrefixOf` "group" ->
-					return (Set (number Group sss 10))
-                          | m `isPrefixOf` "qualified" ->
-					return (onOff Qualify sss)
-                          | m `isPrefixOf` "equations" ->
-					return (onOff Equations sss)
-                          | m `isPrefixOf` "eqns" ->
-					return (onOff Equations sss)
-                          | otherwise -> return Unknown
+                      []      -> return Status
+                      (s:sss) ->
+                        case optionCmd ss of
+                          Just o  -> return (Set (O o))
+                          Nothing -> if s `isPrefixOf` "group"
+                                     then return (Set (number Group sss 10))
+                                     else return Unknown
               | head cmd == '?' -> return (Help (unwords (tail cmd:ss)))
-              | head cmd == '+' ->
-                          return (Set (number Deeper (tail cmd:ss) 1))
-              | head cmd == '-' ->
-                          return (Set (number Shallower (tail cmd:ss) 1))
+              | head cmd == '+' -> return (number (Set . O . Deeper)
+                                                  (tail cmd:ss) 1)
+              | head cmd == '-' -> return (number (Set . O . Shallower)
+                                                  (tail cmd:ss) 1)
               | head cmd == '!' -> return (Shell (unwords (tail cmd:ss)))
               | otherwise  ->  return Unknown
 
-number :: (Int->a) -> [String] -> Int -> a
-number cons s def = (maybe (cons def) cons . stringToInt . unwords) s
-
-onOff :: (Bool->Mode) -> [String] -> Cmd
-onOff mode s | null s = Set (mode True)
-             | otherwise = case head s of "on" -> Set (mode True)
-                                          "active" -> Set (mode True)
-                                          "off" -> Set (mode False)
-                                          "no" -> Set (mode False)
-                                          _ -> Unknown
-
-stringToInt :: String -> Maybe Int
-stringToInt s = stringToInt' True 0 s
- where
-  stringToInt' True _ ('#':r) = stringToInt' True 0 r -- skip "#" at beginning
-  stringToInt' True _ (' ':r) = stringToInt' True 0 r -- skip " " at beginning
---stringToInt' False i (' ':r) = Just i
-  stringToInt' first i [] = if first then Nothing else  Just i
-  stringToInt' _ i (c:r)
-      | isDigit c = stringToInt' False (i*10+(digitToInt c)) r
-      | otherwise = Nothing
-
-instance Show Keep where
-  showsPrec p All		= showString "all"
-  showsPrec p Unique 		= showString "unique"
-  showsPrec p MostGeneralSoFar	= showString "generalise"
-  showsPrec p MostGeneral	= showString "mostgeneral"
