@@ -1,9 +1,11 @@
 #include <unistd.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <errno.h>
 #include <sys/stat.h>
 #include <signal.h>
+#include <assert.h>
 #include "art.h"
 #include "artutils.h"
 #include "pathutils.h"
@@ -33,63 +35,12 @@ FileOffset q_position;
 FILE	*HatFileRandom, *HatFileSeq, *OutputFile, *BridgeFile;
 unsigned filesize=0, outputsize=0;
 char	*progname;		/* name of browser, not of the subject */
-char	*dir;
-bool	interrupt=False;	/* ^C interrupt at program runtime */
-bool	hat_interrupted=False;	/* ^C interrupt in browsing tool */
+Bool	interrupt=False;	/* ^C interrupt at program runtime */
+Bool	hat_interrupted=False;	/* ^C interrupt in browsing tool */
 
-/* The initialise() routine ensures that all files are available,
- * and opens them ready for further action.  It also fills in the
- * errorRoot and errorMsg globals.
- */
-void
-initialise (int argc, char **argv)
-{
-  int err;
-  char header[8];
-  char *arg;
+FileOffset
+currentfilepos (void) { return q_position; }
 
-  if ((argc==4)&&(strcmp(argv[2],"-remote")==0)) {
-    remoteStartNode = atoi(argv[3]);
-  } else if (argc!=2) {
-    fprintf(stderr,"  [%s]\n",argv[0]);
-    fprintf(stderr,"\tUsage: %s program[.hat]\n",basename(argv[0],0));
-    exit(1);
-  }
-  arg=argv[1];
-
-  dir = dirname(arg);
-  arg = basename(arg,".hat");
-  chdir(dir);
-  progname   = basename(argv[0],0);
-			/* for error messages - /not/ the prog being debugged */
-  filesize   = sizeFile(arg,".hat");
-  outputsize = sizeFile(arg,".hat.output");
-  HatFileRandom = openFile(arg,".hat");		/* one file for random access */
-  OutputFile = openFile(arg,".hat.output");
-  BridgeFile = openFile(arg,".hat.bridge");
-  HatFileSeq = openFile(arg,".hat");		/* another for sequential */
-  q_position = 0;
-
-  err = fread(header,sizeof(char),8,HatFileRandom);
-  if (err!=8) {
-    fprintf(stderr,"%s (error): file %s/%s is too short\n",progname,dir,arg);
-    exit(1);
-  }
-  if (strncmp(header,"Hat",3)) {
-    fprintf(stderr,"%s (error): file %s in directory %s\n",progname,arg,dir);
-    fprintf(stderr,"   does not appear to be a Hat archive.  Quitting.\n");
-    exit(1);
-  }
-  if (strncmp(header+3,VERSION,4)) {
-    fprintf(stderr,"%s (warning): file %s in directory %s\n",progname,arg,dir);
-    fprintf(stderr,"   appears to be a Hat archive in format %s\n",header+3);
-    fprintf(stderr,"   but this tool deals with format version %s\n",VERSION);
-    fprintf(stderr,"   I'm continuing, but there may be unexpected errors.\n");
-  }
-  errorRoot = readFO();
-  errorMsg  = readFO();
-  if (errorMsg==Entered) interrupt=True;
-}
 
 void
 finalise (void)
@@ -116,7 +67,7 @@ openFile (char* base, char* ext)
   if (file = fopen(filename,"r")) {
     return file;
   } else {
-    fprintf(stderr,"%s: cannot open %s/%s\n",progname,dir,filename);
+    fprintf(stderr,"%s: cannot open %s\n",progname,filename);
     exit(1);
   }
 }
@@ -273,7 +224,7 @@ q_peek (void)
  * given the location of the module descriptor in the file.
  */
 void
-readModuleAt (FileOffset fo, char** modname, char** srcname, bool* traced)
+readModuleAt (FileOffset fo, char** modname, char** srcname, Bool* traced)
 {
   char c;
 
@@ -302,6 +253,7 @@ readAtomAt (FileOffset fo)
   FileOffset modpos;
   Ident* id=0;
   int defnpos;
+  int defnposend;
 
   if (fo==Lambda) {
     id = (Ident*)malloc(sizeof(Ident));
@@ -309,6 +261,16 @@ readAtomAt (FileOffset fo)
     id->modname  = id->srcname = "";
     id->fixity   = noFixity;
     id->defnline = id->defncolumn = 0;
+    id->defnlineend = id->defncolumnend = 0;
+    id->isTraced = False;
+    id->atomtype = Abstract;
+  } else if (fo==DoLambda) {
+    id = (Ident*)malloc(sizeof(Ident));
+    id->idname   = "do";
+    id->modname  = id->srcname = "";
+    id->fixity   = noFixity;
+    id->defnline = id->defncolumn = 0;
+    id->defnlineend = id->defncolumnend = 0;
     id->isTraced = False;
     id->atomtype = Abstract;
   } else {
@@ -320,11 +282,14 @@ readAtomAt (FileOffset fo)
           id = (Ident*)malloc(sizeof(Ident));
           modpos = readFO();
           fread(&defnpos,sizeof(int),1,HatFileRandom);
+          fread(&defnposend,sizeof(int),1,HatFileRandom);
           fread(&(id->fixity),sizeof(char),1,HatFileRandom);
           fread(&(id->arity),sizeof(char),1,HatFileRandom);
           id->idname       = readString();
           id->defnline     = ntohl(defnpos)/10000;
           id->defncolumn   = ntohl(defnpos)%10000;
+          id->defnlineend  = ntohl(defnposend)/10000;
+          id->defncolumnend= ntohl(defnposend)%10000;
           if (lower5(c)==AtomVariable) {
               id->atomtype = Variable;
           } else if (hasFields(c)) {
@@ -340,7 +305,8 @@ readAtomAt (FileOffset fo)
           id->idname   = readString();
           id->modname  = id->srcname = "";
           id->fixity   = noFixity;
-          id->defnline = id->defncolumn = 0;
+          id->defnline = id->defncolumn = id->defnlineend = 
+            id->defncolumnend = 0;
           id->isTraced = False;
           id->atomtype = Abstract;
           tmp = (char*)malloc(strlen(id->idname)+3);
@@ -354,7 +320,7 @@ readAtomAt (FileOffset fo)
           break;
     }
   }
-  HIDE(fprintf(stderr,"readAtomAt 0x%x -> %s %s %s %d %d %d\n",fo,id->idname,id->modname,id->srcname,id->defnline,id->defncolumn,id->fixity);)
+  HIDE(fprintf(stderr,"readAtomAt 0x%x -> %s %s %s %d %d %d %d %d\n",fo,id->idname,id->modname,id->srcname,id->defnline,id->defncolumn,id->defnlineend,id->defncolumnend,id->fixity);)
   return id;
 }
 
@@ -370,8 +336,9 @@ readSRAt (FileOffset fo)
   FileOffset modpos;
   char *modname, *srcname;
   int usepos;
+  int useposend;
   char c;
-  bool dummy;
+  Bool dummy;
   SrcRef *sr;
 
   HIDE(fprintf(stderr,"readSRAt 0x%x\n",fo);)
@@ -387,15 +354,30 @@ readSRAt (FileOffset fo)
     modpos = readFO();
     fread(&usepos,sizeof(int),1,HatFileRandom);
     usepos = ntohl(usepos);
+    fread(&useposend,sizeof(int),1,HatFileRandom);
+    useposend = ntohl(useposend);
     readModuleAt(modpos, &modname, &(sr->srcname), &dummy);
     sr->line    = usepos/10000;
     sr->column  = usepos%10000;
+    sr->lineend = useposend/10000;
+    sr->columnend = useposend%10000;
     return sr;
   } else {
     return (SrcRef*)0;
   }
 }
 
+char*
+srFile (SrcRef* sr)      { return sr->srcname; }
+int
+srLine (SrcRef* sr)      { return sr->line; }
+int
+srColumn (SrcRef* sr)    { return sr->column; }
+int
+srLineEnd (SrcRef* sr)   { return sr->lineend; }
+int
+srColumnEnd (SrcRef* sr) { return sr->columnend; }
+
 
 /* readValueAt() returns a struct containing a readable notation of the
  * value-Exp stored at the given location in the file.
@@ -413,8 +395,10 @@ readValueAt (FileOffset fo)
   id->srcname  = (char*)0;
   id->fixity   = noFixity;
   id->arity    = 0;
-  id->defnline   = 0;
-  id->defncolumn = 0;
+  id->defnline      = 0;
+  id->defncolumn    = 0;
+  id->defnlineend   = 0;
+  id->defncolumnend = 0;
   id->isTraced   = False;
   id->atomtype   = Literal;
 
@@ -501,11 +485,13 @@ readValueAt (FileOffset fo)
 }
 
 /* Skip a complete node in the file, given that the tag byte has already
- * been read. (Sequential Access)
+ * been read. (Sequential Access)  Returns the parent pointer of the node,
+ * if it has one, although in many cases this is ignored.
  */
-void
+FileOffset
 q_skipNode (char tag)
 {
+  FileOffset parent=0;
   switch (lower5(tag)) {
     case Module:
         q_readString(); q_readString();
@@ -513,89 +499,91 @@ q_skipNode (char tag)
     case SrcPos:
         q_readFO();
         { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }
+        { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }
         break;
     case ExpApp:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO(); q_readFO(); q_readFO();
+        parent = q_readFO(); q_readFO(); q_readFO();
         { char arity; q_fread(&arity,sizeof(char),1,HatFileSeq);
           while (arity--) { q_readFO(); } }
         break;
     case ExpValueApp:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO(); q_readFO();
+        parent = q_readFO(); q_readFO();
         { char arity; q_fread(&arity,sizeof(char),1,HatFileSeq);
           while (arity--) { q_readFO(); } }
         break;
     case ExpChar:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO();
+        parent = q_readFO();
         { char x; q_fread(&x,sizeof(char),1,HatFileSeq); }
         break;
     case ExpInt:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO();
+        parent = q_readFO();
         { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }
         break;
     case ExpInteger:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO();
+        parent = q_readFO();
         q_readString();
         break;
     case ExpRat:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO();
+        parent = q_readFO();
         { int x[2]; q_fread(&x[0],sizeof(int),2,HatFileSeq); }
         break;
     case ExpRational:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO();
+        parent = q_readFO();
         q_readString(); q_readString();
         break;
     case ExpFloat:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO();
+        parent = q_readFO();
         { float x; q_fread(&x,sizeof(float),1,HatFileSeq); }
         break;
     case ExpDouble:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO();
+        parent = q_readFO();
         { double x; q_fread(&x,sizeof(double),1,HatFileSeq); }
         break;
     case ExpValueUse:
     case ExpConstUse:
     case ExpProjection:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO(); q_readFO();
+        parent = q_readFO(); q_readFO();
         break;
     case ExpConstDef:
-        q_readFO(); q_readFO(); q_readFO();
+        parent = q_readFO(); q_readFO(); q_readFO();
         break;
     case ExpGuard:
     case ExpCase:
     case ExpIf:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO(); q_readFO(); q_readFO();
+        parent = q_readFO(); q_readFO(); q_readFO();
         break;
     case ExpFieldUpdate:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO(); q_readFO(); q_readFO();
+        parent = q_readFO(); q_readFO(); q_readFO();
         { char arity; q_fread(&arity,sizeof(char),1,HatFileSeq);
           arity = 2*arity;
           while (arity--) { q_readFO(); } }
         break;
     case ExpHidden:
-        q_readFO(); q_readFO();
+        parent = q_readFO(); q_readFO(); q_readFO();
         break;
     case ExpForward:
-        q_readFO();
+        parent = q_readFO();	/* note different to value of parentNode() */
         break;
     case ExpDoStmt:
         if (hasSrcPos(tag)) q_readFO();
-        q_readFO();
+        parent = q_readFO();	/* note different to value of parentNode() */
         break;
     case AtomVariable:
         q_readFO();
         { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }
+        { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }
         { char x; q_fread(&x,sizeof(char),1,HatFileSeq); }
         { char x; q_fread(&x,sizeof(char),1,HatFileSeq); }
         q_readString();
@@ -603,6 +591,7 @@ q_skipNode (char tag)
     case AtomConstructor:
         q_readFO();
         { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }
+        { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }
         { char x; q_fread(&x,sizeof(char),1,HatFileSeq); }
         { char arity; q_fread(&arity,sizeof(char),1,HatFileSeq);
           q_readString();
@@ -611,8 +600,12 @@ q_skipNode (char tag)
     case AtomAbstract:
         q_readString();
         break;
+    case ListCons:
+        q_readFO();
+        q_readFO();
     default: break;
   }
+  return parent;
 }
 
 
@@ -635,13 +628,14 @@ readTraceAt (FileOffset fo, char** expr, SrcRef** sr, int* infix
     return fo;
   }
 
-  if (fo<=Lambda) {
+  if (fo<=DoLambda) {
     switch (fo) {
       case Root:        sprintf(buf,"<Root>"); break;
       case Unevaluated: sprintf(buf,"_"); break;
       case Entered:     sprintf(buf,"_|_"); break;
       case Interrupted: sprintf(buf,"{^C}"); break;
       case Lambda:      sprintf(buf,"(\\..)"); break;
+      case DoLambda:    sprintf(buf,"do"); break;
     }
     *expr = strdup(buf);
     *infix = noFixity;
@@ -917,10 +911,10 @@ openHatFile (char* prog, char* arg)
     fprintf(stderr,"   does not appear to be a Hat archive.  Quitting.\n");
     exit(1);
   }
-  if (strncmp(header+3,VERSION,4)) {
+  if (strncmp(header+3,FILEVERSION,4)) {
     fprintf(stderr,"%s (warning): file %s\n",progname,arg);
     fprintf(stderr,"   appears to be a Hat archive in format %s\n",header+3);
-    fprintf(stderr,"   but this tool deals with format version %s\n",VERSION);
+    fprintf(stderr,"   but this tool deals with format version %s\n",FILEVERSION);
     fprintf(stderr,"   I'm continuing, but there may be unexpected errors.\n");
   }
   errorRoot = readFO();
@@ -978,7 +972,7 @@ int
 getNodeType (FileOffset fo)
 {
   char c;
-  if (fo<=Lambda) return -1;
+  if (fo<=DoLambda) return -1;
   HIDE(fprintf(stderr,"getNodeType 0x%x\n",fo);)
   freadAt(fo,&c,sizeof(char),1,HatFileRandom);
   return (int)(lower5(c));	/* lower 5 bits are the identifying tag */
@@ -990,9 +984,10 @@ parentNode (FileOffset fo)
 {
   char c;
   HIDE(fprintf(stderr,"parentNode 0x%x\n",fo);)
-  if (fo<=Lambda) return 0;
+  if (fo<=DoLambda) return 0;
   freadAt(fo,&c,sizeof(char),1,HatFileRandom);
   switch (lower5(c)) {
+    case ListCons:
     case Module:
     case SrcPos:
     case AtomVariable:
@@ -1072,10 +1067,10 @@ getFixity (FileOffset fo)
   } else
     return (int)noFixity;
 }
-bool
+Bool
 isLiteral (FileOffset fo)
 {
-  bool b;
+  Bool b;
   Ident *name = readValueAt(fo);
   if (name) {
     b = (name->atomtype == Literal);
@@ -1084,10 +1079,10 @@ isLiteral (FileOffset fo)
   } else
     return False;
 }
-bool
+Bool
 isConstructor (FileOffset fo)
 {
-  bool b;
+  Bool b;
   Ident *name = readValueAt(fo);
   if (name) {
     b = (name->atomtype == Constructor) || (name->atomtype == ConstrFields);
@@ -1096,10 +1091,10 @@ isConstructor (FileOffset fo)
   } else
     return False;
 }
-bool
+Bool
 isConstrFields (FileOffset fo)
 {
-  bool b;
+  Bool b;
   char c;
   Ident *name;
   freadAt(fo,&c,sizeof(char),1,HatFileRandom);
@@ -1122,10 +1117,10 @@ isConstrFields (FileOffset fo)
   } else
     return False;
 }
-bool
+Bool
 isLambda (FileOffset fo)
 {
-  bool b; unsigned char c;
+  Bool b; unsigned char c;
   HIDE(fprintf(stderr,"isLambda 0x%x\n",fo);)
   freadAt(fo,&c,sizeof(unsigned char),1,HatFileRandom);
   switch (lower5(c)) {
@@ -1140,6 +1135,24 @@ isLambda (FileOffset fo)
         break;
   }
 }
+Bool
+isDoLambda (FileOffset fo)
+{
+  Bool b; unsigned char c;
+  HIDE(fprintf(stderr,"isDoLambda 0x%x\n",fo);)
+  freadAt(fo,&c,sizeof(unsigned char),1,HatFileRandom);
+  switch (lower5(c)) {
+    case ExpValueUse:
+        if (hasSrcPos(c)) { readFO(); }		/* skip usage position */
+        readFO();				/* skip parent */
+        if (readFO()==DoLambda) return True;
+        else return False;
+        break;
+    default:
+        return False;
+        break;
+  }
+}
 
 
 /* Get the an name of an identifier out of its struct Ident. */
@@ -1157,7 +1170,11 @@ int
 identDefnLine (Ident* id) { return id->defnline; }
 int
 identDefnCol  (Ident* id) { return id->defncolumn; }
-bool
+int
+identDefnLineEnd (Ident* id) { return id->defnlineend; }
+int
+identDefnColEnd  (Ident* id) { return id->defncolumnend; }
+Bool
 identIsTraced (Ident* id) { return id->isTraced; }
 
 
@@ -1189,6 +1206,7 @@ getExpArity (FileOffset fo)
         if (!hasFields(c)) { return 0; }
         readFO();				/* skip module */
         { int x; fread(&x,sizeof(int),1,HatFileRandom); }
+        { int x; fread(&x,sizeof(int),1,HatFileRandom); }
         { char x; fread(&x,sizeof(char),1,HatFileRandom); }
         { char arity; fread(&arity,sizeof(char),1,HatFileRandom);
           return (int)arity; }
@@ -1329,6 +1347,118 @@ getExpArg (FileOffset fo, int n)
   }
 }
 
+/* Get the n'th subexpression (*not* its value), 
+ * for any node that has subexpressions. 
+ */
+FileOffset
+peekExpArg (FileOffset fo, int n)
+{
+  char c;
+  int i=0;
+  FileOffset ptr;
+  HIDE(fprintf(stderr,"peekExpArg 0x%x\n",fo);)
+  assert (n>=0);
+  freadAt(fo,&c,sizeof(char),1,HatFileRandom);
+  switch (lower5(c)) {
+    case ExpApp:
+        if (hasSrcPos(c)) { readFO(); }		/* skip usage position */
+        readFO();				/* skip parent */
+        readFO();				/* skip result */
+        ptr = readFO();				/* fun/constructor */
+        if (n==0) return ptr;
+        fread(&c,sizeof(char),1,HatFileRandom);	/* get arity */
+        assert (n<=c);
+        for (i=1; i<n; i++) readFO();		/* skip other args */
+        ptr = readFO();		                /* get n'th arg */
+        return ptr;
+        break;
+    case ExpValueApp:
+        if (hasSrcPos(c)) { readFO(); }		/* skip usage position */
+        readFO();				/* skip parent */
+        ptr = readFO();				/* fun/constructor */
+        if (n==0) return ptr;
+        fread(&c,sizeof(char),1,HatFileRandom);	/* get arity */
+        assert (n<=c);
+        for (i=1; i<n; i++) readFO();		/* skip other args */
+        ptr = readFO();			        /* get n'th arg */
+        return ptr;
+        break;
+    case ExpValueUse:
+        if (hasSrcPos(c)) { readFO(); }		/* skip usage position */
+        readFO();				/* skip parent */
+        ptr = readFO();				/* only arg */
+        return ptr;	/* this is an atom */
+        break;
+    case ExpConstDef:
+    case ExpConstUse:
+        if (hasSrcPos(c)) { readFO(); }		/* skip usage position */
+        readFO();				/* skip parent */
+        ptr = readFO();				/* only arg */
+        return ptr;
+        break;
+    case ExpGuard:
+    case ExpCase:
+    case ExpIf:
+        if (hasSrcPos(c)) { readFO(); }		/* skip usage position */
+        readFO();				/* skip parent */
+        readFO();				/* skip result */
+        ptr = readFO();				/* get condition */
+        return ptr;
+        break;
+    case ExpFieldUpdate:
+        if (hasSrcPos(c)) { readFO(); }		/* skip usage position */
+        readFO();				/* skip parent */
+        readFO();				/* skip result */
+        ptr = readFO();				/* exp/constructor */
+        if (n==0) return ptr;
+        fread(&c,sizeof(char),1,HatFileRandom);	/* get arity */
+        assert (n<=c);
+        for (i=0; i<c; i++) readFO();		/* skip binder labels */
+        for (i=1; i<n; i++) readFO();		/* skip other bindees */
+        ptr = readFO();			        /* get n'th bindee */
+        return ptr;
+        break;
+    case ExpProjection:
+        if (hasSrcPos(c)) { readFO(); }		/* skip usage position */
+        readFO();				/* skip parent */
+        ptr = readFO();				/* get expr */
+        return ptr;
+        break;
+    case ExpForward:
+        ptr = readFO();				/* get expr */
+        return ptr;
+        break;
+    case ExpHidden:
+        readFO();				/* skip parent */
+        readFO();                               /* skip result */
+        ptr = readFO();			        /* get child list */
+        return ptr;
+        break;
+    case ListCons:
+        ptr = readFO();                         /* get element */
+        if (n==1) return ptr;
+        ptr = readFO();                         /* get tail */
+        return ptr;
+        break;
+    case ExpChar:
+    case ExpInt:
+    case ExpInteger:
+    case ExpRat:
+    case ExpRational:
+    case ExpFloat:
+    case ExpDouble:
+    case ExpDoStmt:
+    case Module:
+    case SrcPos:
+    case AtomVariable:
+    case AtomConstructor:
+    case AtomAbstract:
+    default:
+        assert (False);
+        break;
+  }
+}
+
 /* Only for an ExpFieldUpdate node, or an ExpConstructor that has fields,
  * get the atom pointer of the n'th label.
  */
@@ -1364,6 +1494,7 @@ getFieldLabel (FileOffset fo, int n)
         if (hasFields(c)) {
           readFO();				/* skip module ptr */
           { int x; fread(&x,sizeof(int),1,HatFileRandom); }
+          { int x; fread(&x,sizeof(int),1,HatFileRandom); }
           { char x; fread(&x,sizeof(char),1,HatFileRandom); }
           { char arity; fread(&arity,sizeof(char),1,HatFileRandom);
             readString();
@@ -1390,7 +1521,7 @@ getSrcRef (FileOffset fo)
   char c;
   int i;
   HIDE(fprintf(stderr,"getSrcRef 0x%x\n",fo);)
-  if (fo<=Lambda) return 0;
+  if (fo<=DoLambda) return 0;
   freadAt(fo,&c,sizeof(char),1,HatFileRandom);
   if (hasSrcPos(c)) {
     return readFO();			/* get SrcRef */
@@ -1399,47 +1530,6 @@ getSrcRef (FileOffset fo)
   }
 }
 
-/* Get information out of a SrcRef node. */
-char *
-getSrcRefFile (FileOffset fo)
-{
-  SrcRef *sr;
-  char *id=0;
-  if (fo==0) return "";
-  sr = readSRAt(fo);
-  if (sr) {
-    id = sr->srcname;
-    free(sr);
-  }
-  return id;
-}
-int
-srcRefLine (FileOffset fo)
-{
-  SrcRef *sr;
-  int line=0;
-  if (fo==0) return 0;
-  sr = readSRAt(fo);
-  if (sr) {
-    line = sr->line;
-    free(sr);
-  }
-  return line;
-}
-int
-srcRefCol (FileOffset fo)
-{
-  SrcRef *sr;
-  int col=0;
-  if (fo==0) return 0;
-  sr = readSRAt(fo);
-  if (sr) {
-    col = sr->column;
-    free(sr);
-  }
-  return col;
-}
-
 /* Get the Atom node for the fun position of an expression. 
  * If the node doesn't look right, we give back a 0 reference.
  */
@@ -1489,41 +1579,6 @@ getDefnRef (FileOffset fo)
   }
 }
 
-/* Get source position information out of an Atom node. */
-char *
-getDefnFile (FileOffset fo)
-{
-  Ident *i;
-  char *id;
-  if (fo==0) return "";
-  i  = readAtomAt(fo);
-  id = i->srcname;
-  free(i);
-  return id;
-}
-int
-defnLine (FileOffset fo)
-{
-  Ident *i;
-  int line;
-  if (fo==0) return 0;
-  i = readAtomAt(fo);
-  line = i->defnline;
-  free(i);
-  return line;
-}
-int
-defnCol (FileOffset fo)
-{
-  Ident *i;
-  int col;
-  if (fo==0) return 0;
-  i = readAtomAt(fo);
-  col = i->defncolumn;
-  free(i);
-  return col;
-}
-
 
 /* peekTrace() takes a peek backwards at a trace (or indirection), skipping
  * over any Hidden or Projection nodes to find the nearest "real" trace.
@@ -1536,7 +1591,7 @@ peekTrace(FileOffset fo)
   char c;
 
   while (1) {		/* iterate until we find a `real' node */
-    if (fo<=Lambda) return 0;	/* trace is Root/Unevaluated/Entered/etc */
+    if (fo<=DoLambda) return 0;	/* trace is Root/Unevaluated/Entered/etc */
     HIDE(fprintf(stderr,"peekTrace 0x%x\n",fo);)
     freadAt(fo,&c,sizeof(char),1,HatFileRandom);
     switch (c) {
@@ -1582,7 +1637,7 @@ peekTrace(FileOffset fo)
 static FileOffset mostRecentHidden=0;
 
 FileOffset
-getResult (FileOffset fo, bool stopAtHidden)
+getResult (FileOffset fo, Bool stopAtHidden)
 {
   if (fo==mostRecentHidden) return fo;
   mostRecentHidden=0;
@@ -1591,13 +1646,13 @@ getResult (FileOffset fo, bool stopAtHidden)
 
 #define fixInterrupt(fo)	(interrupt && fo==Entered ? Interrupted : fo)
 FileOffset
-getResultNoCycle (FileOffset fo, bool stopAtHidden)
+getResultNoCycle (FileOffset fo, Bool stopAtHidden)
 {
   char c;
   FileOffset result;
 
   HIDE(fprintf(stderr,"getResult 0x%x\n",fo);)
-  if (fo<=Lambda) return fixInterrupt(fo);      /* trace is Unevaluated etc */
+  if (fo<=DoLambda) return fixInterrupt(fo);      /* trace is Unevaluated etc */
   freadAt(fo,&c,sizeof(char),1,HatFileRandom);
   switch (lower5(c)) {
     case ExpApp:
@@ -1610,7 +1665,7 @@ getResultNoCycle (FileOffset fo, bool stopAtHidden)
         result = readFO();			/* get result */
         HIDE(fprintf(stderr,"getResult: result is 0x%x\n",result);)
         if (result==fo) return fo;
-        else if (result<=Lambda) return fixInterrupt(fo);
+        else if (result<=DoLambda) return fixInterrupt(fo);
         else return getResultNoCycle(result,False);
         break;
 //  case ExpValueApp:
@@ -1623,14 +1678,14 @@ getResultNoCycle (FileOffset fo, bool stopAtHidden)
         if (hasSrcPos(c)) { readFO(); }		/* skip use position */
         readFO();				/* skip parent */
         result = readFO();			/* get result */
-        if (result<=Lambda) return fixInterrupt(fo);
+        if (result<=DoLambda) return fixInterrupt(fo);
         else return getResultNoCycle(result,False);
 						/* follow ExpConstDef pointer */
         break;
     case ExpConstDef:
         readFO();				/* skip parent */
         result = readFO();			/* get result */
-        if (result<=Lambda) return fixInterrupt(fo);
+        if (result<=DoLambda) return fixInterrupt(fo);
         return getResultNoCycle(result,False);
         break;
     case ExpForward:
@@ -1664,7 +1719,7 @@ getResultNoCycle (FileOffset fo, bool stopAtHidden)
           readFO();				/* skip parent */
           result = readFO();			/* get result */
           if (result==fo) return fo;
-          else if (result<=Lambda) return fixInterrupt(fo);
+          else if (result<=DoLambda) return fixInterrupt(fo);
           else return getResultNoCycle(result,False);
         }
         break;
@@ -1689,7 +1744,7 @@ peekResult (FileOffset fo)
   FileOffset result;
 
   HIDE(fprintf(stderr,"peekResult 0x%x\n",fo);)
-  if (fo<=Lambda) return fixInterrupt(fo);      /* trace is Unevaluated etc */
+  if (fo<=DoLambda) return fixInterrupt(fo);      /* trace is Unevaluated etc */
   freadAt(fo,&c,sizeof(char),1,HatFileRandom);
   switch (lower5(c)) {
     case ExpApp:
@@ -1763,3 +1818,20 @@ void
 ctrlC (int sig)
 { hat_interrupted = True; }
 
+
+/* sequential access from Haskell */
+void
+q_init (void)
+{
+  q_position  = 0x10;
+  fseek(HatFileSeq,q_position,SEEK_SET);
+}
+char
+q_tag (void)
+{
+  char c;
+  c = (char)fgetc(HatFileSeq);
+  q_position+=1;
+  return lower5(c);
+}
+
