@@ -18,11 +18,14 @@
  *   Update to new file format.
  * 12 June 2002 - Colin
  *   Added -g option to generate dot-code for graph diagram.
+ * 4 August 2004 - Olaf
+ *   Update to extended position format (begin and end location).
  */
 
 /* #include <unistd.h> */
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
@@ -32,8 +35,8 @@
 /* out-of-range values must be <= 31 && not overlap Exps and Atoms */
 #define ANYEXP	22
 #define ANYATOM	23
-#define HEADER	29
-#define INVALID	30
+#define HEADER  24
+#define INVALID	30 
 #define BEYOND	31
 
 /* prototypes */
@@ -78,11 +81,11 @@ tag2str (int k)
   case AtomVariable:    return "AtomVariable"; break;
   case AtomConstructor: return "AtomConstructor"; break;
   case AtomAbstract:    return "AtomAbstract"; break;
+  case ListCons:        return "ListCons"; break;
 
   case ANYEXP:  return "Exp"; break;
   case ANYATOM: return "Atom"; break;
-  case HEADER:  return "HEADER"; break;
-  case INVALID: return "INVALID"; break;
+  case HEADER:  return "HEADER/INVALID"; break; /* same for INVALID */
   case BEYOND:  return "beyond end of file";
 
   default:              return "unknown/unused"; break;
@@ -98,6 +101,7 @@ ref2str (int k)
   case Entered:      return "Entered"; break;
   case Interrupted:  return "Interrupted"; break;
   case Lambda:       return "Lambda"; break;
+  case DoLambda:     return "DoLambda"; break;
   default:           return "Ref"; break;
   }
 }
@@ -188,10 +192,10 @@ main (int argc, char *argv[])
       fprintf(stderr,"   does not appear to be a Hat archive.  Quitting.\n");
       exit(1);
     }
-    if (strncmp(header+3,VERSION,4)) {
+    if (strncmp(header+3,FILEVERSION,4)) {
       fprintf(stderr,"hat-check (warning): file %s\n",filename);
       fprintf(stderr,"   appears to be a Hat archive in format %s\n",header+3);
-      fprintf(stderr,"   but this tool deals with format version %s\n",VERSION);
+      fprintf(stderr,"   but this tool deals with format version %s\n",FILEVERSION);
       fprintf(stderr,"   I'm continuing, but there may be unexpected errors.\n");
     }
     fseek(f,0,SEEK_SET);	/* reset to beginning of file */
@@ -290,7 +294,7 @@ reportstats (void)
   if (rmode) printf("%7.1f", pc(1,1));
   printf("%10u   %-20s%12u%10.1f\n",
     1, "header", headspace, pc(headspace,grandspace));
-  for (k=0; k<=AtomAbstract; (k==ExpDoStmt? k=AtomVariable: k++)) {
+  for (k=0; k<=ListCons; (k==ExpDoStmt? k=AtomVariable: k++)) {
     if (rmode) printf("%7.1f", pc(reachcount[k], count[k]));
     printf("%10u   %-20s%12u%10.1f\n",
       count[k], tag2str(k), space[k],
@@ -371,14 +375,16 @@ readfourbytes (void)
   return ntohl(slot);
 }
 
-#define POSNMAX 30
+#define POSNMAX 40
 char posnbuf[POSNMAX+1];
 
 char*
 readposn (void)
 {
-  unsigned long posn = readfourbytes();
-  sprintf(posnbuf, "line %u, column %u,", posn/10000, posn%10000);
+  unsigned long begin = readfourbytes();
+  unsigned long end = readfourbytes();
+  sprintf(posnbuf, "position %u:%u-%u:%u,", 
+          begin/10000, begin%10000, end/10000, end%10000);
   return posnbuf;
 }
 
@@ -522,6 +528,7 @@ dopointer (int okzero,
       case Entered:
       case Interrupted:
       case Lambda:
+      case DoLambda:
                 printf("(%s)",ref2str(requireoffset));
                 break;
       default:  printf("(%s 0x%x)", tag2str(requiretag), requireoffset);
@@ -587,7 +594,7 @@ nextnode (void)
   }
   {
     int k = lo5(b);
-    if ((ExpDoStmt<k && k<AtomVariable) || k>AtomAbstract) {
+    if ((ExpDoStmt<k && k<AtomVariable) || k>ListCons) {
       fprintf(stderr, "strange tag %d at byte offset 0x%x\n",
                       k, offset);
       exit(1);
@@ -596,6 +603,14 @@ nextnode (void)
       if (rmode && marked) reachcount[k]++;
     }
     switch (k) {
+    case ListCons:
+	if (gmode) printf("%d [label=\"0x%x ListCons\"]\n", offset, offset);
+        if (amode) printf("0x%x:  %-20s\t", offset, tag2str(k));
+	if (amode) printf("elem=");
+        dopointer(NONZERO, ANYEXP, readpointer(), k, offset, "e");
+	if (amode) printf(" tail=");
+        dopointer(MAYBEZERO, ListCons, readpointer(), k, offset, "t");
+      break;
     case Module:
       if (amode) {
         if (tracedModule(b))
@@ -829,6 +844,8 @@ nextnode (void)
         dopointer(MAYBEZERO, ANYEXP, readpointer(), k, offset, "p");
 	if (amode) printf(" result=");
         dopointer(MAYBEZERO, ANYEXP, readpointer(), k, offset, "r");
+	if (amode) printf(" children=");
+	dopointer(MAYBEZERO, ListCons, readpointer(), k, offset, "c");
 	break;
       case ExpForward:
 	if (gmode) printf("%d [label=\"0x%x ExpForward\"]\n", offset, offset);
@@ -907,12 +924,17 @@ markfrom (unsigned long root, unsigned long *buf)
      */
     {
       int k = lo5(tag);
-      if ((ExpDoStmt < k && k < AtomVariable) || k > AtomAbstract) {
+      if ((ExpDoStmt < k && k < AtomVariable) || k > ListCons) {
 	fprintf(stderr, "strange tag %d at 0x%x\n",
                 	k, root);
 	exit(1);
       }
       switch (k) {
+        case ListCons:
+          fread(buf,sizeof(unsigned long),2,f);	/* two pointers */
+          markfrom(getpointer(buf+1),buf+1);
+          markfrom(getpointer(buf),buf);
+	  break;
         case Module: break;
         case AtomAbstract: break;
 
@@ -952,11 +974,11 @@ markfrom (unsigned long root, unsigned long *buf)
 	      break;
             case ExpValueUse:
             case ExpConstUse:
-            case ExpHidden:
             case ExpProjection:
               fread(buf+pos,sizeof(unsigned long),1,f);	/* one extra pointer */
               pos++;
 	      break;
+            case ExpHidden:
             case ExpConstDef:
             case ExpFieldUpdate:	/* ignore fieldnames for now */
             case ExpGuard:
