@@ -5,7 +5,7 @@
 #include <stdio.h>
 #include <string.h>
 #include <signal.h>
-#include <glib.h>
+#include "finitemap.h"
 #include "art.h"
 #include "artutils.h"
 #include "observeutils.h"
@@ -19,11 +19,11 @@
 #endif
 
 /* Some finite maps (i.e. lookup tables):
- * mapAtom2Info :: FileNode -> Info
+ * mapAtom2Info    :: FileNode -> Info
  * mapExp2Atom     :: FileNode -> Atom
  * mapContext2Atom :: FileNode -> Atom
-*/
-GTree *mapAtom2Info, *mapExp2Atom, *mapContext2Atom;
+ */
+FiniteMap mapAtom2Info, mapExp2Atom, mapContext2Atom;
 
 /* Insert into mapAtom2Info */
 void
@@ -34,28 +34,34 @@ insert_mapAtom2Info (FileOffset atom, char* var, unsigned char arity)
   info->node  = atom;
   info->var   = var;
   info->arity = arity;
-  g_tree_insert(mapAtom2Info,(gpointer)atom,(gpointer)info);
+  FM_insert(mapAtom2Info,(cast)atom,(cast)info);
 }
 /* Insert into either mapExp2Atom or mapContext2Atom */
 void
-insert_map2 (GTree* map2, FileOffset exp, FileOffset atom, unsigned char arity)
+insert_map2 (FiniteMap map2, FileOffset exp, FileOffset atom
+            ,unsigned char arity)
 {
   Info *info = (Info*)0;
-  info = g_tree_lookup(mapAtom2Info,(gpointer)atom);
+  info = FM_lookup(mapAtom2Info,(cast)atom);
   if (info) {
     Atom *fun;
     fun = (Atom*)malloc(sizeof(Atom));
     fun->atom  = atom;
     fun->arity = info->arity - arity;
-    g_tree_insert(map2,(gpointer)exp,(gpointer)fun);
+    FM_insert(map2,(cast)exp,(cast)fun);
   }
 }
+/* free memory when no longer needed */
+void
+cleanup_info (FileOffset fo, Info* info) { free(info); }
+void
+cleanup_atom (FileOffset fo, Atom* atom) { free(atom); }
 
 
 /* Some global values for an observation search.  Only one search can
  * be active at once.
  */
-bool	o_srcpos, o_context, o_recursive;
+Bool	o_srcpos, o_context, o_recursive;
 char	*o_caller, *o_callee;
 char	*o_module;
 unsigned	o_linecol;
@@ -65,7 +71,7 @@ unsigned char	o_arity=0;	/* oversaturate/override arity */
 
 /* Initialise the global state ready for a new observation to start. */
 void
-setObserveContext (bool hascontext, bool rec, int arity, char* caller)
+setObserveContext (Bool hascontext, Bool rec, int arity, char* caller)
 {
 /*struct sigaction sig;*/
   o_srcpos    = False;
@@ -79,13 +85,16 @@ setObserveContext (bool hascontext, bool rec, int arity, char* caller)
 /*sig.sa_handler = ctrlC; sigemptyset(&sig.sa_mask); sig.sa_flags=0; */
 /*sigaction(SIGINT,&sig,(void*)0); */
   signal(SIGINT,ctrlC);
-  /* Reset the maps here, but have neglected to free the tree contents. */
-  if (mapAtom2Info)    g_tree_destroy(mapAtom2Info);
-  if (mapExp2Atom)     g_tree_destroy(mapExp2Atom);
-  if (mapContext2Atom) g_tree_destroy(mapContext2Atom);
-  mapAtom2Info    = g_tree_new((GCompareFunc)fileoffset_compare);
-  mapExp2Atom     = g_tree_new((GCompareFunc)fileoffset_compare);
-  mapContext2Atom = g_tree_new((GCompareFunc)fileoffset_compare);
+  /* Reset the maps here, freeing the tree contents also. */
+  if (mapAtom2Info)    FM_destroy(mapAtom2Info);
+  if (mapExp2Atom)     FM_destroy(mapExp2Atom);
+  if (mapContext2Atom) FM_destroy(mapContext2Atom);
+  mapAtom2Info    = FM_new((FMComparison)fileoffset_compare
+                          ,(FMFreeItem)cleanup_info);
+  mapExp2Atom     = FM_new((FMComparison)fileoffset_compare
+                          ,(FMFreeItem)cleanup_atom);
+  mapContext2Atom = FM_new((FMComparison)fileoffset_compare
+                          ,(FMFreeItem)cleanup_atom);
   HIDE(fprintf(stderr,"setObserveContext: filepos=0x%x\n",ftell(HatFileSeq));)
 }
 
@@ -176,9 +185,9 @@ varSearch (void)
           for (i=0; i<size; i++) q_readFO();    /* skip args */
           /* First check if this is a possible caller context */
           if (o_context) {
-            atom = (Atom*)g_tree_lookup(mapContext2Atom,(gpointer)fun);
+            atom = (Atom*)FM_lookup(mapContext2Atom,(cast)fun);
             if (atom) {
-              it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom->atom);
+              it = (Info*)FM_lookup(mapAtom2Info,(cast)atom->atom);
               if (it && !strcmp(it->var,o_caller)) {
                 insert_map2(mapContext2Atom,node,atom->atom,size);
                 HIDE(fprintf(stderr,"App of %s (context) ",it->var);)
@@ -186,24 +195,24 @@ varSearch (void)
             }
           }
           /* Then check if we have found the right callee */
-          atom = (Atom*)g_tree_lookup(mapExp2Atom,(gpointer)fun);
+          atom = (Atom*)FM_lookup(mapExp2Atom,(cast)fun);
           if (atom) {
-            it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom->atom);
+            it = (Info*)FM_lookup(mapAtom2Info,(cast)atom->atom);
             if (it && !strcmp(it->var,o_callee)) {
               HIDE(fprintf(stderr,"App of %s (callee)",it->var);)
               if (size >= atom->arity) {  /* is not undersaturated */
                 if (!o_recursive) {  /* if excluding recursive calls */
                   insert_map2(mapContext2Atom,node,atom->atom,size);
-                  atom = (Atom*)g_tree_lookup(mapContext2Atom,(gpointer)parent);
+                  atom = (Atom*)FM_lookup(mapContext2Atom,(cast)parent);
                   if (atom) {
-                    it =(Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom->atom);
+                    it =(Info*)FM_lookup(mapAtom2Info,(cast)atom->atom);
                     if (it && !strcmp(it->var,o_callee)) return 0;
                     else return node;
                   } else return node;
                 } else if (o_context) {	 /* if context matters */
-                  atom = (Atom*)g_tree_lookup(mapContext2Atom,(gpointer)parent);
+                  atom = (Atom*)FM_lookup(mapContext2Atom,(cast)parent);
                   if (atom) {
-                    it =(Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom->atom);
+                    it =(Info*)FM_lookup(mapAtom2Info,(cast)atom->atom);
                     HIDE(fprintf(stderr,"(context is %s)",it->var);)
                     if (it && !strcmp(it->var,o_caller)) return node;
                   }
@@ -220,7 +229,7 @@ varSearch (void)
           fun = q_readFO();     /* fun ptr is an Atom ref */
           q_fread(&size,sizeof(unsigned char),1,HatFileSeq);    /* get arity */
           for (i=0; i<size; i++) q_readFO();    /* skip args */
-          it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)fun);
+          it = (Info*)FM_lookup(mapAtom2Info,(cast)fun);
           if (it) {
             if (size >= it->arity) return node;
             else insert_map2(mapExp2Atom,node,fun,size);
@@ -230,6 +239,7 @@ varSearch (void)
     case AtomVariable:
         q_readFO();             /* skip module pointer */
         { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }   /* skip line/col */
+        { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }   /* skip line/col */
         { char x; q_fread(&x,sizeof(char),1,HatFileSeq); } /* skip fixity */
         { unsigned char arity, tmp;
           char *id;
@@ -249,12 +259,12 @@ varSearch (void)
         { FileOffset parent, atom; Info *it;
           parent = q_readFO();	/* get parent */
           atom = q_readFO();	/* get atom */
-          if (atom==Lambda) {
+          if ((atom==Lambda)||(atom==DoLambda)) {
             if (o_context || !o_recursive) {
               Atom *atom;	/* shadows outer scope */
-              atom = (Atom*)g_tree_lookup(mapContext2Atom,(gpointer)parent);
+              atom = (Atom*)FM_lookup(mapContext2Atom,(cast)parent);
               if (atom) {
-                it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom->atom);
+                it = (Info*)FM_lookup(mapAtom2Info,(cast)atom->atom);
                 if (it) {
                   if ( (!o_recursive && !strcmp(it->var,o_callee))
                      || (o_context && !strcmp(it->var,o_caller)) ) {
@@ -264,7 +274,7 @@ varSearch (void)
               }
             }
           } else {
-            it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom);
+            it = (Info*)FM_lookup(mapAtom2Info,(cast)atom);
             if (it) {
               if (!strcmp(it->var,o_callee)) {
                 insert_map2(mapExp2Atom,node,atom,0);
@@ -283,7 +293,7 @@ varSearch (void)
           q_readFO();		/* skip parent */
           result = q_readFO();	/* result might contain desired function */
           atom = q_readFO();	/* get atom */
-          it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom);
+          it = (Info*)FM_lookup(mapAtom2Info,(cast)atom);
           if (it) {
             HIDE(fprintf(stderr,"ConstDef %s",it->var);)
             if (!strcmp(it->var,o_callee))
@@ -302,16 +312,16 @@ varSearch (void)
         { FileOffset exp; Atom *def; Info *it;
           q_readFO();		/* skip parent */
           exp = q_readFO();	/* get ExpConstDef location */
-          def = (Atom*)g_tree_lookup(mapExp2Atom,(gpointer)exp);
+          def = (Atom*)FM_lookup(mapExp2Atom,(cast)exp);
           if (def) {
             insert_map2(mapExp2Atom,node,def->atom,def->arity);	// allows oversat apps
-            it = g_tree_lookup(mapAtom2Info,(gpointer)def->atom);
+            it = FM_lookup(mapAtom2Info,(cast)def->atom);
             HIDE(if (it) fprintf(stderr,"%s",it->var);)
             if (it && !strcmp(it->var,o_callee)) return node;
           } else {
-            def = (Atom*)g_tree_lookup(mapContext2Atom,(gpointer)exp);
+            def = (Atom*)FM_lookup(mapContext2Atom,(cast)exp);
             if (def) {
-              it = g_tree_lookup(mapAtom2Info,(gpointer)def->atom);
+              it = FM_lookup(mapAtom2Info,(cast)def->atom);
               if (it && !strcmp(it->var,o_caller))
                 insert_map2(mapContext2Atom,node,def->atom,def->arity);
             }
@@ -326,9 +336,9 @@ varSearch (void)
             parent = q_readFO();
             q_readFO();	/* skip result */
             q_readFO();	/* skip condition */
-            atom = (Atom*)g_tree_lookup(mapContext2Atom,(gpointer)parent);
+            atom = (Atom*)FM_lookup(mapContext2Atom,(cast)parent);
             if (atom) {
-              it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom->atom);
+              it = (Info*)FM_lookup(mapAtom2Info,(cast)atom->atom);
               if (it) {
                 if ( (!o_recursive && !strcmp(it->var,o_callee))
                    || (o_context && !strcmp(it->var,o_caller)) ) {
@@ -382,6 +392,7 @@ srcSearch (void)
             FileOffset mod; unsigned linecol;
             mod = q_readFO();
             q_fread(&linecol,sizeof(unsigned),1,HatFileSeq);
+            { int x; q_fread(&x,sizeof(int),1,HatFileSeq); }/* skip line/col */
             HIDE(fprintf(stderr,"Line/Col %d ",linecol);)
             if (mod==o_module_fo && linecol==o_linecol) {
               HIDE(fprintf(stderr,"(Got it!)");)
@@ -433,7 +444,7 @@ searchCAFResult (FileOffset caf, FileOffset value, unsigned char arity
   HIDE(fprintf(stderr
               ,"searchCAF: caf=0x%x, value=0x%x, arity=0x%x, hidden=0x%x\n"
               ,caf,value,arity,mostRecentHidden);)
-  if ((value<Lambda) || (value==caf)) return;
+  if ((value<DoLambda) || (value==caf)) return;
   if (value==mostRecentHidden) return;
   freadAt(value,&c,sizeof(unsigned char),1,HatFileRandom);
   switch (lower5(c)) {  /* lower 5 bits identify the TraceType */
@@ -448,17 +459,17 @@ searchCAFResult (FileOffset caf, FileOffset value, unsigned char arity
           fread(&size,sizeof(unsigned char),1,HatFileRandom);	/* arity */
           if (fun < caf) {	/* fun already seen in linear scan */
             if (o_context) {
-              atom = (Atom*)g_tree_lookup(mapContext2Atom,(gpointer)fun);
+              atom = (Atom*)FM_lookup(mapContext2Atom,(cast)fun);
               if (atom) {
-                it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom->atom);
+                it = (Info*)FM_lookup(mapAtom2Info,(cast)atom->atom);
                 if (it && !strcmp(it->var,o_caller)) {
                   insert_map2(mapContext2Atom,caf,atom->atom,size);
                 }
               }
             }
-            atom = (Atom*)g_tree_lookup(mapExp2Atom,(gpointer)fun);
+            atom = (Atom*)FM_lookup(mapExp2Atom,(cast)fun);
             if (atom) {
-              it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)atom->atom);
+              it = (Info*)FM_lookup(mapAtom2Info,(cast)atom->atom);
               if (it && !strcmp(it->var,o_callee)
                      && ((size+arity)<atom->arity)) {
                 HIDE(fprintf(stderr,"searchCAF: STORING caf=0x%x var=%s, size=%d\n",caf,it->var,size);)
@@ -478,9 +489,10 @@ searchCAFResult (FileOffset caf, FileOffset value, unsigned char arity
           readFO();		/* parent */
           var = readFO();	/* atom */
           if (var==Lambda) return;
+          if (var==DoLambda) return;
           if (var < caf) {
             HIDE(fprintf(stderr,"searchCAF: var=0x%x < caf=0x%x\n",var,caf);)
-            it = (Info*)g_tree_lookup(mapAtom2Info,(gpointer)var);
+            it = (Info*)FM_lookup(mapAtom2Info,(cast)var);
             HIDE(if (it) fprintf(stderr,"searchCAF: var=%s\n",it->var);)
             if (it && !strcmp(it->var,o_callee) && (arity<it->arity)) {
               HIDE(fprintf(stderr,"searchCAF: STORING caf=0x%x var=%s, size=%d\n",caf,it->var,arity);)
@@ -519,6 +531,7 @@ searchCAFResult (FileOffset caf, FileOffset value, unsigned char arity
     case AtomVariable:
         readFO();	/* skip module pointer */
         { int x; fread(&x,sizeof(int),1,HatFileRandom); }   /* skip line/col */
+        { int x; fread(&x,sizeof(int),1,HatFileRandom); }   /* skip line/col */
         { char x; fread(&x,sizeof(char),1,HatFileRandom); } /* skip fixity */
         { unsigned char size;
           char *id;
