@@ -11,7 +11,8 @@ import Maybe(isNothing,fromJust)
 import List(isPrefixOf,nub,minimumBy,elemIndex)
 
 import Syntax
-import TokenId (TokenId,tPrelude,visImport,t_Tuple,getUnqualified,t_Arrow)
+import TokenId (TokenId(..),tPrelude,visImport,t_Tuple,getUnqualified,t_Arrow)
+import SysDeps(PackedString, unpackPS, packString)
 import AssocTree
 import OsOnly
 import Flags
@@ -26,17 +27,17 @@ import AssocTree
 -- parse tree.  The .hx file mentions all exported identifiers, both
 -- those defined in this module, and those reexported from imports.
 
-toAuxFile :: Flags -> FilePath -> Module TokenId -> IO ()
-toAuxFile flags aux
+toAuxFile :: Environment -> Flags -> FilePath -> Module TokenId -> IO ()
+toAuxFile finalEnv flags aux
           (Module pos modid exports imports fixdecls (DeclsParse decls)) =
   do
     let (identMap,definedTypesAndClasses) = mkIdentMap decls
     let (_,definedExported) = 
-          extendEnv (initAuxInfo (not (sDbgTrusted flags))) 
+          extendEnv finalEnv (initAuxInfo (not (sDbgTrusted flags))) 
             (visibleIn exports unspecYes modid)
             (initAT,identMap)
 	    (map DeclFixity fixdecls ++ decls)
-    (fullInfo,fullIdentMap) <- getImports (visibleIn exports unspecNo)
+    ((fullInfo,fullIdentMap),_) <- getImports (visibleIn exports unspecNo)
 					  definedExported
                                           flags imports
     writeFile aux ((showString "module " . shows modid . showChar '\n' .
@@ -63,21 +64,46 @@ toAuxFile flags aux
 -- import and which to ignore.  
 
 getImports :: (TokenId->Visibility) -> Environment
-		 -> Flags -> [ImpDecl TokenId] -> IO Environment
+	   -> Flags -> [ImpDecl TokenId] -> IO (Environment,[Environment])
 getImports reexport (alreadyGot,identMap) flags impdecls = do
-    let importFiles = filter normalImport (map impData impdecls)
+    let importFiles = map impData impdecls
+        importHiddenEntities = map getHidden impdecls
     auxInfos <- mapM getAuxFile importFiles
     let allInfo = zip importFiles auxInfos
-    return ( foldr extendImportEnv alreadyGot allInfo
-           , foldr extendIdentMap identMap allInfo )
+    return ((foldr extendImportEnv alreadyGot allInfo
+            ,foldr extendIdentMap identMap allInfo )
+           ,zipWith makeEnv importHiddenEntities auxInfos)
   where
-    normalImport (modid,_) = 
+    getHidden (Import _ (Hiding entities)) = entities
+    getHidden (ImportQ _ (Hiding entities)) = entities
+    getHidden (ImportQas _ _ (Hiding entities)) = entities
+    getHidden (Importas _ _ (Hiding entities)) = entities
+    getHidden _ = []
+
+    makeEnv :: [Entity TokenId] -> [(Identifier,AuxiliaryInfo)] -> Environment
+    makeEnv entities auxInfos =
+      foldr addEnv (initAT,initAT) auxInfos
+      where
+      addEnv :: (Identifier,AuxiliaryInfo) -> Environment -> Environment
+      addEnv (identifier,auxInfo) (auxTree,idMap) =
+        if identifier `isAmong` entities
+          then (if notTyCls identifier
+                  then (addAT auxTree const identifier auxInfo
+                       ,addAT idMap const (subTid identifier) identifier) 
+                  else (addTyCls (\x->True) auxTree identifier auxInfo,idMap)) 
+          else (auxTree,idMap)
+
+    normalImport modid = 
       not (sPrelude flags && "TraceOrig" `isPrefixOf` getUnqualified modid)
+      
     getAuxFile :: (TokenId,Visibility) -> IO [(Identifier,AuxiliaryInfo)]
-    getAuxFile (modid,importVisible) = do
+    getAuxFile (modid,importVisible) = 
+      if normalImport modid 
+        then do
         (_,f) <- readFirst (fixImportNames (sUnix flags) "hx" (show modid)
                                            (sIncludes flags ++ sPreludes flags))
 	(return . map (myRead (show modid)) . tail . lines) f
+        else return []
 
     extendImportEnv ((modid,importVisible), auxInfos) got =
         foldr (\(k,v) t-> if notGot k t && importVisible k && reexport modid k
@@ -108,6 +134,11 @@ getImports reexport (alreadyGot,identMap) flags impdecls = do
 
     notGot k t = case lookupAT t k of { Nothing -> True; Just _ -> False }
 
+    getModule (Import (_,modid) is) = modid
+    getModule (ImportQ (_,modid) is) = modid 
+    getModule (ImportQas (_,modid) _ is) = modid
+    getModule (Importas (_,modid) _ is)  = modid
+
     impData (Import (_,modid) is)      = (modid, impSpec is)
     impData (ImportQ (_,modid) is)     = (modid, impSpec is)
     impData (ImportQas (_,modid) _ is) = (modid, impSpec is)
@@ -218,25 +249,30 @@ unspecNo  = \_->False
 -- resolve fixity conflicts and label it with arity information, the local
 -- environment of identifiers is built with all identifiers in scope visible.
 
-extendEnv :: InitAuxInfo -> Visibility -> Environment -> [Decl TokenId] 
-          -> ([TokenId],Environment)
-extendEnv iai visible (init,identMap) decls = (irrefutableIds,(env,identMap))
+extendEnv :: Environment -> InitAuxInfo -> Visibility -> Environment 
+          -> [Decl TokenId] -> ([TokenId],Environment)
+extendEnv finalEnv iai visible (init,identMap) decls = 
+  (irrefutableIds,(env,identMap))
   where
-  (irrefutableIds,env) = foldrPat (auxInfo iai visible identMap) init decls
+  (irrefutableIds,env) = 
+     foldrPat (auxInfo finalEnv iai visible identMap) init decls
+                -- pass final environment as argument
 
 
 -- auxInfo is the main gatherer of information about declarations.
 -- It decides what AuxiliaryInfo to attach to each Identifier in
 -- the environment.
-auxInfo :: InitAuxInfo -> Visibility -> IdentMap -> Decl TokenId -> AuxTree 
+-- It does not recursively descend to local declarations.
+auxInfo :: Environment 
+        -> InitAuxInfo -> Visibility -> IdentMap -> Decl TokenId -> AuxTree 
         -> ([TokenId],AuxTree)
 -- Add varid/varop identifier, with arity.
-auxInfo iai visible identMap (DeclFun _ f clauses) t
+auxInfo env iai visible identMap (DeclFun _ f clauses) t
     | visible key  = ([],addAT t replaceArity key ((emptyAux iai){args = a}))
     where a   = let (Fun pats rhs local) = head clauses in length pats
           key = Var (show f)
 -- Add varop identifier declared in infix equation, with arity.
-auxInfo iai visible identMap 
+auxInfo env iai visible identMap 
   (DeclPat (Alt pat@(ExpInfixList _ es) rhs local)) t
     | len >= 3  =
 	let (_:defn:_) = es in
@@ -249,18 +285,18 @@ auxInfo iai visible identMap
                 addPat iai Irrefutable visible pat (t,identMap)
     where len = length es
 -- Add varid identifiers declared in a pattern binding.
-auxInfo iai visible identMap (DeclPat (Alt pat rhs local)) t =
+auxInfo env iai visible identMap (DeclPat (Alt pat rhs local)) t =
     (\(i,(e,m)) -> ([],e)) $ addPat iai Irrefutable visible pat (t,identMap)
 -- Add varid identifier declared as a primitive, with arity.
-auxInfo iai visible identMap (DeclPrimitive _ f a _) t
+auxInfo env iai visible identMap (DeclPrimitive _ f a _) t
     | visible key  = ([],addAT t replaceArity key ((emptyAux iai){args = a}))
 						where key = Var (show f)
 -- Add varid identifier declared as a foreign import, with arity.
-auxInfo iai visible identMap (DeclForeignImp _ _ _ f a _ _ _) t
+auxInfo env iai visible identMap (DeclForeignImp _ _ _ f a _ _ _) t
     | visible key  = ([],addAT t replaceArity key ((emptyAux iai){args = a}))
 						where key = Var (show f)
 -- Add type, constructor, with arity, and any associated field names.
-auxInfo iai visible identMap (DeclData sort _ (Simple _ typ _) tycons _) t =
+auxInfo env iai visible identMap (DeclData sort _ (Simple _ typ _) tycons _) t =
   ([],addAT (foldr doCon t tycons) const 
         (TypeClass sTyp) (TyCls (Ty constructors fieldlabels)))
   where
@@ -285,11 +321,11 @@ auxInfo iai visible identMap (DeclData sort _ (Simple _ typ _) tycons _) t =
 	| otherwise = t
 	where key = Field (show typ) (show f)
 -- Add type synonym
-auxInfo _ visible identMap (DeclType (Simple _ ty args) body) t =
+auxInfo env _ visible identMap (DeclType (Simple _ ty args) body) t =
   ([],addAT t const (TypeClass (show ty)) 
-       (TyCls (splitSynonym (map snd args) body)))
+       (TyCls (splitSynonym env (map snd args) body)))
 -- Add class and class method identifier, arity is always -1.
-auxInfo iai visible identMap (DeclClass _ _ cls _ _ (DeclsParse decls)) t =
+auxInfo env iai visible identMap (DeclClass _ _ cls _ _ (DeclsParse decls)) t =
   ([],addAT (foldr doMethod t decls) const
         (TypeClass sCls) (TyCls (Cls methods)))
   where
@@ -305,10 +341,10 @@ auxInfo iai visible identMap (DeclClass _ _ cls _ _ (DeclsParse decls)) t =
 		     | otherwise = t
 		     where key = Method (show cls) (show meth)
 -- Add normal infix decl for identifier.
-auxInfo iai visible identMap (DeclFixity f) t = 
+auxInfo env iai visible identMap (DeclFixity f) t = 
   ([],fixInfo iai visible identMap f t)
 -- No other form of decl matters.
-auxInfo _ visible identMap _ t = ([],t)
+auxInfo env _ visible identMap _ t = ([],t)
 --auxInfo visible identMap (DeclTypeRenamed _ _) t =
 --auxInfo visible identMap (DeclDataPrim _ _ _) t =
 --auxInfo visible identMap (DeclConstrs _ _ _) t =
@@ -458,8 +494,11 @@ addPat iai patSort visible pat (env,identMap) =
 
 -- determines the outer functional or applicative part of a type synonym body
 -- this part can then be expanded when transforming types for workers
-splitSynonym :: [TokenId] -> Type TokenId -> TyCls
-splitSynonym tyVars rhs = 
+-- the *final* environment is needed, because in the body of a type synonym
+-- a type synonym may appear; because type synonyms shall not be recursive,
+-- a blackhole cannot occur for type correct programs.
+splitSynonym :: Environment -> [TokenId] -> Type TokenId -> TyCls
+splitSynonym env tyVars rhs = 
   case go rhs of
     Syn 1 THelper -> Syn 0 THelper -- nothing to split off (bogus THelper)
     syn -> syn
@@ -467,15 +506,54 @@ splitSynonym tyVars rhs =
   -- it is vital that this `go' agrees with the `go' in `splitSynonym' in
   -- TraceTrans.
   go :: Type TokenId -> TyCls
-  go (TypeCons _ tyCon tys) | getUnqualified tyCon == "->" = 
-    case tys of
-      [] -> Syn 0 TFun
-      [ty] -> Syn 1 (TApp TFun THelper)
-      [ty1,ty2] -> let Syn h ty2' = go ty2 
-                   in Syn (h+1) (TApp (TApp TFun THelper) ty2')
+  go (TypeCons _ tyCon tys) 
+    | getUnqualified tyCon == "->" = 
+      case tys of
+        [] -> Syn 0 TFun
+        [ty] -> Syn 1 (TApp TFun THelper)
+        [ty1,ty2] -> let Syn h ty2' = go ty2 
+                     in Syn (h+1) (TApp (TApp TFun THelper) ty2')
+    | isExpandableTypeSynonym info = go (expandTypeSynonym info tyCon tys)
+    where
+    info = lookupTyCls env tyCon
   go (TypeApp ty1 ty2) = Syn (h1+h2) (TApp ty1' ty2')
     where
     Syn h1 ty1' = go ty1
     Syn h2 ty2' = go ty2
   go (TypeVar _ tyVar) = Syn 0 (TVar (fromJust (elemIndex tyVar tyVars)))
-  go _ = Syn 1 THelper
\ No newline at end of file
+  go _ = Syn 1 THelper
+
+
+isExpandableTypeSynonym :: Maybe AuxiliaryInfo -> Bool
+isExpandableTypeSynonym (Just (TyCls (Syn n _))) = n > 0
+isExpandableTypeSynonym _ = False
+
+expandTypeSynonym :: Maybe AuxiliaryInfo 
+                  -> TokenId -> [Type TokenId] -> Type TokenId
+expandTypeSynonym (Just (TyCls (Syn n body))) tySyn tys = fst (go body 1)
+  where
+  go :: TySynBody -> Int -> (Type TokenId,Int)
+  go THelper n = 
+    (TypeCons noPos (nameTransTySynHelper tySyn n) tys
+    ,n+1)
+  go (TVar v) n = (tys!!v,n)
+  go TFun n = (TypeCons noPos t_Arrow [],n)
+  go (TApp ty1 ty2) n = (TypeApp ty1' ty2',n2)
+    where
+    (ty1',n1) = go ty1 n
+    (ty2',n2) = go ty2 n1
+
+nameTransTySynHelper :: TokenId -> Int -> TokenId
+nameTransTySynHelper tySyn no = updateToken tySyn (++ ("___" ++ show no))
+
+updateToken :: TokenId -> (String -> String) -> TokenId
+updateToken (Visible n) f = 
+  Visible (packString . reverse . f . unqual $ n) 
+updateToken (Qualified m n) f =
+  Qualified m (packString . reverse . f . unqual $ n)
+ 
+unqual :: PackedString -> String
+unqual = reverse . unpackPS
+
+lookupTyCls :: Environment -> TokenId -> Maybe AuxiliaryInfo
+lookupTyCls (auxTree,_) id = lookupAT auxTree (TypeClass (show id))
