@@ -6,7 +6,8 @@ module AuxLabelAST
 import List (nubBy)
 import Char (isUpper)
 import Maybe (isJust)
-import AuxTypes(Environment,mkIdentMap,useIdentMap,Identifier(..)
+import AuxTypes(Environment,mkIdentMap,useIdentMap,useEnvironment
+               ,Identifier(..)
                ,AuxiliaryInfo(letBound),InitAuxInfo,initAuxInfo,defaultAuxInfo)
 import AuxFile(Visibility,PatSort(Refutable,Irrefutable)
               ,extendEnv,getImports,addPat,foldrPat)
@@ -26,18 +27,21 @@ import Extra (Pos,noPos)
 -- also need to resolve the fixity of all applications (originally
 -- done in a later phase of the compiler).
 
-auxLabelSyntaxTree :: Flags -> Module TokenId -> IO (Module TraceId)
+auxLabelSyntaxTree :: Flags -> Module TokenId 
+                   -> IO (Environment,Module TraceId)
 auxLabelSyntaxTree flags
 	mod@(Module p modId exports imports fixdecls (DeclsParse decls)) =
   do
     let (identMap,_) = mkIdentMap decls
-    let (irrefutableIds,localenv) = 
-          extendEnv iai vi (initAT,identMap) (map DeclFixity fixdecls ++ decls)
-    totalenv <- getImports (\_->vi) localenv flags imports
+    (importenv,hiddenEnvs) <- 
+      getImports (\_->vi) (initAT,identMap) flags imports
+    let (irrefutableIds,totalenv) = 
+          extendEnv totalenv iai vi importenv 
+            (map DeclFixity fixdecls ++ decls)
     let Module p modId exports imports fixdecls decls = 
-          relabel iai totalenv mod
-    return $ Module p modId exports imports fixdecls 
-               (addVars iai totalenv irrefutableIds decls)
+          relabelModule iai totalenv importenv hiddenEnvs mod
+    return $ (totalenv,Module p modId exports imports fixdecls 
+                        (addVars iai totalenv irrefutableIds decls))
   where
   iai = initAuxInfo . not . sDbgTrusted $ flags
 
@@ -55,22 +59,6 @@ vi = \_-> True
 -- is decided by the caller, but for `lookEnv', it is decided by the
 -- environment.
 
--- The following hack is needed, because identMap only knows about global
--- identifiers and because variables, field labels and methods share the
--- same name space.
-useEnvironment :: Environment -> TokenId -> Maybe AuxiliaryInfo
-useEnvironment (env,identMap) id =
-  let v = useIdentMap identMap id in
-  case v of
-    -- a field or method may be shadowed by a normal variable
-    -- hence have to search for such a variable first
-    Field _ name -> case lookupAT env (Var name) of
-                      Just info -> Just info
-                      Nothing -> lookupAT env v
-    Method _ name -> case lookupAT env (Var name) of
-                       Just info -> Just info
-                       Nothing -> lookupAT env v
-    _ -> lookupAT env v
 
 letVar :: Environment -> TokenId -> TraceId
 letVar env id =
@@ -105,13 +93,18 @@ lookupTyCls (auxTree,_) id =
 class Relabel g where
   relabel :: InitAuxInfo -> Environment -> g TokenId -> g TraceId
 
-instance Relabel Module where
-  relabel iai env (Module p mod Nothing imps fixs decls) =
-    Module p (mkModule iai mod) Nothing (map (relabel iai env) imps) []
+relabelModule :: InitAuxInfo -> Environment -> Environment -> [Environment] 
+              -> Module TokenId -> Module TraceId
+relabelModule iai env impEnv hiddenEnvs 
+  (Module p mod Nothing imps fixs decls) =
+    Module p (mkModule iai mod) Nothing 
+      (zipWith (relabelImports iai impEnv) hiddenEnvs imps) []
       (relabel iai env decls)
-  relabel iai env (Module p mod (Just exps) imps fixs decls) =
+relabelModule iai env impEnv hiddenEnvs 
+  (Module p mod (Just exps) imps fixs decls) =
     Module p (mkModule iai mod) (Just (map (relabel iai env) exps))
-			(map (relabel iai env) imps) [] (relabel iai env decls)
+      (zipWith (relabelImports iai impEnv) hiddenEnvs imps) [] 
+      (relabel iai env decls)
 					-- fixdecls are folded into the decls
 
 instance Relabel Export where
@@ -120,23 +113,22 @@ instance Relabel Export where
   relabel iai env (ExportModid pos id)      = 
     ExportModid pos (mkModule iai id)
 
-instance Relabel ImpDecl where
-  relabel iai env (Import (pos,id) impspec) =
-    Import (pos,mkModule iai id) (relabel iai env impspec)
-  relabel iai env (ImportQ (pos,id) impspec) =
-    ImportQ (pos,mkModule iai id) (relabel iai env impspec)
-  relabel iai env (ImportQas (p1,id1) (p2,id2) impspec) =
-    ImportQas (p1,mkModule iai id1) (p2,mkModule iai id2) 
-      (relabel iai env impspec)
-  relabel iai env (Importas (p1,id1) (p2,id2) impspec) =
-    Importas (p1,mkModule iai id1) (p2,mkModule iai id2) 
-      (relabel iai env impspec)
 
-instance Relabel ImpSpec where
-  relabel iai env (NoHiding entities) = 
-    NoHiding (map (relabel iai env) entities)
-  relabel iai env (Hiding entities)   = 
-    Hiding (map (relabel iai env) entities)
+relabelImports iai env hiddenEnv (Import (pos,id) impspec) =
+  Import (pos,mkModule iai id) (relabelImpSpec iai env hiddenEnv impspec)
+relabelImports iai env hiddenEnv (ImportQ (pos,id) impspec) =
+  ImportQ (pos,mkModule iai id) (relabelImpSpec iai env hiddenEnv impspec)
+relabelImports iai env hiddenEnv (ImportQas (p1,id1) (p2,id2) impspec) =
+  ImportQas (p1,mkModule iai id1) (p2,mkModule iai id2) 
+    (relabelImpSpec iai env hiddenEnv impspec)
+relabelImports iai env hiddenEnv (Importas (p1,id1) (p2,id2) impspec) =
+  Importas (p1,mkModule iai id1) (p2,mkModule iai id2) 
+    (relabelImpSpec iai env hiddenEnv impspec)
+
+relabelImpSpec iai env hiddenEnv (NoHiding entities) = 
+  NoHiding (map (relabel iai env) entities)
+relabelImpSpec iai env hiddenEnv (Hiding entities)   = 
+  Hiding (map (relabel iai hiddenEnv) entities)
 
 instance Relabel Entity where
   relabel _ env (EntityVar p id) = 
@@ -203,7 +195,7 @@ instance Relabel Decl where
     infixFunExps = infixFun exps
     Just (e1,pos,fun,e2) = infixFunExps
   relabel iai env (DeclPat (Alt pat rhs ds@(DeclsParse decls))) =
-    let (irrefutableDeclIds,newEnv) = extendEnv iai vi env decls
+    let (irrefutableDeclIds,newEnv) = extendEnv undefined iai vi env decls
     in  DeclPat 
           (Alt (relabel iai newEnv pat) (relabel iai newEnv rhs) 
             (addVars iai newEnv (nubBy (\x y -> show x == show y) -- for scope 
@@ -302,7 +294,7 @@ instance Relabel FunDep where
 
 instance Relabel Fun where
   relabel iai env (Fun pats rhs ds@(DeclsParse decls)) =
-    let (irrefutableDeclIds,declEnv) = extendEnv iai vi env decls
+    let (irrefutableDeclIds,declEnv) = extendEnv undefined iai vi env decls
         (irrefutableIds,newEnv) = 
           foldrPat (addPat iai Refutable vi) declEnv pats 
     in
@@ -315,7 +307,7 @@ instance Relabel Fun where
 instance Relabel Alt where
   -- only used for Alts in cases, not in pattern bindings
   relabel iai env (Alt pat rhs ds@(DeclsParse decls)) =
-    let (irrefutableDeclIds,declEnv) = extendEnv iai vi env decls
+    let (irrefutableDeclIds,declEnv) = extendEnv undefined iai vi env decls
         (irrefutableIds,newEnv) = 
           addPat iai Refutable vi pat declEnv
     in
@@ -338,7 +330,7 @@ instance Relabel Exp where
              then id else ExpLet p (addVars iai newEnv irrefutableIds noDecls))
             (relabel iai newEnv exp))
   relabel iai env (ExpLet p ds@(DeclsParse decls) exp) =
-	let (irrefutableIds,newEnv) = extendEnv iai vi env decls in
+	let (irrefutableIds,newEnv) = extendEnv undefined iai vi env decls in
 	ExpLet p (addVars iai newEnv irrefutableIds (relabel iai newEnv ds)) 
           (relabel iai newEnv exp)
   relabel iai env (ExpDo p stmts) =
@@ -354,7 +346,8 @@ instance Relabel Exp where
                    else (StmtLet (addVars iai newEnv irrefutableIds noDecls):))
                   (doStmts newEnv ss)
 	  doStmts env (s@(StmtLet (DeclsParse decls)):ss) =
-		let (irrefutableIds,newEnv) = extendEnv iai vi env decls 
+		let (irrefutableIds,newEnv) = 
+                      extendEnv undefined iai vi env decls 
                     StmtLet reDecls = relabel iai newEnv s
                 in (StmtLet (addVars iai newEnv irrefutableIds reDecls) 
                    : doStmts newEnv ss)
